@@ -170,7 +170,7 @@ std::vector<std::string> WBComponentBasic::get_command_dropdown_items(const std:
     std::string colors= _wb->get_wb_options().get_string("workbench.model.Figure:ColorList");
     std::vector<std::string> colorList;
 
-    colorList= split_string(colors, "\n");
+    colorList= base::split(colors, "\n");
 
     if (!colorList.empty())
     {
