@@ -107,7 +107,9 @@ ModelDiagramForm::ModelDiagramForm(WBComponent *owner, const model_DiagramRef &v
 
 ModelDiagramForm::~ModelDiagramForm()
 {
+  sigc::connection(_signal_id).disconnect();
   delete _mini_view;
+  delete _features;
 }
 
 void ModelDiagramForm::model_list_changed(grt::internal::OwnedList *list, bool added, const grt::ValueRef &value)
@@ -402,6 +404,7 @@ mdc::CanvasItem *ModelDiagramForm::get_leaf_item_at(const mdc::Point &pos)
   return _view->get_leaf_item_at(pos);
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void ModelDiagramForm::handle_mouse_button(mdc::MouseButton button, bool press, int x, int y, mdc::EventState state)
 {
@@ -485,6 +488,21 @@ void ModelDiagramForm::handle_mouse_button(mdc::MouseButton button, bool press,
   _view->handle_mouse_button(button, press, x, y, state);
 }
 
+//--------------------------------------------------------------------------------------------------
+
+void ModelDiagramForm::handle_mouse_double_click(mdc::MouseButton button, int x, int y, mdc::EventState state)
+{
+  stop_editing();
+  
+  mdc::Point pos(_view->window_to_canvas(x, y));
+  
+  if (button != mdc::ButtonLeft && button != mdc::ButtonMiddle)
+    return;
+
+  _view->handle_mouse_double_click(button, x, y, state);
+}
+
+//--------------------------------------------------------------------------------------------------
 
 void ModelDiagramForm::handle_mouse_move(int x, int y, mdc::EventState state)
 {
@@ -570,7 +588,7 @@ void ModelDiagramForm::set_tool(const std::string &tool)
 
   _tool= tool;
 
-  WBComponent *compo= _owner->get_wb()->get_component_named(split_string(tool, "/")[0]);
+  WBComponent *compo= _owner->get_wb()->get_component_named(base::split(tool, "/")[0]);
   if (compo)
     compo->setup_canvas_tool(this, tool);
   else
@@ -1437,3 +1455,14 @@ std::string ModelDiagramForm::get_title()
 
 //--------------------------------------------------------------------------------------------------
 
+/**
+ * Called by the context model when it connects itself to a diagram (to listen for changes).
+ * For this task it binds this form to the callback, which requires that we disconnect that
+ * when this form is destroyed.
+ */
+void wb::ModelDiagramForm::change_signal_id(ChangeSignal::iterator signal_id)
+{
+  _signal_id = signal_id;
+}
+
+//--------------------------------------------------------------------------------------------------
