@@ -1,20 +1,20 @@
 /* 
- * (c) 2009-2010 Sun Microsystems, Inc.
+ * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
 #include "stdafx.h"
@@ -304,6 +304,13 @@ static void call_new_file(wb::WBContextSQLIDE *sqlide)
     editor->new_sql_script_file();
 }
 
+static void call_new_scratch(wb::WBContextSQLIDE *sqlide)
+{
+  Db_sql_editor *editor= sqlide->get_active_sql_editor();
+  if (editor)
+    editor->new_sql_scratch_area();
+}
+
 static bool validate_new_file(wb::WBContextSQLIDE *sqlide)
 {
   return (NULL != sqlide->get_active_sql_editor());
@@ -322,6 +329,13 @@ static bool validate_open_file(wb::WBContextSQLIDE *sqlide)
   return (NULL != sqlide->get_active_sql_editor());
 }
 
+static void call_run_file(wb::WBContextSQLIDE *sqlide)
+{
+  std::string path= sqlide->get_wbui()->get_wb()->show_file_dialog("open", _("Execute SQL Script"), "SQL Files (*.sql)|*.sql");
+  if (!path.empty())
+    sqlide->run_file(path);
+}
+
 static void call_save_file(wb::WBContextSQLIDE *sqlide)
 {
   Db_sql_editor *editor = sqlide->get_active_sql_editor();
@@ -589,12 +603,16 @@ void WBContextSQLIDE::init()
 
   _wbui->get_command_ui()->add_builtin_command("query.newFile", sigc::bind(sigc::ptr_fun(call_new_file), this), 
                                                sigc::bind(sigc::ptr_fun(validate_new_file), this));
+  _wbui->get_command_ui()->add_builtin_command("query.newScratch", sigc::bind(sigc::ptr_fun(call_new_scratch), this), 
+                                               sigc::bind(sigc::ptr_fun(validate_new_file), this));
   _wbui->get_command_ui()->add_builtin_command("query.openFile", sigc::bind(sigc::ptr_fun(call_open_file), this), 
                                                sigc::bind(sigc::ptr_fun(validate_open_file), this));
   _wbui->get_command_ui()->add_builtin_command("query.saveFile", sigc::bind(sigc::ptr_fun(call_save_file), this), 
                                                sigc::bind(sigc::ptr_fun(validate_save_file), this));
   _wbui->get_command_ui()->add_builtin_command("query.saveFileAs", sigc::bind(sigc::ptr_fun(call_save_file_as), this), 
                                                sigc::bind(sigc::ptr_fun(validate_save_file), this));
+  _wbui->get_command_ui()->add_builtin_command("query.runFile", sigc::bind(sigc::ptr_fun(call_run_file), this), 
+                                               sigc::bind(sigc::ptr_fun(validate_open_file), this));
 
   _wbui->get_command_ui()->add_builtin_command("query.export", sigc::bind(sigc::ptr_fun(call_export), this), 
                                               sigc::bind(sigc::ptr_fun(validate_export), this));
@@ -641,8 +659,12 @@ grt::ListRef<app_ToolbarItem> WBContextSQLIDE::get_toolbar_items(const std::stri
   WBContext *wb= _wbui->get_wb();
  
   if (name == "main")
-    return app_ToolbarRef::cast_from(wb->get_grt()->unserialize(make_path(wb->get_datadir(), "data/dbquery_toolbar.xml")))->items();
-  
+  {
+    if (!_toolbar.is_valid())
+      _toolbar = app_ToolbarRef::cast_from(wb->get_grt()->unserialize(make_path(wb->get_datadir(), "data/dbquery_toolbar.xml")));
+
+    return _toolbar->items();
+  }
   return grt::ListRef<app_ToolbarItem>();
 }
 
@@ -728,6 +750,19 @@ void WBContextSQLIDE::open_document(const std::string &path)
                                   _("OK"));
 }
 
+
+void WBContextSQLIDE::run_file(const std::string &path)
+{
+  Db_sql_editor *editor= get_active_sql_editor();
+  if (editor)
+    editor->open_sql_script_file(path, true, true);
+  else
+    mforms::Utilities::show_error(_("Execute SQL Script"),
+                                  _("Please select a connected SQL Editor tab to run a script file."),
+                                  _("OK"));
+}
+
+
 static bool compare(Db_sql_editor::Ptr ptr, Db_sql_editor *editor)
 {
   return ptr.lock().get() == editor;
@@ -747,6 +782,10 @@ void WBContextSQLIDE::editor_will_close(Db_sql_editor* editor)
     {
       if (dynamic_cast<db_query_EditorConcreteImplData*>(editors[i]->get_data())->editor_object().get() == editor)
       {
+        db_query_EditorRef editor(editors[i]);
+        
+        editor->reset_references();
+        
         editors.remove(i);
         break;
       }
