@@ -267,57 +267,60 @@ void WorkbenchCentral::repaint(cairo_t* cr, int areax, int areay, int areaw, int
       double next_offset= get_next_offset(iterator, (iterator + 1) == _starters.end());
       if (total_width + (int) next_offset > available_width)
         break;
-      total_width +=(int)  next_offset;
+      total_width += (int) next_offset;
       starter_count++;
     }
 
-    // Starter buttons are right aligned if they all fit into the available space. Otherwise we
-    // distribute the remaining space equally to all starter distances.
-    int remaining_space= 0;
-    if (starter_count == _starters.size())
-      current_offset += available_width - total_width;
-    else
-      remaining_space= available_width - total_width;
-
-    int extra_offset= 0;
-
-    // Do an integer div and keep the remaining pixel(s) separately to show smooth layout changes when resizing.
-    extra_offset= remaining_space / starter_count;
-    remaining_space= available_width - extra_offset * starter_count;
-
-    for (std::vector<StarterEntry>::iterator iterator= _starters.begin(); 
-      iterator != _starters.end(), starter_count > 0; iterator++)
+    if (starter_count > 0)
     {
-      // Store final position of the starter for later hit tests.
-      iterator->bounds.left= current_offset;
-      iterator->bounds.top= starter_top;
+      // Starter buttons are right aligned if they all fit into the available space. Otherwise we
+      // distribute the remaining space equally to all starter distances.
+      int remaining_space= 0;
+      if (starter_count == _starters.size())
+        current_offset += available_width - total_width;
+      else
+        remaining_space= available_width - total_width;
 
-      cairo_set_source_surface(cr, iterator->icon, current_offset + iterator->icon_bounds.left, 
-        starter_top + iterator->icon_bounds.top);
-      cairo_paint(cr);
+      int extra_offset= 0;
 
-      cairo_set_source_rgb(cr, 106 / 255.0, 109 / 255.0, 118 / 255.0);
-      if (iterator->title_line1.length() > 0)
+      // Do an integer div and keep the remaining pixel(s) separately to show smooth layout changes when resizing.
+      extra_offset= remaining_space / starter_count;
+      remaining_space= available_width - extra_offset * starter_count;
+
+      for (std::vector<StarterEntry>::iterator iterator= _starters.begin(); 
+        iterator != _starters.end(), starter_count > 0; iterator++)
       {
-        cairo_move_to(cr, current_offset + iterator->title1_location.x, starter_top + iterator->title1_location.y);
-        cairo_show_text(cr, iterator->title_line1.c_str());
-        cairo_stroke(cr);
-      }
-      if (iterator->title_line2.length() > 0)
-      {
-        cairo_move_to(cr, current_offset + iterator->title2_location.x, starter_top + iterator->title2_location.y);
-        cairo_show_text(cr, iterator->title_line2.c_str());
-        cairo_stroke(cr);
-      }
+        // Store final position of the starter for later hit tests.
+        iterator->bounds.left= current_offset;
+        iterator->bounds.top= starter_top;
 
-      current_offset += (int) get_next_offset(iterator,(iterator + 1) == _starters.end()) + extra_offset;
-      if (remaining_space > 0)
-      {
-        current_offset++;
-        remaining_space--;
-      }
+        cairo_set_source_surface(cr, iterator->icon, current_offset + iterator->icon_bounds.left, 
+          starter_top + iterator->icon_bounds.top);
+        cairo_paint(cr);
 
-      starter_count--;
+        cairo_set_source_rgb(cr, 106 / 255.0, 109 / 255.0, 118 / 255.0);
+        if (iterator->title_line1.length() > 0)
+        {
+          cairo_move_to(cr, current_offset + iterator->title1_location.x, starter_top + iterator->title1_location.y);
+          cairo_show_text(cr, iterator->title_line1.c_str());
+          cairo_stroke(cr);
+        }
+        if (iterator->title_line2.length() > 0)
+        {
+          cairo_move_to(cr, current_offset + iterator->title2_location.x, starter_top + iterator->title2_location.y);
+          cairo_show_text(cr, iterator->title_line2.c_str());
+          cairo_stroke(cr);
+        }
+
+        current_offset += (int) get_next_offset(iterator,(iterator + 1) == _starters.end()) + extra_offset;
+        if (remaining_space > 0)
+        {
+          current_offset++;
+          remaining_space--;
+        }
+
+        starter_count--;
+      }
     }
   }
 
@@ -1376,13 +1379,15 @@ Workspace::Workspace(HomeScreen* owner)
   _connections= new ActionList(_owner, COLUMN_LIST_LEFT_SPACING, COLUMN_LIST_RIGHT_SPACING, NULL,
     "db_connection_34x34.png", ActionOpenConnectionFromList);
   _connections->set_spacer_back_color(0xf4f7fc);
+  _connections_menu = 0;
+  /*
   _connections_menu = new Menu();
   _connections_menu->set_handler(sigc::mem_fun(this, &Workspace::handle_command));
   _connections_menu->add_item("Delete Connection...", "delete_connection");
   _connections_menu->add_separator();
   _connections_menu->add_item("Delete All Connections...", "delete_all_connections");
   _connections->set_context_menu(_connections_menu);
-  
+  */
   _left_column.set_back_color("#f4f7fc");
   _left_column.add(_sql_top, false, true);
   _left_column.add(_connections, true, true);
@@ -1410,12 +1415,15 @@ Workspace::Workspace(HomeScreen* owner)
   
   _models= new ActionList(_owner, COLUMN_LIST_LEFT_SPACING, COLUMN_LIST_RIGHT_SPACING, _separator_gradient,
     "db_model_34x34.png", ActionOpenEERModelFromList);
+  _models_menu = 0;
+  /*
   _models_menu = new Menu();
   _models_menu->set_handler(sigc::mem_fun(this, &Workspace::handle_command));
   _models_menu->add_item("Remove Model File from List", "remove_file");
   _models_menu->add_separator();
   _models_menu->add_item("Clear List", "clear_mru_list");
   _models->set_context_menu(_models_menu);
+   */
 
   _middle_column.set_back_color("#f4f7fc");
   _middle_column.add(_eer_top, false, true);
@@ -1448,13 +1456,14 @@ Workspace::Workspace(HomeScreen* owner)
   _server_instances= new ActionList(_owner, COLUMN_LIST_LEFT_SPACING, COLUMN_LIST_RIGHT_SPACING, NULL,
     "db_server_34x34.png", ActionManageInstanceFromList);
   _server_instances->set_spacer_back_color(0xf4f7fc);
-  _servers_menu = new Menu();
+  _servers_menu = 0;
+  /*_servers_menu = new Menu();
   _servers_menu->set_handler(sigc::mem_fun(this, &Workspace::handle_command));
   _servers_menu->add_item("Delete Server Instance...", "delete_instance");
   _servers_menu->add_separator();
   _servers_menu->add_item("Delete All Server Instances...", "delete_all_instances");
   _server_instances->set_context_menu(_servers_menu);
-  
+  */
   _right_column.set_back_color("#f4f7fc");
   _right_column.add(_admin_top, false, true);
   _right_column.add(_server_instances, true, true);
@@ -1544,40 +1553,67 @@ void Workspace::clear_list(HomeScreenListType list)
 
 //--------------------------------------------------------------------------------------------------
 
-void Workspace::handle_command(const std::string& command)
+static Menu *mforms_menu_from_menu_item_list(const bec::MenuItemList &menu_items)
+{
+  Menu *menu = new Menu();
+  
+  for (bec::MenuItemList::const_iterator item = menu_items.begin(); item != menu_items.end(); ++item)
+  {
+    if (item->type == bec::MenuAction)
+    {
+      int i = menu->add_item(item->caption, item->name);
+      menu->set_item_enabled(i, item->enabled);
+    }
+    else if (item->type == bec::MenuSeparator)
+      menu->add_separator();
+  }
+
+  return menu;
+}
+
+
+void Workspace::set_context_menu_items(HomeScreenListType list, const bec::MenuItemList &menu_items)
+{
+  switch (list)
+  {
+    case HomeScreenSQLDevList:
+      _connections_menu = mforms_menu_from_menu_item_list(menu_items);
+      _connections_menu->set_handler(sigc::bind(sigc::mem_fun(this, &Workspace::handle_command), list));
+      _connections->set_context_menu(_connections_menu);
+      break;
+    case HomeScreenModellingList:
+      _models_menu = mforms_menu_from_menu_item_list(menu_items);
+      _models_menu->set_handler(sigc::bind(sigc::mem_fun(this, &Workspace::handle_command), list));
+      _models->set_context_menu(_models_menu);
+      break;
+    case HomeScreenAdministrationList:
+      _servers_menu = mforms_menu_from_menu_item_list(menu_items);
+      _servers_menu->set_handler(sigc::bind(sigc::mem_fun(this, &Workspace::handle_command), list));
+      _server_instances->set_context_menu(_servers_menu);
+      break;
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void Workspace::handle_command(const std::string& command, HomeScreenListType list)
 {
   grt::ValueRef object;
-  HomeScreenAction action= ActionNone;
-
-  if (command == "delete_connection")
+  
+  switch (list)
   {
-    action= ActionDeleteConnection;
-    object= _connections->selected()->get_object();
-  }
+    case HomeScreenSQLDevList:
+      object= _connections->selected()->get_object();
+      break;
+    case HomeScreenModellingList:
+      object= _models->selected()->get_object();
+      break;
+    case HomeScreenAdministrationList:
+      object = _server_instances->selected()->get_object();
+      break;
+  }      
 
-  if (command == "delete_all_connections")
-    action= ActionDeleteAllConnections;
-
-  if (command == "remove_file")
-  {
-    action= ActionRemoveModelFile;
-    object= _models->selected()->get_object();
-  }
-
-  if (command == "clear_mru_list")
-    action= ActionClearModelFileList;
-
-  if (command == "delete_instance")
-  {
-    action= ActionDeleteServerInstance;
-    object= _server_instances->selected()->get_object();
-  }
-
-  if (command == "delete_all_instances")
-    action= ActionDeleteAllServerInstances;
-
-  if (action != ActionNone)
-    _owner->trigger_callback(action, object);
+  _owner->handle_context_menu(list, object, command);
 }
 
 //----------------- HomeScreen ---------------------------------------------------------------------
@@ -1627,6 +1663,13 @@ void HomeScreen::trigger_callback(HomeScreenAction action, const grt::ValueRef &
 
 //--------------------------------------------------------------------------------------------------
 
+void HomeScreen::set_context_menu_items(HomeScreenListType list, const bec::MenuItemList &menu_items)
+{
+  _workspace_content->set_context_menu_items(list, menu_items);
+}
+
+//--------------------------------------------------------------------------------------------------
+
 /**
  * Adds a new entry to the specified list on the home screen.
  */
