@@ -1,20 +1,20 @@
 /* 
- * (c) 2007-2008 MySQL AB, 2008-2010 Sun Microsystems, Inc.
+ * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
 #include "stdafx.h"
@@ -141,7 +141,7 @@ static bool match_context(const std::string &item_context, const std::string &cu
 
 static bool filter_context_platform(const app_CommandItemRef &item, const std::string &context)
 {
-  std::vector<std::string> plats(split_string(item->platform(), ","));
+  std::vector<std::string> plats(base::split(item->platform(), ","));
   
   if (!plats.empty())
   {
@@ -486,7 +486,10 @@ static bool parse_key(const std::string &key, mdc::KeyInfo &info)
     {mdc::KEscape,   "Escape"},
     {mdc::KTab,      "Tab"},
     {mdc::KEnter,    "Enter"},
-    {mdc::KNone,         ""}
+    {mdc::KSpace,    "Space"},
+    {mdc::KPlus,     "Plus"},
+    {mdc::KMinus,    "Minus"},
+    {mdc::KNone,         ""}      
   };
 
   info.keycode= mdc::KNone;
@@ -581,7 +584,7 @@ static bool parse_shortcut(const std::string &shortcut, mdc::KeyInfo &key, mdc::
   if (shortcut.empty())
     return false;
 
-  std::vector<std::string> parts= split_string(shortcut, "+");
+  std::vector<std::string> parts= base::split(shortcut, "+");
   mods= mdc::SNone;
 
   for (size_t c= parts.size()-1, i= 0; i < c; i++)
@@ -721,11 +724,6 @@ static MenuItemType get_menu_item_type(const std::string &type)
 void CommandUI::append_menu_items(const grt::ListRef<app_MenuItem> &plist, const std::string &context,
                                   std::vector<bec::MenuItem> *items)
 {
-#ifdef EDITION_OSS
-  bool hide_missing= true;
-  //_wb->get_wb_options().get_int("workbench:OSSHideMissing", 0);
-#endif
-
   for (size_t c= plist.count(), i= 0; i < c; i++)
   {
     bec::MenuItem item;
@@ -835,15 +833,7 @@ void CommandUI::append_menu_items(const grt::ListRef<app_MenuItem> &plist, const
 
 #ifdef EDITION_OSS
     if (bec::has_suffix(mitem.id(), "/SE"))
-    {
-      if (hide_missing)
-        continue;
-      if (item.type != MenuSeparator)
-      {
-        item.enabled= false;
-        item.type= MenuUnavailable;
-      }
-    }
+      continue;
 #endif
 
     if (fix_caption)
@@ -883,10 +873,7 @@ std::vector<bec::MenuItem> CommandUI::get_main_menus()
     
 #ifdef EDITION_OSS
     if (bec::has_suffix(mitem.id(), "/SE"))
-    {
-      //if (hide_missing)
-        continue;
-    }
+      continue;
 #endif
     
     item.type= MenuCascade;
@@ -978,7 +965,7 @@ std::vector<bec::MenuItem> CommandUI::get_menu_items(const std::string &menu, co
         app_PluginGroupRef group(groups.get(i));
         std::string category= group->category();
         
-        if (context != category)
+        if (context != category && category != "Others")
         {
           if (category == "SQLEditor" && context != WB_CONTEXT_QUERY)
             continue;
@@ -998,6 +985,26 @@ std::vector<bec::MenuItem> CommandUI::get_menu_items(const std::string &menu, co
           items.push_back(item);
         }
       }
+      
+      {
+        std::vector<app_PluginRef> plugins(_wb->get_plugin_manager()->get_plugins_for_group("Others/Menu/Ungrouped"));
+        bec::ArgumentPool argpool;
+        
+        _wb->update_plugin_arguments_pool(argpool);
+        
+        for (std::vector<app_PluginRef>::const_iterator iter= plugins.begin(); iter != plugins.end(); ++iter)
+        {
+          bec::MenuItem item;
+          item.name= std::string("plugin:").append(*(*iter)->name());
+          item.oid= item.name;
+          item.caption= (*iter)->caption();
+          item.enabled= _wb->get_grt_manager()->check_plugin_runnable(*iter, argpool);
+          item.checked= false;
+          item.type= MenuAction;
+          items.push_back(item);
+        }        
+      }
+      
       if (items.empty())
       {
         bec::MenuItem item;
@@ -1220,7 +1227,7 @@ std::vector<std::string> CommandUI::get_dropdown_items(const std::string &name,
   std::vector<std::string> items;
   WBComponent *compo;
 
-  compo= _wb->get_component_named(split_string(name, "/")[0]);
+  compo= _wb->get_component_named(base::split(name, "/")[0]);
   if (compo)
   {
     std::string::size_type p = option.find(':');
@@ -1239,7 +1246,7 @@ void CommandUI::select_dropdown_item(const std::string &name, const std::string
 {
   WBComponent *compo;
 
-  compo= _wb->get_component_named(split_string(name, "/")[0]);
+  compo= _wb->get_component_named(base::split(name, "/")[0]);
   if (compo)
   {
     std::string::size_type p = option.find(':');
@@ -1256,7 +1263,7 @@ void CommandUI::toggle_checkbox_item(const std::string &name, const std::string
 {
   WBComponent *compo;
 
-  compo= _wb->get_component_named(split_string(name, "/")[0]);
+  compo= _wb->get_component_named(base::split(name, "/")[0]);
   if (compo)
   {
     std::string::size_type p = option.find(':');
