@@ -94,6 +94,99 @@ void WBContextUI::show_home_screen()
     _home_screen= mforms::manage(new HomeScreen(_wb->get_grt()));
     _home_screen->set_on_close(sigc::bind(sigc::ptr_fun(on_home_closing), this));
     _home_screen->set_callback((home_screen_action_callback)&WBContextUI::home_action_callback, this);
+    _home_screen->handle_context_menu = sigc::mem_fun(this, &WBContextUI::handle_home_context_menu);
+    // setup context menus
+    bec::MenuItemList items, pitems;
+    bec::MenuItem item, sep;
+
+    sep.type = bec::MenuSeparator;
+    item.name= "open_connection";
+    item.caption= "Query Database";
+    items.push_back(item);
+    {
+      std::list<std::string> groups;
+      bec::ArgumentPool argument_pool;
+      _wb->update_plugin_arguments_pool(argument_pool);
+      argument_pool.add_entries_for_object("selectedConnection", db_mgmt_ConnectionRef(_wb->get_grt()), "db.mgmt.Connection");
+      groups.push_back("Menu/Home/Connections");
+      pitems = _wb->get_grt_manager()->get_plugin_context_menu_items(groups, argument_pool);
+      if (!pitems.empty())
+      {
+        items.push_back(sep);
+        items.insert(items.end(), pitems.begin(), pitems.end());
+      }
+    }
+    if (!items.empty())
+      items.push_back(sep);
+    item.name= "delete_connection";
+    item.caption= "Delete Connection...";
+    items.push_back(item);
+
+    item.caption= "Delete All Connections...";
+    item.name= "delete_connection_all";
+    items.push_back(item);
+
+    _home_screen->set_context_menu_items(HomeScreenSQLDevList, items);
+    
+    //
+    items.clear();
+    item.name= "open_model";
+    item.caption= "Open Model File";
+    items.push_back(item);    
+    {
+      std::list<std::string> groups;
+      bec::ArgumentPool argument_pool;
+      _wb->update_plugin_arguments_pool(argument_pool);
+      argument_pool.add_simple_value("selectedModelFile", grt::ValueRef());
+      groups.push_back("Menu/Home/ModelFiles");
+      pitems = _wb->get_grt_manager()->get_plugin_context_menu_items(groups, argument_pool);
+      if (!pitems.empty())
+      {
+        items.push_back(sep);
+        items.insert(items.end(), pitems.begin(), pitems.end());
+      }      
+    }
+    if (!items.empty())
+      items.push_back(sep);    
+    item.name= "remove_model";
+    item.caption= "Remove Model File from List";
+    items.push_back(item);
+    
+    item.caption= "Clear List";
+    item.name= "remove_model_all";
+    items.push_back(item);    
+    _home_screen->set_context_menu_items(HomeScreenModellingList, items);
+    
+    //
+    items.clear();
+    item.name= "open_administrator";
+    item.caption= "Manage Instance";
+    items.push_back(item);        
+    {
+      std::list<std::string> groups;
+      bec::ArgumentPool argument_pool;
+      _wb->update_plugin_arguments_pool(argument_pool);
+      argument_pool.add_entries_for_object("selectedInstance", db_mgmt_ServerInstanceRef(_wb->get_grt()), "db.mgmt.ServerInstance");
+      groups.push_back("Menu/Home/Instance");
+      pitems = _wb->get_grt_manager()->get_plugin_context_menu_items(groups, argument_pool);
+      if (!pitems.empty())
+      {
+        items.push_back(sep);
+        items.insert(items.end(), pitems.begin(), pitems.end());
+      }      
+    }
+    if (!items.empty())
+      items.push_back(sep);
+
+    item.name= "delete_instance";
+    item.caption= "Delete Server Instance...";
+    items.push_back(item);
+    
+    item.caption= "Delete All Server Instances...";
+    item.name= "delete_instance_all";
+    items.push_back(item);
+    
+    _home_screen->set_context_menu_items(HomeScreenAdministrationList, items);
   }
   mforms::App::get()->dock_view(_home_screen, "maintab");
   mforms::App::get()->set_view_title(_home_screen, "Home");
@@ -147,6 +240,116 @@ void WBContextUI::starter_action_callback(StarterAction action, const app_Starte
 
 //--------------------------------------------------------------------------------------------------
 
+void WBContextUI::handle_home_context_menu(HomeScreenListType list, const grt::ValueRef &object, const std::string &action)
+{
+  if (action == "open_connection")
+  {
+    handle_home_action(ActionOpenConnectionFromList, object);
+  }
+  else if (action == "delete_connection")
+  {
+    db_mgmt_ConnectionRef connection(db_mgmt_ConnectionRef::cast_from(object));
+    
+    std::string name= connection->name();
+    std::string text= strfmt(_("Do you want to delete connection %s?"), name.c_str());
+    int answer= Utilities::show_warning(_("Delete Connection"), text,  _("Delete"), _("Cancel"));
+    if (answer == mforms::ResultOk)
+    {
+      grt::ListRef<db_mgmt_Connection> connections(_wb->get_root()->rdbmsMgmt()->storedConns());
+      connections->remove(connection);
+      refresh_home_connections();
+    }
+  }
+  else if (action == "delete_connection_all")
+  {
+    std::string text= _("Do you really want to delete all defined connections?");
+    int answer= Utilities::show_warning(_("Delete All Connections"), text,  _("Delete"), _("Cancel"));
+    if (answer == mforms::ResultOk)
+    {
+      grt::ListRef<db_mgmt_Connection> connections(_wb->get_root()->rdbmsMgmt()->storedConns());
+      while (connections->count() > 0)
+        connections->remove(0);
+      refresh_home_connections();
+    }
+  }
+  else if (action == "open_model")
+  {
+    handle_home_action(ActionOpenEERModelFromList, object);
+  }
+  else if (action == "remove_model")
+  {
+    _wb->get_root()->options()->recentFiles()->remove(object);
+    refresh_home_models();
+  }
+  else if (action == "remove_model_all")
+  {
+    std::string text= _("Do you want to remove all entries from the model list?");
+    int answer= Utilities::show_warning(_("Clear Model Entry List"), text,  _("Delete"), _("Cancel"));
+    if (answer == mforms::ResultOk)
+    {
+      grt::StringListRef file_names(_wb->get_root()->options()->recentFiles());
+      for (int index= file_names->count() - 1; index >= 0; index--)
+      {
+        if (g_str_has_suffix(file_names[index].c_str(), ".mwb"))
+          file_names->remove(index);
+      }
+      refresh_home_models();
+    }
+  }
+  else if (action == "open_administrator")
+  {
+    handle_home_action(ActionManageInstanceFromList, object);
+  }
+  else if (action == "delete_instance")
+  {
+    db_mgmt_ServerInstanceRef instance(db_mgmt_ServerInstanceRef::cast_from(object));
+    
+    std::string name= instance->name();
+    std::string text= strfmt(_("Do you want to delete server instance %s?"), name.c_str());
+    int answer= Utilities::show_warning(_("Delete Server Instance"), text,  _("Delete"), _("Cancel"));
+    if (answer == mforms::ResultOk)
+    {
+      grt::ListRef<db_mgmt_ServerInstance> instances(_wb->get_root()->rdbmsMgmt()->storedInstances());
+      instances->remove(instance);
+      refresh_home_instances();
+    }
+  }
+  else if (action == "delete_instance_all")
+  {
+    std::string text= _("Do you really want to delete all defined server instances?");
+    int answer= Utilities::show_warning(_("Delete All Server Instances"), text,  _("Delete"), _("Cancel"));
+    if (answer == mforms::ResultOk)
+    {
+      grt::ListRef<db_mgmt_ServerInstance> instances(_wb->get_root()->rdbmsMgmt()->storedInstances());
+      while (instances->count() > 0)
+        instances->remove(0);
+      refresh_home_instances();
+    }
+  }
+  else 
+  {
+    bec::ArgumentPool argument_pool;
+    _wb->update_plugin_arguments_pool(argument_pool);
+    
+    switch (list)
+    {
+      case HomeScreenSQLDevList:
+        argument_pool.add_entries_for_object("selectedConnection", db_mgmt_ConnectionRef::cast_from(object));
+        break;
+      case HomeScreenModellingList:
+        argument_pool.add_simple_value("selectedModelFile", grt::StringRef::cast_from(object));
+        break;
+      case HomeScreenAdministrationList:
+        argument_pool.add_entries_for_object("selectedInstance", db_mgmt_ServerInstanceRef::cast_from(object));
+        break;        
+    }
+
+    get_command_ui()->activate_command(action, argument_pool);
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
 void WBContextUI::start_plugin(const std::string& title, const std::string& command, bool force_external)
 {
   try
@@ -434,7 +637,7 @@ void WBContextUI::handle_home_action(HomeScreenAction action, const grt::ValueRe
       }
       catch(grt::module_error& err)
       {
-        std::string msg = strfmt("Error in sercurityManager module: %s",err.what());
+        std::string msg = strfmt("Error in securityManager module: %s",err.what());
         _wb->show_status_text(msg);
       }
       break;
@@ -451,90 +654,6 @@ void WBContextUI::handle_home_action(HomeScreenAction action, const grt::ValueRe
       refresh_home_instances((int)_wb->get_root()->rdbmsMgmt()->storedInstances().get_index(instance));        
       break;
     }
-      
-    case ActionDeleteConnection:
-    {
-      db_mgmt_ConnectionRef connection(db_mgmt_ConnectionRef::cast_from(object));
-      
-      std::string name= connection->name();
-      std::string text= strfmt(_("Do you want to delete connection %s?"), name.c_str());
-      int answer= Utilities::show_warning(_("Delete Connection"), text,  _("Delete"), _("Cancel"));
-      if (answer == mforms::ResultOk)
-      {
-        grt::ListRef<db_mgmt_Connection> connections(_wb->get_root()->rdbmsMgmt()->storedConns());
-        connections->remove(connection);
-        refresh_home_connections();
-      }
-      break;
-    }
-      
-    case ActionDeleteAllConnections:
-    {
-      std::string text= _("Do you really want to delete all defined connections?");
-      int answer= Utilities::show_warning(_("Delete All Connections"), text,  _("Delete"), _("Cancel"));
-      if (answer == mforms::ResultOk)
-      {
-        grt::ListRef<db_mgmt_Connection> connections(_wb->get_root()->rdbmsMgmt()->storedConns());
-        while (connections->count() > 0)
-          connections->remove(0);
-        refresh_home_connections();
-      }
-      break;
-    }
-      
-    case ActionRemoveModelFile:
-    {
-      _wb->get_root()->options()->recentFiles()->remove(object);
-      refresh_home_models();
-      break;
-    }
-      
-    case ActionClearModelFileList:
-    {
-      std::string text= _("Do you want to remove all entries from the model list?");
-      int answer= Utilities::show_warning(_("Clear Model Entry List"), text,  _("Delete"), _("Cancel"));
-      if (answer == mforms::ResultOk)
-      {
-        grt::StringListRef file_names(_wb->get_root()->options()->recentFiles());
-        for (int index= file_names->count() - 1; index >= 0; index--)
-        {
-          if (g_str_has_suffix(file_names[index].c_str(), ".mwb"))
-            file_names->remove(index);
-        }
-        refresh_home_models();
-      }
-      break;
-    }
-      
-    case ActionDeleteServerInstance:
-    {
-      db_mgmt_ServerInstanceRef instance(db_mgmt_ServerInstanceRef::cast_from(object));
-      
-      std::string name= instance->name();
-      std::string text= strfmt(_("Do you want to delete server instance %s?"), name.c_str());
-      int answer= Utilities::show_warning(_("Delete Server Instance"), text,  _("Delete"), _("Cancel"));
-      if (answer == mforms::ResultOk)
-      {
-        grt::ListRef<db_mgmt_ServerInstance> instances(_wb->get_root()->rdbmsMgmt()->storedInstances());
-        instances->remove(instance);
-        refresh_home_instances();
-      }
-      break;
-    }
-      
-    case ActionDeleteAllServerInstances:
-    {
-      std::string text= _("Do you really want to delete all defined server instances?");
-      int answer= Utilities::show_warning(_("Delete All Server Instances"), text,  _("Delete"), _("Cancel"));
-      if (answer == mforms::ResultOk)
-      {
-        grt::ListRef<db_mgmt_ServerInstance> instances(_wb->get_root()->rdbmsMgmt()->storedInstances());
-        while (instances->count() > 0)
-          instances->remove(0);
-        refresh_home_instances();
-      }
-      break;
-    }
   }
 }
 
@@ -630,6 +749,20 @@ void WBContextUI::refresh_home_instances(int selected)
   grt::ListRef<db_mgmt_ServerInstance> instances(_wb->get_root()->rdbmsMgmt()->storedInstances());
   
   _home_screen->clear_list(HomeScreenAdministrationList);
+
+  // If there are no connections defined yet then create entries for all currently installed
+  // local servers.
+  if (instances->count() == 0)
+  {
+    grt::GRT* grt = _wb->get_grt();
+    grt::Module* module = grt->get_module("Workbench");
+    if (module == NULL)
+      throw std::logic_error("Internal error: can't find Workbench module.");
+
+    grt::StringListRef arguments(grt);
+    module->call_function("createInstancesFromLocalServers", arguments);
+  }
+
   for (grt::ListRef<db_mgmt_ServerInstance>::const_iterator end = instances.end(),
        inst = instances.begin(); inst != end; ++inst)
   {
@@ -646,9 +779,23 @@ void WBContextUI::refresh_home_instances(int selected)
 void WBContextUI::refresh_home_connections(int selected)
 {
   if (!_home_screen) return;
-  grt::ListRef<db_mgmt_Connection> connections(_wb->get_root()->rdbmsMgmt()->storedConns());  
+  grt::ListRef<db_mgmt_Connection> connections(_wb->get_root()->rdbmsMgmt()->storedConns());
   
   _home_screen->clear_list(HomeScreenSQLDevList);
+
+  // If there are no connections defined yet then create entries for all currently installed
+  // local servers.
+  if (connections->count() == 0)
+  {
+    grt::GRT* grt = _wb->get_grt();
+    grt::Module* module = grt->get_module("Workbench");
+    if (module == NULL)
+      throw std::logic_error("Internal error: can't find Workbench module.");
+    
+    grt::StringListRef arguments(grt);
+    module->call_function("createConnectionsFromLocalServers", arguments);
+  }
+
   for (grt::ListRef<db_mgmt_Connection>::const_iterator end = connections.end(),
        inst = connections.begin(); inst != end; ++inst)
   {
