@@ -1,20 +1,20 @@
 /* 
- * (c) 2007-2008 MySQL AB, 2008-2010 Sun Microsystems, Inc.
+ * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
 #include "stdafx.h"
@@ -135,7 +135,7 @@ void PluginManagerImpl::set_plugin_enabled(const app_PluginRef &plugin, bool fla
   {
     disabled_list.remove(idx);
     if (plugin->groups().count() == 0)
-      add_plugin_to_group(plugin, "Others/Ungrouped");
+      add_plugin_to_group(plugin, "Others/Menu/Ungrouped");
     else
     {
       for (size_t d= plugin->groups().count(), j= 0; j < d; j++)
@@ -252,12 +252,17 @@ void PluginManagerImpl::rescan_plugins()
         grt->send_warning("Duplicate plugin name "+*plugin->name(),
                           base::strfmt("There is more than one plugin with the name %s (in %s and %s).",
                           plugin->name().c_str(), (*pm)->name().c_str(), _plugin_source_module[plugin->name()].c_str()));
+        g_message("dup");
+        // must reset internal references in the object or we get a leak because of the cycles
+        plugin->reset_references();
         continue;
       }
       
       if (!check_plugin_validity(plugin, *pm))
+      {
+        g_message("invalid");
         continue;
-
+      }
       _plugin_source_module[plugin->name()]= (*pm)->name();
 
       // add to the plugin list
@@ -266,13 +271,14 @@ void PluginManagerImpl::rescan_plugins()
 
       if (disabled_plugins.find(*plugin->name()) != disabled_plugins.end())
       {
+        g_message("dis");
         if (grt->verbose())
           grt->send_info("Plugin "+*plugin->name()+" is disabled, skipping...", "");
         continue;
       }
 
       if (plugin->groups().count() == 0)
-        add_plugin_to_group(plugin, "Others/Ungrouped");
+        add_plugin_to_group(plugin, "Others/Menu/Ungrouped");
       else
       {
         for (size_t d= plugin->groups().count(), j= 0; j < d; j++)
@@ -311,6 +317,9 @@ void PluginManagerImpl::add_plugin_to_group(const app_PluginRef &plugin, const s
 
   if (group.is_valid())
     group->plugins().insert(plugin);
+  // TODO: warning for now disabled, fix group loading
+  //else 
+  //  g_warning("Plugin %s could not be added to unknown group %s", plugin->name().c_str(), group_name.c_str());
 }
 
 
@@ -1241,4 +1250,4 @@ grt::BaseListRef ArgumentPool::build_argument_list(const app_PluginRef &plugin)
     fargs.ginsert(argument);
   }  
   return fargs;
-}
\ No newline at end of file
+}
