@@ -1,20 +1,20 @@
 /* 
  * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
- * Boston, MA 02110-1301  USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
 #include "stdafx.h"
@@ -121,7 +121,7 @@ std::string CatalogHelper::dbobject_list_to_dragdata(const std::list<db_Database
 std::list<db_DatabaseObjectRef> CatalogHelper::dragdata_to_dbobject_list(const db_CatalogRef &catalog, const std::string &data)
 {
   std::list<db_DatabaseObjectRef> dbobjects;
-  std::vector<std::string> items= split_string(data, "\n");
+  std::vector<std::string> items= base::split(data, "\n");
   for (std::vector<std::string>::const_iterator item= items.begin(); item != items.end(); ++item)
   {
     db_DatabaseObjectRef object= dragdata_to_dbobject(catalog, *item);
@@ -251,10 +251,17 @@ db_TableRef TableHelper::create_associative_table(const db_SchemaRef &schema,
   if (atable.has_member("defaultCollationName"))
     atable->set_member("defaultCollationName", table1.get_member("defaultCollationName"));
 
-  create_foreign_key_to_table(atable, table1, true, mandatory1, true, true, rdbms, global_options, options);
+  db_ForeignKeyRef first_fk = create_foreign_key_to_table(atable, table1, true, mandatory1, true, true, rdbms, global_options, options);
   schema->tables().insert(atable);//Insert put newly added FK into schema to avoid name collision
   create_foreign_key_to_table(atable, table2, true, mandatory2, true, true, rdbms, global_options, options);
   
+  // when the 1st FK is created, the PK and FK will have the same columns, so the index will be reused
+  // when the 2nd FK is created, the PK will have both columns so it can't be used by the 1st FK, so we have
+  // to create a new one for it
+  db_IndexRef index = create_index_for_fk(schema->get_grt(), first_fk);
+  first_fk->index(index);
+  atable->indices().insert(index);
+  
   undo.end("Create Associative Table");
 
   return atable;
@@ -446,6 +453,25 @@ void TableHelper::update_foreign_key_index(grt::GRT *grt, const db_ForeignKeyRef
 }
 
 
+bool TableHelper::create_missing_indexes_for_foreign_keys(const db_TableRef &table)
+{
+  bool created= false;
+  GRTLIST_FOREACH(db_ForeignKey, table->foreignKeys(), fk)
+  {
+    db_IndexRef index;
+    // check if an index already exists for the FK columns and if not, create one
+    if (!(*fk)->index().is_valid() && !find_index_usable_by_fk(*fk).is_valid())
+    {
+      index = create_index_for_fk(table->get_grt(), *fk);
+      (*fk)->index(index);
+      table->indices().insert(index);
+      created= true;
+    }
+  }
+  return created;
+}
+
+
 /** 
  ****************************************************************************
  * @brief Creates a foreign key in table, refering to ref_table
@@ -466,12 +492,12 @@ void TableHelper::update_foreign_key_index(grt::GRT *grt, const db_ForeignKeyRef
  ****************************************************************************
  */
 db_ForeignKeyRef TableHelper::create_foreign_key_to_table(const db_TableRef &table, 
-                                                       const db_TableRef &ref_table,
-                                                       bool mandatory, bool ref_mandatory,
-                                                       bool many, bool identifying,
+                                                          const db_TableRef &ref_table,
+                                                          bool mandatory, bool ref_mandatory,
+                                                          bool many, bool identifying,
                                                           const db_mgmt_RdbmsRef &rdbms,
-                                                       const grt::DictRef &global_options,
-                                                       const grt::DictRef &options)
+                                                          const grt::DictRef &global_options,
+                                                          const grt::DictRef &options)
 {
   db_ForeignKeyRef new_fk;
   db_IndexRef pk= ref_table->primaryKey();
@@ -556,7 +582,9 @@ db_ForeignKeyRef TableHelper::create_foreign_key_to_table(const db_TableRef &tab
   {
     index = create_index_for_fk(grt, new_fk);
     new_fk->index(index);
+    // index is inserted later down
   }
+
   // add the FK to the source table
   table->foreignKeys().insert(new_fk);
   undo.set_description_for_last_action("Add Foreign Key to Table");
@@ -778,11 +806,14 @@ ColumnTypeCompareResult ColumnHelper::compare_column_types(const db_ColumnRef &f
 
 void ColumnHelper::set_default_value(db_ColumnRef column, const std::string &value)
 {
-  // TODO: this also should affect the isNotNull flag, but sometimes e.g. for 
-  // auto-inc or timestamp columns we shouldn't touch it, need to investigate 
-  // the full list of exceptions, but not sure if all these fields are still present in 5.1
   column->defaultValueIsNull(g_strcasecmp(value.c_str(), "NULL") ? 0 : 1);
   column->defaultValue(value.c_str());
+
+  // If a default value of NULL was set then the column can no longer be a not-null column.
+  if (column->defaultValueIsNull())
+    column->isNotNull(false);
+
+  // Handling auto-increment columns is MySQL specific and done elsewhere.
 }
 
 //------------------------------------------------------------------------------------
