@@ -1,20 +1,20 @@
 /* 
  * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
 
@@ -139,6 +139,7 @@ bool Recordset::reset(Recordset_data_storage::Ptr data_storage_ptr, bool rethrow
       _column_names.push_back("id");
       _column_types.push_back(int());
       _real_column_types.push_back(int());
+      _column_quoting.push_back(false);
 
       {
         sqlite::query q(*data_swap_db, "select coalesce(max(id)+1, 0) from `data`");
@@ -988,6 +989,15 @@ bec::MenuItemList Recordset::get_popup_menu_items(const std::vector<int> &rows,
   item.enabled = clicked_column >= 0 && rows.size() == 1 && !ro;
 #endif
   items.push_back(item);
+
+  item.name = "set_to_function";
+  item.caption = "Mark Field Value as a Function";
+#ifdef _WIN32
+  item.enabled = clicked_column >= 0 && !ro;
+#else
+  item.enabled = clicked_column >= 0 && rows.size() == 1 && !ro;
+#endif
+  items.push_back(item);
   
   item.name = "delete_row";
   item.caption = "Delete Row(s)";
@@ -1069,6 +1079,24 @@ bool Recordset::activate_popup_menu_item(const std::string &action, const std::v
       return true;
     }
   }
+  else if (action == "set_to_function")
+  {
+    if (rows.size() == 1 && clicked_column >= 0)
+    {
+      bec::NodeId node;
+      Cell cell;
+      
+      node.append(rows[0]);
+      std::string function;
+      if (!get_cell(cell, node, clicked_column, false))
+        function = "";
+      else
+        function = boost::apply_visitor(_var_to_str, *cell);
+      if (!g_str_has_prefix(function.c_str(), "\\func"))
+        set_field(node, clicked_column, std::string("\\func ")+function);
+      return true;
+    }
+  }  
   else if (action == "delete_row")
   {
     if (rows.size() > 0)
@@ -1257,6 +1285,9 @@ std::string Recordset::status_text()
     if (del_count > 0)
       status_text += strfmt(", deleted %i", del_count);
   }
+  status_text.append(".");
+  if (!status_text_trailer.empty())
+    status_text.append(" ").append(status_text_trailer);
   
   return status_text;
 }
