@@ -1,20 +1,20 @@
 /* 
  * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
 
@@ -121,6 +121,7 @@ void Recordset_sql_storage::do_unserialize(Recordset *recordset, sqlite::connect
   Recordset::Column_names &column_names= get_column_names(recordset);
   Recordset::Column_types &column_types= get_column_types(recordset);
   Recordset::Column_types &real_column_types= get_real_column_types(recordset);
+  Recordset::Column_quoting &column_quoting= get_column_quoting(recordset);
 
   _pkey_columns.clear();
   _fields_order.clear(); //! make auto var & bind like var_list
@@ -145,6 +146,9 @@ void Recordset_sql_storage::do_unserialize(Recordset *recordset, sqlite::connect
     real_column_types.reserve(column_names.size());
     std::fill_n(std::back_inserter(real_column_types), column_names.size(), std::string());
 
+    //XXX CHECK if above code is ok, what is this supposed to do? is it even called ?
+    column_quoting.reserve(column_names.size());
+    
     if (!column_names.empty())
     {
       // data
@@ -232,7 +236,7 @@ void Recordset_sql_storage::do_serialize(const Recordset *recordset, sqlite::con
 {
   _sql_script= std::string();
   Sql_script sql_script;
-  generate_sql_script(recordset, data_swap_db, sql_script, false);
+  generate_inserts(recordset, data_swap_db, sql_script);
   std::ostringstream oss;
   std::copy(sql_script.statements.begin(), sql_script.statements.end(), std::ostream_iterator<std::string>(oss, ";\n"));
   _sql_script= oss.str();
@@ -349,7 +353,9 @@ void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqli
   const Recordset::Column_names &column_names= get_column_names(recordset);
   const Recordset::Column_types &column_types= get_column_types(recordset);
   const Recordset::Column_types &real_column_types= get_real_column_types(recordset);
+  const Recordset::Column_quoting &column_quoting = get_column_quoting(recordset);
   const size_t partition_count= recordset->data_swap_db_partition_count();
+  const sqlide::VarToStr var_to_str;
 
   RowId min_new_rowid= recordset->min_new_rowid();
   ColumnId editable_col_count= recordset->get_column_count();
@@ -453,7 +459,9 @@ void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqli
 
                 data_row_rs->get_variant(partition_column, v);
                 values+= strfmt("%s, ",
-                  boost::apply_visitor(qv, column_types[column], v).c_str());
+                  column_quoting[column] || sqlide::is_var_null(v)?
+                  boost::apply_visitor(qv, column_types[column], v).c_str():
+                  boost::apply_visitor(var_to_str, v).c_str());
                 if (blob_columns[column])
                   sql_bindings.push_back(v);
               }
@@ -497,7 +505,9 @@ void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqli
                 data_row_rs->get_variant(partition_column, v);
                 values+= strfmt("`%s`=%s, ",
                   column_names[column].c_str(),
-                  boost::apply_visitor(qv, column_types[column], v).c_str());
+                  column_quoting[column] || sqlide::is_var_null(v)?
+                  boost::apply_visitor(qv, column_types[column], v).c_str():
+                  boost::apply_visitor(var_to_str, v).c_str());
                 if (blob_columns[column] && _binding_blobs)
                   sql_bindings.push_back(v);
               }
@@ -544,7 +554,10 @@ void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqli
             ColumnId partition_column= col - col_begin;
             data_rs->get_variant(partition_column, v);
             values+= strfmt("%s, ",
-              boost::apply_visitor(qv, column_types[col], v).c_str());
+                  column_quoting[partition_column] || sqlide::is_var_null(v)?
+                  boost::apply_visitor(qv, column_types[partition_column], v).c_str():
+                  boost::apply_visitor(var_to_str, v).c_str());
+
           }
         }
         if (!values.empty())
@@ -559,3 +572,75 @@ void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqli
     }
   }
 }
+
+
+void Recordset_sql_storage::generate_inserts(const Recordset *recordset, sqlite::connection *data_swap_db, Sql_script &sql_script)
+{
+  if (_is_sql_script_substitute_enabled)
+  {
+    sql_script= _sql_script_substitute;
+    return;
+  }
+
+  const Recordset::Column_names &column_names= get_column_names(recordset);
+  const Recordset::Column_types &column_types= get_column_types(recordset);
+  const Recordset::Column_types &real_column_types= get_real_column_types(recordset);
+  const Recordset::Column_quoting &column_quoted= get_column_quoting(recordset);
+  const size_t partition_count= recordset->data_swap_db_partition_count();
+
+  RowId min_new_rowid= recordset->min_new_rowid();
+  ColumnId editable_col_count= recordset->get_column_count();
+  sqlide::QuoteVar qv;
+  init_variant_quoter(qv);
+  std::string full_table_name= this->full_table_name();
+
+  if (0 == editable_col_count)
+    return;
+
+  std::vector<bool> blob_columns(editable_col_count);
+  for (ColumnId col= 0; editable_col_count > col; ++col)
+    blob_columns[col]= sqlide::is_var_blob(real_column_types[col]);
+
+    std::string col_names;
+    for (ColumnId col= 0; editable_col_count > col; ++col)
+      col_names += strfmt("`%s`, ", column_names[col].c_str());
+    if (!col_names.empty())
+      col_names.resize(col_names.size()-2);
+
+    std::list<boost::shared_ptr<sqlite::query> > data_queries(partition_count);
+    Recordset::prepare_partition_queries(data_swap_db, "select * from `data%s`", data_queries);
+    std::vector<boost::shared_ptr<sqlite::result> > data_results(data_queries.size());
+    if (Recordset::emit_partition_queries(data_swap_db, data_queries, data_results))
+    {
+      bool next_row_exists= true;
+      do
+      {
+        sqlite::Variant v;
+        std::string values;
+        for (size_t partition= 0; partition < partition_count; ++partition)
+        {
+          boost::shared_ptr<sqlite::result> &data_rs= data_results[partition];
+          for (ColumnId col_begin= partition * Recordset::DATA_SWAP_DB_TABLE_MAX_COL_COUNT, col= col_begin,
+            col_end= std::min<ColumnId>(editable_col_count, (partition + 1) * Recordset::DATA_SWAP_DB_TABLE_MAX_COL_COUNT); col < col_end; ++col)
+          {
+            ColumnId partition_column= col - col_begin;
+            data_rs->get_variant(partition_column, v);
+            sqlide::VarToStr var_to_str;
+
+            values+= strfmt("%s, ",//boost::apply_visitor(var_to_str, v).c_str());
+                            column_quoted[col] || sqlide::is_var_null(v)?
+                            boost::apply_visitor(qv, column_types[col], v).c_str():
+                            boost::apply_visitor(var_to_str, v).c_str());
+          }
+        }
+        if (!values.empty())
+          values.resize(values.size()-2);
+        std::string sql= strfmt("INSERT INTO %s (%s) VALUES (%s)", _omit_schema_qualifier ? table_name().c_str() : full_table_name.c_str(), col_names.c_str(), values.c_str());
+        sql_script.statements.push_back(sql);
+
+        BOOST_FOREACH (boost::shared_ptr<sqlite::result> &data_rs, data_results)
+          next_row_exists= data_rs->next_row();
+      }
+      while (next_row_exists);
+    }
+  }
