@@ -1,35 +1,31 @@
 /* 
- * © 2007-2008 MySQL AB, 2008-2009 Sun Microsystems, Inc.
+ * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
+
 #include "stdafx.h"
 
 #include "figure_common.h"
-#include "grt/common.h"
-/**
- * @file  figure_common.cpp
- * @brief 
- */
-
+#include "string_utilities.h"
 
 
 mdc::FontSpec parse_font_spec(const std::string &font)
 {
-  std::vector<std::string> parts= bec::split_string(font, " ");
+  std::vector<std::string> parts= base::split(font, " ");
   mdc::FontSpec fs;
   int size= 12;
 
@@ -135,6 +131,7 @@ bool Titlebar::on_leave(mdc::CanvasItem *target, const mdc::Point &point)
   return false;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 bool Titlebar::on_click(mdc::CanvasItem *target, const mdc::Point &point, mdc::MouseButton button, mdc::EventState state)
 {
@@ -143,6 +140,16 @@ bool Titlebar::on_click(mdc::CanvasItem *target, const mdc::Point &point, mdc::M
   return false;
 }
 
+//--------------------------------------------------------------------------------------------------
+
+bool Titlebar::on_double_click(mdc::CanvasItem *target, const mdc::Point &point, mdc::MouseButton button, mdc::EventState state)
+{
+  if (!_hub || !_hub->figure_double_click(_owner->represented_object(), target, point, button, state))
+    return super::on_double_click(target, point, button, state);
+  return false;
+}
+
+//--------------------------------------------------------------------------------------------------
 
 void Titlebar::set_icon(cairo_surface_t *icon)
 {
@@ -272,6 +279,7 @@ bool FigureItem::on_leave(mdc::CanvasItem *target, const mdc::Point &point)
   return false;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 bool FigureItem::on_click(mdc::CanvasItem *target, const mdc::Point &point, mdc::MouseButton button, mdc::EventState state)
 {
@@ -280,6 +288,16 @@ bool FigureItem::on_click(mdc::CanvasItem *target, const mdc::Point &point, mdc:
   return false;
 }
 
+//--------------------------------------------------------------------------------------------------
+
+bool FigureItem::on_double_click(mdc::CanvasItem *target, const mdc::Point &point, mdc::MouseButton button, mdc::EventState state)
+{
+  if (target!= this || !_hub->figure_double_click(_owner->represented_object(), target, point, button, state))
+    return super::on_double_click(target, point, button, state);
+  return false;
+}
+
+//--------------------------------------------------------------------------------------------------
 
 mdc::Point FigureItem::get_intersection_with_line_to(const mdc::Point &p)
 {
@@ -382,6 +400,13 @@ bool BaseFigure::on_click(mdc::CanvasItem *target, const mdc::Point &point, mdc:
   return false;
 }
 
+bool BaseFigure::on_double_click(mdc::CanvasItem *target, const mdc::Point &point, mdc::MouseButton button, mdc::EventState state)
+{
+  if (target!= this || !_hub->figure_double_click(represented_object(), target, point, button, state))
+    return super::on_double_click(target, point, button, state);
+  return false;
+}
+
 void BaseFigure::unset_color()
 {
   set_draw_background(false);
@@ -654,6 +679,7 @@ bool CaptionFigure::on_leave(mdc::CanvasItem *target, const mdc::Point &point)
   return false;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 bool CaptionFigure::on_click(mdc::CanvasItem *target, const mdc::Point &point, mdc::MouseButton button, mdc::EventState state)
 {
@@ -662,6 +688,16 @@ bool CaptionFigure::on_click(mdc::CanvasItem *target, const mdc::Point &point, m
   return false;
 }
 
+//--------------------------------------------------------------------------------------------------
+
+bool CaptionFigure::on_double_click(mdc::CanvasItem *target, const mdc::Point &point, mdc::MouseButton button, mdc::EventState state)
+{
+  if (!_hub->figure_double_click(_owner_object, target, point, button, state))
+    return super::on_double_click(target, point, button, state);
+  return false;
+}
+
+//--------------------------------------------------------------------------------------------------
 
 ShrinkableBox::ShrinkableBox(mdc::Layer *layer, mdc::Box::Orientation orientation)
 : mdc::Box(layer, orientation), _manual_resizing(false)
