@@ -1,20 +1,20 @@
 /* 
- * © 2007-2008 MySQL AB, 2008-2009 Sun Microsystems, Inc.
+ * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
 #include "stdafx.h"
@@ -109,7 +109,7 @@ void workbench_physical_Connection::ImplData::set_foreign_key(const db_ForeignKe
   
   self()->_foreignKey= fk;
   
-  if (owner_valid)
+  if (owner_valid && fk.is_valid())
     workbench_physical_DiagramRef::cast_from(self()->owner())->get_data()->add_fk_mapping(fk, self());
 
   update_connected_tables();
@@ -278,7 +278,8 @@ void workbench_physical_Connection::ImplData::fk_member_changed(const std::strin
   if (name == "owner")
   {
     _fk_changed_conn.disconnect();
-    _fk_changed_conn= self()->foreignKey()->owner()->signal_foreignKeyChanged().connect(sigc::mem_fun(this, &ImplData::fk_changed));
+    if (self()->foreignKey()->owner().is_valid())
+      _fk_changed_conn= self()->foreignKey()->owner()->signal_foreignKeyChanged().connect(sigc::mem_fun(this, &ImplData::fk_changed));
   }
 }
 
