@@ -1,20 +1,20 @@
 /* 
  * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
 #include "stdafx.h"
@@ -25,6 +25,8 @@
 #include "FindDialogBE.h"
 #include "Wb.h"
 
+using namespace System::Threading;
+
 namespace MySQL {
 namespace Workbench {
   
@@ -130,8 +132,9 @@ void WbFrontendCallbacks::create_main_form_view_wrapper(const std::string& view_
 
   if (0 == view_name.compare(WB_MAIN_VIEW_DB_QUERY))
   {
-    form= gcnew MySQL::GUI::Workbench::Db_sql_editor(
-      IntPtr(dynamic_cast<::Db_sql_editor*>(form_be.get())));
+    boost::shared_ptr<::Db_sql_editor> ref(boost::static_pointer_cast<::Db_sql_editor>(form_be));
+    form= gcnew MySQL::GUI::Workbench::Db_sql_editor(&ref);
+   //   IntPtr(dynamic_cast<::Db_sql_editor*>(form_be.get())));
   }
 
   create_main_form_view_delegate(CppStringToNativeRaw(view_name), form);
@@ -203,6 +206,42 @@ std::string WbFrontendCallbacks::get_bounds(mforms::App *app)
 
 //--------------------------------------------------------------------------------------------------
 
+static int message_loop_exit_code = -1; // Can stay unguarded. We only use it from the main thread.
+
+/**
+ * Establishes a local event loop which is needed by the python debugger. This is potentially a very
+ * dangerous and hard-to-debug thing so use it carefully.
+ * Additionally, if you fail to call exit_event_loop the call will never return and block closing
+ * the application.
+ */
+int WbFrontendCallbacks::enter_event_loop(mforms::App *app, float max_wait_time)
+{
+  message_loop_exit_code = -100000;
+  int remaining_milliseconds;
+  if (max_wait_time <= 0)
+    remaining_milliseconds = MAXINT;
+  else
+    remaining_milliseconds = (int) (1000 * max_wait_time);
+  while (message_loop_exit_code == -100000 && remaining_milliseconds > 0)
+  {
+    Application::DoEvents();
+    Thread::Sleep(100);
+    remaining_milliseconds -= 100;
+  }
+  if (remaining_milliseconds == 0)
+    return -1;
+  return message_loop_exit_code;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void WbFrontendCallbacks::exit_event_loop(mforms::App *app, int ret_code)
+{
+  message_loop_exit_code = ret_code;
+}
+
+//--------------------------------------------------------------------------------------------------
+
 void WbFrontendCallbacks::set_app_command(MySQL::Forms::Manager^ mgr, StringAppCommandAppViewStringDelegate^ dt)
 {
   app_command_delegate= dt;
@@ -221,9 +260,11 @@ void WbFrontendCallbacks::set_app_command(MySQL::Forms::Manager^ mgr, StringAppC
   DEF_CALLBACK2(std::string, ::mforms::App*, const std::string&, mgr, f->_app_impl, WbFrontendCallbacks, get_resource_path);
   DEF_CALLBACK2(void, ::mforms::App*, const std::string&, mgr, f->_app_impl, WbFrontendCallbacks, set_status_text);
   DEF_CALLBACK1(std::string, ::mforms::App*, mgr, f->_app_impl, WbFrontendCallbacks, get_bounds);
+  DEF_CALLBACK2(int, ::mforms::App*, float, mgr, f->_app_impl, WbFrontendCallbacks, enter_event_loop);
+  DEF_CALLBACK2(void, ::mforms::App*, int, mgr, f->_app_impl, WbFrontendCallbacks, exit_event_loop);
 }
 
 //--------------------------------------------------------------------------------------------------
 
 } // namespace Workbench
-} // namespace MySQL
\ No newline at end of file
+} // namespace MySQL
