@@ -1,20 +1,20 @@
 /* 
  * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
 #ifndef __WB_H__
@@ -193,7 +193,7 @@ public:
   delegate bool BoolStrStrStrStrDelegate(String^ str0, String^ str1, String^ str2, String^ str3);
   delegate bool BoolStrStrFloatDelegate(String^ str1, String^ str2, float f3);
   delegate bool BoolStrIntStrPtrDelegate(String^ str, int i, [Out] String^ %result);
-  delegate BaseWindowsCanvasView^ CanvasViewStringStringDelegate(String^ str1, String^ str2);
+  delegate BaseWindowsCanvasView^ CanvasViewStringStringIntPtrDelegate(String^ str1, String^ str2, IntPtr payload);
   delegate void VoidCanvasViewDelegate(BaseWindowsCanvasView^ canvasView);
   delegate void VoidStrUIFormDelegate(String^ str1, UIForm^ form);
   delegate IntPtr IntPtrGRTManagerModuleStrStrGrtListFlagsDelegate(GrtManager^ grtManager, GrtModule^ module, String^ str1, String^ str2, GrtValue^ grtlist, GUIPluginFlags flags);
@@ -237,9 +237,9 @@ private:
   // bool (string, int, string&)
   delegate bool BoolStrIntStrPtrWrapperDelegate(const std::string &str, int, std::string &res);
   typedef bool (*WbFrontendCallbacks::BOOL_STR_INT_STRPTR_CB)(const std::string &str, int, std::string &res);
-  // CanvasView * (string, string)
-  delegate ::mdc::CanvasView * CanvasViewStringStringWrapperDelegate(const std::string& str1, const std::string &str2);
-  typedef ::mdc::CanvasView * (*WbFrontendCallbacks::CANVASVIEW_STR_STR_CB)(const std::string& str1, const std::string& str2);
+  // CanvasView * (model_DiagramRef&)
+  delegate ::mdc::CanvasView* CanvasViewDiagramWrapperDelegate(const model_DiagramRef& diagram);
+  typedef ::mdc::CanvasView* (*WbFrontendCallbacks::CANVASVIEW_DIAGRAM_CB)(const model_DiagramRef& diagram);
   // void (CanvasView *)
   delegate void VoidCanvasViewWrapperDelegate(::mdc::CanvasView *canvas_view);
   typedef void (*WbFrontendCallbacks::VOID_CANVASVIEW_CB)(::mdc::CanvasView *canvas_view);
@@ -298,11 +298,12 @@ private:
   { shell_output_delegate(CppStringToNative(str1)); }
 
   // CanvasView
-  CanvasViewStringStringDelegate^ create_diagram_delegate;
-  CanvasViewStringStringWrapperDelegate^ create_diagram_wrapper_delegate;
-  ::mdc::CanvasView *create_diagram_wrapper(const std::string& str1, const std::string &str2)
+  CanvasViewStringStringIntPtrDelegate^ create_diagram_delegate;
+  CanvasViewDiagramWrapperDelegate^ create_diagram_wrapper_delegate;
+  ::mdc::CanvasView *create_diagram_wrapper(const model_DiagramRef& model)
   {
-    BaseWindowsCanvasView ^cv = create_diagram_delegate(CppStringToNative(str1), CppStringToNative(str2));
+    BaseWindowsCanvasView ^cv = create_diagram_delegate(CppStringToNative(model->id()),
+      CppStringToNative(model->name()), IntPtr(&model.content()));
     return cv->get_unmanaged_object();
   }
 
@@ -416,7 +417,7 @@ public:
                       VoidStrDelegate^ show_status_text,
                       BoolStrStrFloatDelegate^ show_progress,
                       BoolStrIntStrPtrDelegate^ request_input,
-                      CanvasViewStringStringDelegate^ create_diagram,
+                      CanvasViewStringStringIntPtrDelegate^ create_diagram,
                       VoidCanvasViewDelegate^ destroy_view,
                       VoidCanvasViewDelegate^ switched_view,
                       VoidCanvasViewDelegate^ tool_changed,
@@ -511,13 +512,13 @@ public:
     inner->request_input= sigc::ptr_fun(cb);
   }
 
-  void set_create_diagram(CanvasViewStringStringDelegate^ dt)
+  void set_create_diagram(CanvasViewStringStringIntPtrDelegate^ dt)
   {
     create_diagram_delegate= dt;
     create_diagram_wrapper_delegate=
-      gcnew CanvasViewStringStringWrapperDelegate(this, &WbFrontendCallbacks::create_diagram_wrapper);
+      gcnew CanvasViewDiagramWrapperDelegate(this, &WbFrontendCallbacks::create_diagram_wrapper);
     IntPtr ip = Marshal::GetFunctionPointerForDelegate(create_diagram_wrapper_delegate);
-    CANVASVIEW_STR_STR_CB cb = static_cast<CANVASVIEW_STR_STR_CB>(ip.ToPointer());
+    CANVASVIEW_DIAGRAM_CB cb = static_cast<CANVASVIEW_DIAGRAM_CB>(ip.ToPointer());
     inner->create_diagram= sigc::ptr_fun(cb);
   }
 
@@ -655,6 +656,8 @@ public:
   static std::string get_resource_path(mforms::App *app, const std::string &file);
   static void set_status_text(mforms::App *app, const std::string &text);
   static std::string get_bounds(mforms::App *app);
+  static int enter_event_loop(mforms::App *app, float max_wait_time);
+  static void exit_event_loop(mforms::App *app, int ret_code);
 
   void set_app_command(MySQL::Forms::Manager^ mgr, StringAppCommandAppViewStringDelegate^ dt);
 };
@@ -912,6 +915,11 @@ public:
   bool has_unsaved_changes()
   { return inner->get_wb()->has_unsaved_changes(); }
 
+  void open_document(String^ file)
+  {
+    inner->get_wb()->open_document(NativeToCppString(file));
+  }
+
   bool save_changes()
   { return inner->get_wb()->save_changes(); }
 
