@@ -32,6 +32,7 @@ using namespace mforms;
 
 
 #define PYTHON_SCRIPT_TEMPLATE \
+"# -*- coding: utf-8 -*-\n"\
 "# MySQL Workbench Python script\n"\
 "# <description>\n"\
 "# Written in MySQL Workbench %wbversion%\n"\
@@ -46,6 +47,7 @@ using namespace mforms;
 
 
 #define PYTHON_MODULE_TEMPLATE \
+"# -*- coding: utf-8 -*-\n"\
 "# MySQL Workbench module\n"\
 "# <description>\n"\
 "# Written in MySQL Workbench %wbversion%\n"\
@@ -383,7 +385,8 @@ private:
     }
     else if (_python_script.get_active())
     {
-      if (!_filename.get_string_value().empty())
+      // python scripts must have a filename (otherwise we can't set breakpoints)
+      //if (!_filename.get_string_value().empty())
       {
         std::string fn= _filename.get_string_value();
         if (!g_str_has_suffix(fn.c_str(), ".py"))
