@@ -21,95 +21,166 @@
 #include "grt_shell_window.h"
 #include "file_functions.h"
 #include "string_utilities.h"
+#include "wb_iterators.h"
 #include <errno.h>
 
 #include "grt/common.h"
 #include "grt_code_editor.h"
 #include "grt_plugin_wizard.h"
+#include "grt_python_debugger.h"
 
 using namespace MySQL::Geometry;
 
+
+#define EDITOR_TAB_OFFSET 2
+
 //--------------------------------------------------------------------------------------------------
 
 GRTShellWindow::GRTShellWindow(wb::WBContext* context)
 : mforms::Form(0, (mforms::FormFlag)(mforms::FormResizable | mforms::FormMinimizable | mforms::FormHideOnClose)), 
   _context(context),
-  _hsplitter(true), _side_tab(mforms::TabViewPalette), _main_tab(mforms::TabViewDocumentClosable),
-  _header_panel(mforms::FilledHeaderPanel), _files_tree((mforms::TreeOptions)0),
+  _toolbar(true),
+  _content(false), _padding_box(false), _hsplitter(true), _side_tab(mforms::TabViewPalette),
+  _main_tab(mforms::TabViewDocumentClosable),
   _global_box1(false), _global_box2(false), _global_splitter(false), _global_combo(mforms::SelectorCombobox),
   _global_tree((mforms::TreeOptions)0), _global_list(true), 
+#ifdef _WIN32
+  _side_header_panel(mforms::FilledHeaderPanel),
+  _lower_tab(mforms::TabViewPalette),
+  _lower_header_panel(mforms::FilledHeaderPanel),
+#else
+  _lower_tab(mforms::TabViewDocument),
+#endif
   _classes_box(false), _classes_splitter(false), _classes_tree(false), _classes_text(mforms::VerticalScrollBar),
   _modules_splitter(false), _modules_tree(false), _modules_text(mforms::VerticalScrollBar),
+  _right_splitter(false), 
   _shell_box(false), _shell_text(mforms::VerticalScrollBar), _shell_hbox(true),
-  _snippet_box(false), _snippet_vbox(false), _snippet_bbox(true), _snippet_list(mforms::TreeShowHeader),
-#ifdef _WIN32 // Can be removed once we have code editors for all platforms.
+  _output_text(mforms::VerticalScrollBar),
+  _snippet_splitter(false), _snippet_page(false),
   _snippet_text()
-#else
-  _snippet_text(mforms::BothScrollBars)
-#endif
 {
   set_title(("Workbench Scripting Shell"));
-  set_content(&_hsplitter);
+  set_content(&_content);
   signal_closed().connect(sigc::mem_fun(this, &GRTShellWindow::shell_closed));
   
+  _content.add(&_toolbar, false, true);
+
 #ifdef _WIN32
-  set_padding(Padding(6, 6, 6, 6));
+  _content.add(&_padding_box, true, true);
+  _padding_box.add(&_hsplitter, true, true);
+
+  _padding_box.set_padding(6);
   set_back_color("#283752");
-    _hsplitter.set_back_color("#283752");
-  _header_panel.add(&_side_tab);
-  _hsplitter.add(&_header_panel);
+  _hsplitter.set_back_color("#283752");
+  _side_header_panel.add(&_side_tab);
+  _side_header_panel.set_back_color("#283752");
+  _hsplitter.add(&_side_header_panel);
 #else
+  _content.add(&_hsplitter, true, true);
   _hsplitter.add(&_side_tab);
 #endif
 
   _side_tab.signal_tab_changed().connect(sigc::mem_fun(this, &GRTShellWindow::side_tab_changed));
 
-  _hsplitter.add(&_main_tab);
+  _hsplitter.add(&_right_splitter);
 
   // Side bar consists of 4 pages: files, globals tree, classes tree and modules tree.
   _first_show= true;
 
+  // Setup toolbar.
+#ifdef _WIN32
+  _toolbar.set_size(-1, 24);
+  _toolbar.set_back_color("#BCC7D8");
+#endif
+
+  _toolbar.set_padding(2);
+  _toolbar.set_spacing(4);
+
+  add_tool_button("tiny_new_doc.png", sigc::mem_fun(this, &GRTShellWindow::add_new_script),
+    _("Create a new file from a template"));
+  add_tool_button("tiny_load.png", sigc::mem_fun(this, &GRTShellWindow::open_script_file),
+    _("Open a script file"));
+
+  add_tool_separator();
+  _save_button = add_tool_button("tiny_save.png", sigc::bind(sigc::mem_fun(this, &GRTShellWindow::save_file), false),
+    _("Save file"));
+  _save_as_button = add_tool_button("tiny_saveas.png", sigc::bind(sigc::mem_fun(this, &GRTShellWindow::save_file), true),
+    _("Save file with a new name"));
+
+  add_tool_separator();
+  //  if (_editing_module)
+  {
+    //   add_tool_button("tiny_refresh.png", sigc::mem_fun(this, &GRTCodeEditor::execute),
+    //                   "Refresh the module");
+  }
+  //  else
+  {
+    _run_button = add_tool_button("tiny_execute.png", sigc::mem_fun(this, &GRTShellWindow::execute_file),
+      _("Execute script"));
+    _continue_button = add_tool_button("debug_continue.png", sigc::mem_fun(this, &GRTShellWindow::debug_continue), 
+      _("Continue execution until next breakpoint"));
+    _pause_button = add_tool_button("debug_pause.png", sigc::mem_fun(this, &GRTShellWindow::debug_pause),
+                                    "Pause script execution");
+    _step_button = add_tool_button("debug_step.png", sigc::mem_fun(this, &GRTShellWindow::debug_step), 
+      _("Step to next statement"));
+    _step_into_button = add_tool_button("debug_step_into.png", sigc::mem_fun(this, &GRTShellWindow::debug_step_into), 
+      _("Step into next function"));
+    _step_out_button = add_tool_button("debug_step_out.png", sigc::mem_fun(this, &GRTShellWindow::debug_step_out),
+      _("Finish execution of current function"));
+    _stop_button = add_tool_button("debug_stop.png", sigc::mem_fun(this, &GRTShellWindow::debug_stop),
+      _("Stop executing script"));
+
+    _continue_button->show(false);
+    _step_button->set_enabled(false);
+    _step_into_button->set_enabled(false);
+    _step_out_button->set_enabled(false);
+    _continue_button->set_enabled(false);
+    _stop_button->set_enabled(false);
+    _pause_button->set_enabled(false);
+  }  
+
+#ifndef _WIN32
+  // TODO: remove as soon as all platforms support closable tabs.
+  add_tool_button("Discard.png", sigc::mem_fun(this, &GRTShellWindow::close_tab),
+    _("Close this script tab"), false);
+#endif
+  add_tool_separator();
+
+  add_tool_button("clear_output.png", sigc::bind(sigc::mem_fun(_output_text, &mforms::TextBox::set_value), ""), 
+    _("Clear script output"), false);
+
+  add_tool_button("snippet_add.png", sigc::mem_fun(this, &GRTShellWindow::add_snippet), 
+    _("Add a new snippet"));
+  _snippet_delete_button = add_tool_button("snippet_del.png", sigc::mem_fun(this, &GRTShellWindow::del_snippet), 
+    _("Delete the selected snippet"));
+  _snippet_copy_button = add_tool_button("snippet_clipboard.png", sigc::mem_fun(this, &GRTShellWindow::copy_snippet), 
+    _("Copy snippet text to the clipboard"));
+  add_tool_separator();
+
   // Files
   mforms::Box *files_box= mforms::manage(new mforms::Box(false));
   
-  {
-    mforms::Box *bar = mforms::manage(new mforms::Box(true));
-    files_box->add(bar, false, true);
-    
 #ifdef _WIN32
-    bar->set_size(-1, 24);
-    bar->set_back_color("#BCC7D8");
+  _files_tree = mforms::manage(new mforms::TreeView(mforms::TreeNoBorder));
+#else
+  _files_tree = mforms::manage(new mforms::TreeView(mforms::TreeNone));
 #endif
 
-    bar->set_padding(2);
-    bar->set_spacing(4);
-    
-    App *app= App::get();
-    Button *b = manage(new Button(ToolButton));
-    b->set_icon(app->get_resource_path("tiny_new_doc.png"));
-    b->set_tooltip("Create a new file from a template");
-    b->signal_clicked().connect(sigc::mem_fun(this, &GRTShellWindow::add_new_script));
-    bar->add(b, false, true);
+  _files_menu.add_item("Add New File", "file-from-template");
+  _files_menu.add_item("Open Script File", "open-script");
+  _files_menu.add_separator();
+  _files_menu.add_item("Delete Script", "delete-script");
 
-    b = manage(new Button(ToolButton));
-    b->set_icon(app->get_resource_path("tiny_load.png"));
-    b->set_tooltip("Open a script file");
-    b->signal_clicked().connect(sigc::mem_fun(this, &GRTShellWindow::open_script_file));
-    bar->add(b, false, true);
+  _files_menu.set_handler(sigc::mem_fun(this, &GRTShellWindow::file_menu_activate));
 
-    b = manage(new Button(ToolButton));
-    b->set_icon(app->get_resource_path("Discard.png"));
-    b->set_tooltip("Delete the selected file");
-    b->signal_clicked().connect(sigc::mem_fun(this, &GRTShellWindow::delete_selected_file));
-    bar->add_end(b, false, true);    
-  }
+  _files_tree->set_context_menu(&_files_menu);
 
-  _files_tree.add_column(StringColumnType, "", 400, false);
-  _files_tree.end_columns();
+  _files_tree->add_column(StringColumnType, "", 400, false);
+  _files_tree->end_columns();
 
-  _files_tree.signal_row_activated().connect(sigc::mem_fun(this, &GRTShellWindow::file_list_activated));
+  _files_tree->signal_row_activated().connect(sigc::mem_fun(this, &GRTShellWindow::file_list_activated));
   
-  files_box->add(&_files_tree, true, true);
+  files_box->add(_files_tree, true, true);
 #ifdef _WIN32
   files_box->set_back_color("#FFFFFF");
   ((View*) files_box)->set_padding(Padding(0, 0, 0, 2));
@@ -120,6 +191,7 @@ GRTShellWindow::GRTShellWindow(wb::WBContext* context)
   _global_tree_be= 0;
 #ifdef _WIN32
   _global_splitter.set_back_color("#FFFFFF");
+  _hsplitter.set_back_color("#283752");
   _global_splitter.set_padding(Padding(0, 0, 0, 2));
 #endif
   _side_tab.add_page(&_global_splitter, "Globals");
@@ -135,6 +207,7 @@ GRTShellWindow::GRTShellWindow(wb::WBContext* context)
   _global_box2.add(&_global_entry, false, true);
   _global_entry.set_read_only(true);
   _global_entry.set_back_color("#FFFFFF");
+  
   _global_box2.add(&_global_list, true, true);
   _global_list.add_column(mforms::IconStringGRTColumnType, 0, "Name");
   _global_list.add_column(mforms::StringGRTColumnType, 1, "Value");
@@ -146,6 +219,12 @@ GRTShellWindow::GRTShellWindow(wb::WBContext* context)
   _global_combo.signal_changed().connect(sigc::mem_fun(this, &GRTShellWindow::change_global_path));
   _inspector= 0;
 
+  _global_menu.add_item(_("Copy Value"), "copy_value");
+  _global_menu.add_item(_("Copy Path"), "copy_path");
+  _global_menu.add_item(_("Copy Path for Python"), "copy_path_py");
+  _global_menu.set_handler(sigc::mem_fun(this, &GRTShellWindow::handle_global_menu));
+  
+  _global_tree.set_context_menu(&_global_menu);
   // 2) Classes tree
 #ifdef _WIN32
   _classes_splitter.set_back_color("#FFFFFF");
@@ -183,7 +262,25 @@ GRTShellWindow::GRTShellWindow(wb::WBContext* context)
   _modules_tree.set_column_width(0, 220);
   _modules_tree.signal_changed().connect(sigc::mem_fun(this, &GRTShellWindow::module_selected));
   
+  _right_splitter.add(&_main_tab);
+
+#ifdef _WIN32
+  _right_splitter.set_back_color("#283752");
+  _lower_header_panel.add(&_lower_tab);
+  _right_splitter.add(&_lower_header_panel);
+  _lower_header_panel.set_title(_("Debugging"));
+  _lower_header_panel.set_back_color("#283752");
+#else
+  _right_splitter.add(&_lower_tab);
+ #endif
+ 
   // setup shell
+#ifdef _WIN32
+  _snippet_list = mforms::manage(new TreeView((mforms::TreeOptions)(mforms::TreeShowHeader | mforms::TreeNoBorder)));
+#else
+  _snippet_list = mforms::manage(new TreeView(mforms::TreeShowHeader));
+#endif
+
   _shell_box.add(&_shell_text, true, true);
   _shell_text.set_monospaced(true);
   _shell_text.set_read_only(true);
@@ -197,21 +294,20 @@ GRTShellWindow::GRTShellWindow(wb::WBContext* context)
   
   _shell_entry.signal_action().connect(sigc::mem_fun(this, &GRTShellWindow::shell_action));
 
-  
   // snippets tab
-  _snippet_vbox.set_spacing(8);
-  _snippet_vbox.set_padding(8);
-  _snippet_vbox.add(&_snippet_box, true, true);
-  _snippet_vbox.add(&_snippet_bbox, false, true);
+#ifdef _WIN32
+//  ((View*)&_snippet_page)->set_padding(Padding(0, 0, 0, 21));
+  _snippet_page.set_back_color("#283752");
+  _snippet_splitter.set_back_color("#283752");
+#else
+  _snippet_page.set_spacing(8);
+  _snippet_page.set_padding(8);
+#endif
+  _snippet_page.add(&_snippet_splitter, true, true);
 
   //_snippet_box.set_spacing(8);
-  _snippet_box.add(&_snippet_list);
-  _snippet_box.add(&_snippet_text);
-
-  _snippet_bbox.set_spacing(12);
-  _snippet_bbox.add(&_snippet_add, false, true);
-  _snippet_bbox.add(&_snippet_del, false, true);
-  _snippet_bbox.add_end(&_snippet_copy, false, true);
+  _snippet_splitter.add(_snippet_list);
+  _snippet_splitter.add(&_snippet_text);
 
   _snippet_menu.add_item("Execute Snippet", "execute");
   _snippet_menu.add_item("Send to Script Editor", "new_with_snippet");
@@ -222,38 +318,43 @@ GRTShellWindow::GRTShellWindow(wb::WBContext* context)
 
   _snippet_menu.set_handler(sigc::mem_fun(this, &GRTShellWindow::snippet_menu_activate));
 
-  _snippet_list.set_context_menu(&_snippet_menu);
+  _snippet_list->set_context_menu(&_snippet_menu);
 
-  _snippet_del.set_text("Delete");
-  _snippet_add.set_text("Add");
-  _snippet_copy.set_text("Copy to Clipboard");
-  _snippet_add.signal_clicked().connect(sigc::mem_fun(this, &GRTShellWindow::add_snippet));
-  _snippet_del.signal_clicked().connect(sigc::mem_fun(this, &GRTShellWindow::del_snippet));
-  _snippet_copy.signal_clicked().connect(sigc::mem_fun(this, &GRTShellWindow::copy_snippet));
-
-  _snippet_list.signal_changed().connect(sigc::mem_fun(this, &GRTShellWindow::snippet_selected));
-#ifdef _WIN32
+  _snippet_list->signal_changed().connect(sigc::mem_fun(this, &GRTShellWindow::snippet_selected));
   _snippet_text.set_language(LanguagePython);
-#endif
 
-  _snippet_text.set_font(grtm()->get_app_option_string("workbench.general.Editor:Font"));
   _snippet_text.signal_changed().connect(sigc::mem_fun(this, &GRTShellWindow::snippet_changed));
 
-#ifndef _WIN32
-  _snippet_text.set_monospaced(true);
-#endif
-  
-  _snippet_list.add_column(mforms::StringColumnType, "Snippet", 500, false);
-  _snippet_list.end_columns();
-  _main_tab.add_page(&_snippet_vbox, "Snippets");
+  _snippet_list->add_column(mforms::StringColumnType, "Snippet", 500, false);
+  _snippet_list->end_columns();
+  _main_tab.add_page(&_snippet_page, "Snippets");
 
   _main_tab.signal_tab_closing().connect(sigc::mem_fun(this, &GRTShellWindow::on_tab_closing));
   _main_tab.signal_tab_closed().connect(sigc::mem_fun(this, &GRTShellWindow::on_tab_closed));
+  _main_tab.signal_tab_changed().connect(sigc::mem_fun(this, &GRTShellWindow::on_tab_changed));
 
-  _snippet_box.set_position(400);
-
+  //
+  
+  _output_text.set_read_only(true);
+  _output_text.set_monospaced(true);
+  _lower_tab.add_page(&_output_text, "Output");
+  
+  _debugger = new PythonDebugger(this, &_lower_tab);
+  
+  try
+  {
+    _debugger->init_pdb();
+  }
+  catch (const std::exception)
+  {
+    add_output("Could not initialize the debugger\n");
+    delete _debugger;
+    _debugger= 0;
+  }
+  
   // Minimum size for the entire window.
   set_size(800, 600);
+  _snippet_splitter.set_position(400);
   _hsplitter.set_position(250);
   _global_splitter.set_position(400);
   _modules_splitter.set_position(400);
@@ -262,8 +363,9 @@ GRTShellWindow::GRTShellWindow(wb::WBContext* context)
   grtm()->get_shell()->set_ready_handler(sigc::mem_fun(this, &GRTShellWindow::handle_prompt));
   grtm()->get_shell()->set_output_handler(sigc::mem_fun(this, &GRTShellWindow::handle_output));
 
+  on_tab_changed();
   snippet_selected();
-  side_tab_changed();
+  side_tab_changed();  
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -307,9 +409,9 @@ void GRTShellWindow::show(bool flag)
 {
   if (flag)
     refresh_all();
-  mforms::Form::show(flag);
-
+  
   load_state();
+  mforms::Form::show(flag);
 }
 
 
@@ -364,10 +466,134 @@ void GRTShellWindow::refresh_all()
   refresh_snippets();
 }
 
+
+//--------------------------------------------------------------------------------------------------
+
+void GRTShellWindow::execute_file()
+{
+  GRTCodeEditor *editor = get_active_editor();
+  if (!editor) return;
+
+  grtm()->push_output_callback(sigc::mem_fun(this, &GRTShellWindow::add_output));
+
+  if (_debugger && g_str_has_suffix(editor->get_path().c_str(), ".py"))
+  {
+    _run_button->show(false);
+    _continue_button->show(true);
+    _pause_button->set_enabled(true);
+    
+    _debugger->run(editor);
+    
+    _run_button->show(true);
+    _continue_button->show(false);
+    
+    _step_button->set_enabled(false);
+    _step_into_button->set_enabled(false);
+    _step_out_button->set_enabled(false);
+    _continue_button->set_enabled(false);
+    _stop_button->set_enabled(false);
+    _pause_button->set_enabled(false);
+  }
+  else
+    editor->execute();
+
+  grtm()->pop_output_callback();
+}
+
+
+void GRTShellWindow::debug_step()
+{
+  GRTCodeEditor *editor = get_active_editor();
+
+  if (editor && _debugger && g_str_has_suffix(editor->get_path().c_str(), ".py"))
+  {
+    if (_debugger->program_stopped())
+      _debugger->step();
+    else 
+    {
+      // start the program stopping at the 1st line
+      grtm()->push_output_callback(sigc::mem_fun(this, &GRTShellWindow::add_output));
+    
+      _run_button->show(false);
+      _continue_button->show(true);
+      _pause_button->set_enabled(true);
+      
+      _debugger->run(editor, true);
+      
+      _run_button->show(true);
+      _continue_button->show(false);
+      
+      _step_button->set_enabled(true);
+      _step_into_button->set_enabled(false);
+      _step_out_button->set_enabled(false);
+      _continue_button->set_enabled(false);
+      _stop_button->set_enabled(false);
+      _pause_button->set_enabled(false);
+
+      grtm()->pop_output_callback();
+    }
+  }
+}
+
+
+void GRTShellWindow::debug_step_into()
+{
+  if (_debugger)
+    _debugger->step_into();
+
+}
+
+void GRTShellWindow::debug_step_out()
+{
+  if (_debugger)
+    _debugger->step_out();
+  
+}
+
+
+void GRTShellWindow::debug_continue()
+{
+  if (_debugger)
+    _debugger->continue_();
+}
+
+
+void GRTShellWindow::debug_stop()
+{
+  if (_debugger)
+    _debugger->stop();
+}
+
+
+void GRTShellWindow::debug_pause()
+{
+  if (_debugger)
+    _debugger->pause();
+}
+
+void GRTShellWindow::save_file(bool save_as)
+{
+  GRTCodeEditor *editor = get_active_editor();
+  if (editor)
+    editor->save(save_as);
+}
+
+
+void GRTShellWindow::close_tab()
+{
+  GRTCodeEditor *editor = get_active_editor();
+  if (editor)
+  {
+    if (editor->can_close())
+      close_editor(editor);
+  }
+}
+
 //--------------------------------------------------------------------------------------------------
 
 void GRTShellWindow::side_tab_changed()
 {
+#ifdef _WIN32
   static std::string side_bar_titles[] = {
     _("File Browser"),
     _("Globals Tree"),
@@ -375,7 +601,8 @@ void GRTShellWindow::side_tab_changed()
     _("Modules List")
   };
 
-  _header_panel.set_title(side_bar_titles[_side_tab.get_active_tab()]);
+  _side_header_panel.set_title(side_bar_titles[_side_tab.get_active_tab()]);
+#endif
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -420,11 +647,13 @@ void GRTShellWindow::set_globals_tree_path(const std::string &path)
   }
 }
 
+
 void GRTShellWindow::change_global_path()
 {
   set_globals_tree_path(_global_combo.get_string_value());
 }
 
+
 void GRTShellWindow::change_class_sorting()
 {
   grtm()->get_shared_structs_tree()->set_display_mode((bec::StructsTreeBE::DisplayMode)_classes_sorting.get_selected_index());
@@ -492,6 +721,44 @@ void GRTShellWindow::module_selected()
 }
 
 
+
+void GRTShellWindow::handle_global_menu(const std::string &action)
+{
+  bec::NodeId selected;
+  
+  if (_global_tree.get_selected_node(selected))
+  {
+    if (action == "copy_value")
+    {
+      grt::ValueRef value(_global_tree_be->get_node_value(selected));
+      mforms::Utilities::set_clipboard_text(value.repr());      
+    }
+    else if (action == "copy_path")
+    {
+      mforms::Utilities::set_clipboard_text(_global_tree_be->get_path_for_node(selected, true));
+    }
+    else if (action == "copy_path_py")
+    {
+      std::string path = "grt.root";
+      std::vector<std::string> parts;
+      
+      parts = base::split(_global_tree_be->get_path_for_node(selected, true), "/");
+      for (base::const_range<std::vector<std::string> > p(parts); p; ++p)
+      {
+        if (p->empty()) continue;
+        
+        if (isdigit(p->at(0)))
+          path.append("[").append(*p).append("]");
+        else
+          path.append(".").append(*p);
+      }
+
+      mforms::Utilities::set_clipboard_text(path);
+    }
+  }
+}
+
+
 void GRTShellWindow::save_snippets()
 {
   std::string path = bec::make_path(grtm()->get_user_datadir(), "shell_snippets"+_script_extension);
@@ -503,10 +770,10 @@ void GRTShellWindow::save_snippets()
     return;
   }
   
-  int c = _snippet_list.count();
+  int c = _snippet_list->count();
   for (int i = _global_snippet_count; i < c; i++)
   {
-    std::string snippet = _snippet_list.get_row_tag(i);
+    std::string snippet = _snippet_list->get_row_tag(i);
     std::string::size_type p = 0, l = snippet.size();
 
     while (p < l)
@@ -550,9 +817,9 @@ void GRTShellWindow::load_snippets_from(const std::string &path)
       if (script.size() > 0)
         script.erase(script.size() - 1, 1);
       
-      int row = _snippet_list.add_row();
-      _snippet_list.set(row, 0, name);
-      _snippet_list.set_row_tag(row, script);
+      int row = _snippet_list->add_row();
+      _snippet_list->set(row, 0, name);
+      _snippet_list->set_row_tag(row, script);
     }
     fclose(f);
   }
@@ -561,10 +828,10 @@ void GRTShellWindow::load_snippets_from(const std::string &path)
 
 void GRTShellWindow::refresh_snippets()
 {
-  _snippet_list.clear_rows();
+  _snippet_list->clear_rows();
   
   load_snippets_from(grtm()->get_data_file_path("shell_snippets"+_script_extension));
-  _global_snippet_count = _snippet_list.count();
+  _global_snippet_count = _snippet_list->count();
   load_snippets_from(bec::make_path(grtm()->get_user_datadir(), "shell_snippets"+_script_extension));
   
   snippet_selected();
@@ -595,12 +862,12 @@ void GRTShellWindow::add_snippet()
 {
   std::string snippet = _comment_prefix + " new snippet\n";
   
-  int row = _snippet_list.add_row();
-  _snippet_list.set_row_tag(row, snippet);
-  _snippet_list.set_selected(row);
+  int row = _snippet_list->add_row();
+  _snippet_list->set_row_tag(row, snippet);
+  _snippet_list->set_selected(row);
 
   snippet_selected(); // force snippet to be displayed
-  snippet_changed(); // force setting of the snippet name
+  snippet_changed(0, 0); // force setting of the snippet name
 
   save_state();
 }
@@ -608,10 +875,10 @@ void GRTShellWindow::add_snippet()
 
 void GRTShellWindow::del_snippet()
 {
-  int sel = _snippet_list.get_selected();
+  int sel = _snippet_list->get_selected();
   if (sel >= 0)
   {
-    _snippet_list.delete_row(sel);
+    _snippet_list->delete_row(sel);
     snippet_selected();
     save_snippets();
   }
@@ -620,18 +887,18 @@ void GRTShellWindow::del_snippet()
 
 void GRTShellWindow::copy_snippet()
 {
-  int sel = _snippet_list.get_selected();
+  int sel = _snippet_list->get_selected();
   if (sel >= 0)
-    mforms::Utilities::set_clipboard_text(_snippet_list.get_row_tag(sel));
+    mforms::Utilities::set_clipboard_text(_snippet_list->get_row_tag(sel));
 }
 
 
 void GRTShellWindow::scriptize_snippet()
 {
-  int sel = _snippet_list.get_selected();
+  int sel = _snippet_list->get_selected();
   if (sel >= 0)
   {
-    std::string snippet = _snippet_list.get_row_tag(sel);
+    std::string snippet = _snippet_list->get_row_tag(sel);
     std::string language = "python";
     
     // kind of a hack.. pick language depending on comment type used
@@ -648,10 +915,10 @@ void GRTShellWindow::scriptize_snippet()
 
 void GRTShellWindow::run_snippet()
 {
-  int sel = _snippet_list.get_selected();
+  int sel = _snippet_list->get_selected();
   if (sel >= 0)
   {
-    std::string script = _snippet_list.get_row_tag(sel);
+    std::string script = _snippet_list->get_row_tag(sel);
 
     // auto-select the tab where output goes
     _main_tab.set_active_tab(0);
@@ -692,11 +959,11 @@ void GRTShellWindow::snippet_selected()
 {
   bool read_only = false;
   _snippet_text.set_read_only(false); // Necessary to be able to change the text.
-  int sel = _snippet_list.get_selected();
+  int sel = _snippet_list->get_selected();
   if (sel < 0)
   {
-    _snippet_del.set_enabled(false);
-    _snippet_copy.set_enabled(false);
+    _snippet_delete_button->set_enabled(false);
+    _snippet_copy_button->set_enabled(false);
     _snippet_text.set_value("");
     read_only = true;
     
@@ -708,7 +975,7 @@ void GRTShellWindow::snippet_selected()
     if (sel < _global_snippet_count)
     {
       read_only = true;
-      _snippet_del.set_enabled(false);
+      _snippet_delete_button->set_enabled(false);
       
       for (int i= 0; i < 6; i++)
       {
@@ -720,37 +987,38 @@ void GRTShellWindow::snippet_selected()
     }
     else
     {
-      _snippet_del.set_enabled(true);
+      _snippet_delete_button->set_enabled(true);
       
       for (int i= 0; i < 6; i++)
         _snippet_menu.set_item_enabled(i, true);
     }
-    _snippet_text.set_value(_snippet_list.get_row_tag(sel));
-    _snippet_copy.set_enabled(true);
+    _snippet_text.set_value(_snippet_list->get_row_tag(sel));
+    _snippet_copy_button->set_enabled(true);
   }
 
   _snippet_text.set_read_only(read_only);
 }
 
 
-void GRTShellWindow::snippet_changed()
+void GRTShellWindow::snippet_changed(int line, int linesAdded)
 {
   std::string snippet = _snippet_text.get_string_value();
-  int sel = _snippet_list.get_selected();
+  int sel = _snippet_list->get_selected();
   
   if (sel >= 0)
   {
-    _snippet_list.set_row_tag(sel, snippet);
+    _snippet_list->set_row_tag(sel, snippet);
     
     std::string::size_type p = snippet.find('\n');
     if (p != std::string::npos)
       snippet = snippet.substr(0, p);
-    _snippet_list.set(sel, 0, snippet);
+    _snippet_list->set(sel, 0, snippet);
     
     save_snippets();
   }
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void GRTShellWindow::snippet_menu_activate(const std::string &action)
 {
@@ -764,23 +1032,38 @@ void GRTShellWindow::snippet_menu_activate(const std::string &action)
     del_snippet();
 }
 
+//--------------------------------------------------------------------------------------------------
+
+void GRTShellWindow::file_menu_activate(const std::string &action)
+{
+  if (action == "file-from-template")
+    add_new_script();
+  else if (action == "open-script")
+    open_script_file();
+  else if (action == "delete-script")
+    delete_selected_file();
+}
+
+//--------------------------------------------------------------------------------------------------
 
 GRTCodeEditor *GRTShellWindow::add_editor(bool is_script, const std::string &language)
 {
   GRTCodeEditor *editor = manage(new GRTCodeEditor(this, !is_script, language));
-  
+
   _editors.push_back(editor);
   
   int page = _main_tab.add_page(editor, editor->get_title());
   _main_tab.set_active_tab(page);
   
-  editor->set_position(_main_tab.get_height() - (is_script ? 200 : 0));
-
   save_state();
+  
+  if (language == "python" && _debugger)
+    _debugger->editor_added(editor);
 
   return editor;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void GRTShellWindow::close_editor(GRTCodeEditor *editor)
 {
@@ -793,6 +1076,9 @@ void GRTShellWindow::close_editor(GRTCodeEditor *editor)
     }
   }
 
+  if (_debugger)
+    _debugger->editor_closed(editor);
+  
   _main_tab.remove_page(editor); 
   delete editor;
 
@@ -805,30 +1091,40 @@ void GRTShellWindow::open_file_in_editor(const std::string &path, bool is_script
   if (get_editor_for(path, true) != NULL)
     return;
 
-  char *data= NULL;
-  gsize length= 0;
-  GError *error= NULL;
-  
-  if (!g_file_get_contents(path.c_str(), &data, &length, &error))
-  {
-    Utilities::show_error("Open File",
-                          base::strfmt("Could not open %s: %s", path.c_str(),
-                                       error->message), 
-                          "OK");
-    g_error_free(error);
-    return;
-  }
-
-  std::string language = "python"; // Python module or plugin.
+  std::string language = ""; // No syntax highlighting if file extension is unknown.
   if (g_str_has_suffix(path.c_str(), ".py"))
     language = "python"; // Python script
   else
     if (g_str_has_suffix(path.c_str(), ".lua"))
       language = "lua"; // Lua script
+    else
+      if (g_str_has_suffix(path.c_str(), ".sql"))
+        language = "sql";
   GRTCodeEditor *editor= add_editor(is_script, language);
-  editor->set_path(path);
-  editor->set_text(std::string(data, length));
-  g_free(data);
+  if (!editor->load(path))
+  {
+    close_editor(editor);
+    return;
+  }
+#ifdef _DEBUG
+  editor->test_markup();
+#endif
+}
+
+
+GRTCodeEditor *GRTShellWindow::show_file_at_line(const std::string &path, int line)
+{
+  open_file_in_editor(path, true);
+  GRTCodeEditor *editor = get_editor_for(path, true);
+  if (!editor)
+    add_output(base::strfmt("Cannot open file %s", path.c_str()));
+  else
+  {
+    int start, length;
+    editor->get_editor()->get_range_of_line(line, start, length);
+    editor->get_editor()->set_selection(start, 0);
+  }
+  return editor;
 }
 
 
@@ -852,6 +1148,12 @@ void GRTShellWindow::add_new_script()
   save_state();
 }
 
+
+void GRTShellWindow::add_output(const std::string &text)
+{
+  _output_text.append_text(text, true);
+}
+
 //--------------------------------------------------------------------------------------------------
 
 void GRTShellWindow::set_editor_title(GRTCodeEditor *editor, const std::string &title)
@@ -889,12 +1191,12 @@ void GRTShellWindow::add_files_from_dir(const std::string &dirname, bool is_scri
   {
     if (g_str_has_suffix(name, ".py") || g_str_has_suffix(name, ".lua"))
     {
-      int row= _files_tree.add_row();
-      _files_tree.set(row, 0, std::string("      ")+name);
+      int row= _files_tree->add_row();
+      _files_tree->set(row, 0, std::string("      ")+name);
       if (is_script)
-        _files_tree.set_row_tag(row, std::string("s").append(dirname).append(G_DIR_SEPARATOR_S).append(name));
+        _files_tree->set_row_tag(row, std::string("s").append(dirname).append(G_DIR_SEPARATOR_S).append(name));
       else
-        _files_tree.set_row_tag(row, std::string("m").append(dirname).append(G_DIR_SEPARATOR_S).append(name));
+        _files_tree->set_row_tag(row, std::string("m").append(dirname).append(G_DIR_SEPARATOR_S).append(name));
     }
   }
 
@@ -906,36 +1208,44 @@ void GRTShellWindow::refresh_files()
 {
   int row;
   
-  _files_tree.clear_rows();
+  _files_tree->clear_rows();
   
-  row= _files_tree.add_row();
-  _files_tree.set(row, 0, "User Scripts");
+  row= _files_tree->add_row();
+  _files_tree->set(row, 0, "User Scripts");
   add_files_from_dir(grtm()->get_user_script_path(), true);
 
-  row= _files_tree.add_row();
-  _files_tree.set(row, 0, "User Modules");
+  row= _files_tree->add_row();
+  _files_tree->set(row, 0, "User Modules");
   add_files_from_dir(grtm()->get_user_module_path(), false);
 
-  row= _files_tree.add_row();
-  _files_tree.set(row, 0, "User Libraries");
+  row= _files_tree->add_row();
+  _files_tree->set(row, 0, "User Libraries");
   add_files_from_dir(grtm()->get_user_library_path(), true);  
 }
 
 
 void GRTShellWindow::file_list_activated(int row, int column)
 {
-  std::string path= _files_tree.get_row_tag(row);
+  std::string path= _files_tree->get_row_tag(row);
   if (!path.empty())
   {
     open_file_in_editor(path.substr(1), path[0] == 's');
   }
 }
 
+
+void GRTShellWindow::on_file_save(const std::string &file)
+{
+  refresh_files();
+  if (_debugger)
+    _debugger->refresh_file(file);
+}
+
 //--------------------------------------------------------------------------------------------------
 
 void GRTShellWindow::delete_selected_file()
 {
-  std::string path= _files_tree.get_row_tag(_files_tree.get_selected());
+  std::string path= _files_tree->get_row_tag(_files_tree->get_selected());
   if (!path.empty())
   {
     std::string fn = path.substr(1);
@@ -952,16 +1262,44 @@ void GRTShellWindow::delete_selected_file()
 
 //--------------------------------------------------------------------------------------------------
 
+mforms::Button *GRTShellWindow::add_tool_button(const std::string &image,
+                                    const sigc::slot<void> &action,
+                                    const std::string &tooltip,
+                                    bool left)
+{
+  App *app= App::get();
+  Button *b = manage(new Button(ToolButton));
+  b->set_icon(app->get_resource_path(image));
+  b->set_tooltip(tooltip);
+  b->signal_clicked().connect(action);
+  if (left)
+    _toolbar.add(b, false, true);
+  else
+    _toolbar.add_end(b, false, true);
+  return b;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void GRTShellWindow::add_tool_separator()
+{
+  App *app= App::get();
+  ImageBox* image = manage(new ImageBox());
+  image->set_image(app->get_resource_path("statusbar_separator.png"));
+  image->set_image_align(MiddleCenter);
+  _toolbar.add(image, false, true);
+}
+
+//--------------------------------------------------------------------------------------------------
+
 void GRTShellWindow::load_state()
 {
-  // Restore form's size and position.
   int x = _context->read_state("left", "scripting-shell", 100);
   int y = _context->read_state("top", "scripting-shell", 100);
   int width = _context->read_state("width", "scripting-shell", 800);
   int height = _context->read_state("height", "scripting-shell", 600);
 
   set_size(width, height);
-  // TODO: This crashed WB in linux, cause parent view is not set
   set_position(x, y);
 
   // Restore divider positions.
@@ -969,11 +1307,15 @@ void GRTShellWindow::load_state()
   _global_splitter.set_position(_context->read_state("global-splitter", "scripting-shell", 400));
   _modules_splitter.set_position(_context->read_state("modules-splitter", "scripting-shell", 400));
   _classes_splitter.set_position(_context->read_state("classes-splitter", "scripting-shell", 400));
+  _snippet_splitter.set_position(_context->read_state("snippets-splitter", "scripting-shell", 400));
 
   _shell_text.set_font(grtm()->get_app_option_string("workbench.scripting.ScriptingShell:Font"));
   _snippet_text.set_font(grtm()->get_app_option_string("workbench.scripting.ScriptingEditor:Font"));
   for (std::vector<GRTCodeEditor*>::iterator editor = _editors.begin(); editor != _editors.end(); editor++)
     (*editor)->set_font(grtm()->get_app_option_string("workbench.scripting.ScriptingEditor:Font"));
+  
+  _lower_tab_height = _context->read_state("editor-splitter", "scripting-shell", 400);
+  on_tab_changed();
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -991,6 +1333,7 @@ void GRTShellWindow::save_state()
   _context->save_state("global-splitter", "scripting-shell", _global_splitter.get_position());
   _context->save_state("modules-splitter", "scripting-shell", _modules_splitter.get_position());
   _context->save_state("classes-splitter", "scripting-shell", _classes_splitter.get_position());
+  _context->save_state("snippet-splitter", "scripting-shell", _snippet_splitter.get_position());
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1014,7 +1357,7 @@ bool GRTShellWindow::on_tab_closing(int index)
   if (index == 0 || index == 1)
     return false;
 
-  GRTCodeEditor* editor = _editors[index - 2];
+  GRTCodeEditor* editor = _editors[index - EDITOR_TAB_OFFSET];
   return editor->can_close();
 }
 
@@ -1022,26 +1365,90 @@ bool GRTShellWindow::on_tab_closing(int index)
 
 void GRTShellWindow::on_tab_closed(int index)
 {
-  GRTCodeEditor* editor = _editors[index - 2];
+  GRTCodeEditor* editor = _editors[index - EDITOR_TAB_OFFSET];
   close_editor(editor);
 }
 
 //--------------------------------------------------------------------------------------------------
 
+void GRTShellWindow::on_tab_changed()
+{
+  if (get_active_editor())
+  {
+    _save_button->set_enabled(true);
+    _save_as_button->set_enabled(true);
+    _run_button->set_enabled(true);
+    _step_button->set_enabled(true);
+    
+    _right_splitter.set_expanded(false, true);
+  }
+  else 
+  {
+    _save_button->set_enabled(false);
+    _save_as_button->set_enabled(false);
+    _run_button->set_enabled(false);
+    _step_button->set_enabled(false);
+    _right_splitter.set_expanded(false, false);
+  }
+
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void GRTShellWindow::activate_output_tab()
+{
+  _lower_tab.set_active_tab(0);
+}
+
+//--------------------------------------------------------------------------------------------------
+
 /**
  * Returns the editor which is currently editing the given file.
  */
 GRTCodeEditor* GRTShellWindow::get_editor_for(const std::string& path, bool select_tab)
 {
+#ifdef _WIN32
+  // We probably would need g_utf8_normalize too if we want it really good, but since this is
+  // supposed to be a temporary solution...
+  gchar* path1 = g_utf8_strdown(path.c_str(), -1);
   for (std::vector<GRTCodeEditor*>::iterator editor = _editors.begin(); editor != _editors.end(); editor++)
+  {
+    gchar* path2 = g_utf8_strdown((*editor)->get_path().c_str(), -1);
+    if (g_utf8_collate(path1, path2) == 0)
+    {
+      if (select_tab)
+        _main_tab.set_active_tab(editor - _editors.begin() + EDITOR_TAB_OFFSET);
+
+      g_free(path2);
+      return *editor;
+    }
+    g_free(path2);
+  }
+
+  g_free(path1);
+#else
+  for (std::vector<GRTCodeEditor*>::iterator editor = _editors.begin(); editor != _editors.end(); editor++)
+  {
     if ((*editor)->get_path() == path)
     {
       if (select_tab)
-        _main_tab.set_active_tab(editor - _editors.begin() + 2);
+        _main_tab.set_active_tab(editor - _editors.begin() + EDITOR_TAB_OFFSET);
+
       return *editor;
     }
-
+  }
+#endif
   return NULL;
 }
 
+
+GRTCodeEditor *GRTShellWindow::get_active_editor()
+{
+  int index = _main_tab.get_active_tab() - EDITOR_TAB_OFFSET;
+  if (index >= 0 && index < (int)_editors.size())
+    return _editors[index];
+  return 0;
+}
+
 //--------------------------------------------------------------------------------------------------
+
