@@ -1,20 +1,20 @@
 /* 
  * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
- * Boston, MA 02110-1301  USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
 #include "stdafx.h"
@@ -37,6 +37,7 @@ static struct {
   const char *os_name;
 } platform_strings[] = {
 {"apple-darwin", "MacOS X"},
+{"osx10.6", "MacOS X"}, // <-- became this in MySQL 5.5
 {"-linux", "Linux"},
 {"Win64", "Windows"},
 {"Win32", "Windows"},
@@ -96,7 +97,11 @@ HostMachinePage::HostMachinePage(WizardForm* host, wb::WBContext *wb)
   Label *indent;
 
   indent = manage(new Label(""));
+#ifdef _WIN32
+  indent->set_size(20, 23); // Need a minimum height or the drop down will be cropped.
+#else
   indent->set_size(20, -1);
+#endif
   _dbhost_block.add(indent, false, true);
   _dbhost_block.add(&_dbhost_selector, true, true);
 
@@ -334,6 +339,17 @@ TestDatabaseSettingsPage::TestDatabaseSettingsPage(WizardForm* host)
   add_task(_("Get Server OS"),
            sigc::mem_fun(this, &TestDatabaseSettingsPage::get_server_platform),
            _("Querying server OS type..."));
+
+/*XXX TODO
+  add_task(_("Check Account Privileges"),
+           sigc::mem_fun(this, &TestDatabaseSettingsPage::get_server_platform),
+           _("Checking if account being used has enough privileges..."));
+- check the privs the account has and warn if it can't do something:
+- creating and modifying user accts
+- querying for tables
+- check if it has global schema rights and if not, warn dumps will only work for databases it has access to
+- other stuff
+  */
   
   end_adding_tasks(true, _("Database connection tested successfully."));
   
@@ -499,9 +515,7 @@ HostAndRemoteTypePage::HostAndRemoteTypePage(WizardForm* host)
   _ssh_remote_admin.signal_toggled().connect(sigc::mem_fun(this, &HostAndRemoteTypePage::toggle_remote_admin));
 
   _management_type_box.add(&_no_remote_admin, false, true);
-#ifdef NOT_READY
   _management_type_box.add(&_win_remote_admin, false, true);
-#endif
   _management_type_box.add(&_ssh_remote_admin, false, true);
 
   _no_remote_admin.set_active(true);
@@ -513,6 +527,7 @@ HostAndRemoteTypePage::HostAndRemoteTypePage(WizardForm* host)
   // OS selection.
   _os_panel.set_title(_("Operating System Selection"));
   _os_panel.add(&_os_box);
+  _os_panel.show(false); // Initially hidden to avoid it flashing when switching to the page and before enter is called.
 
   _os_description.set_wrap_text(true);
   _os_description.set_text(_("Select the operating system and the type of database installation "
@@ -541,9 +556,8 @@ HostAndRemoteTypePage::HostAndRemoteTypePage(WizardForm* host)
   _os_box.set_spacing(10);
   _os_panel.set_padding(8);
   
+  _os_panel.show(false);
   add(&_os_panel, false, true);
-
-  toggle_remote_admin();
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -552,6 +566,9 @@ void HostAndRemoteTypePage::enter(bool advancing)
 {
   if (!advancing)
     return;
+  
+  // force check of admin type
+  toggle_remote_admin();
 
   if (wizard()->is_local())
     set_title(_("Specify the installation type for you target operation system"));
@@ -656,9 +673,7 @@ void HostAndRemoteTypePage::toggle_remote_admin()
   // Clear out any OS value if we don't use remote management.
   if (_no_remote_admin.get_active() && !wizard()->is_local())
   {
-    _os_selector.set_selected(-1);
-    _type_selector.set_selected(-1);
-    _os_box.set_enabled(false);
+    _os_panel.show(false);
   }
   else
   {
@@ -670,13 +685,13 @@ void HostAndRemoteTypePage::toggle_remote_admin()
     if (_win_remote_admin.get_active() && !wizard()->is_local())
     {
       detected_os_type = "Windows";
-      _os_box.set_enabled(false);
+      _os_panel.show(false);
       _type_selector.set_selected(-1);
     }
     else
     {
       refresh_profiles = true;
-      _os_box.set_enabled(true);
+      _os_panel.show(true);
       if (detected_os_type.empty() && wizard()->is_local())
       {
 #ifdef _WIN32
@@ -846,7 +861,7 @@ SSHConfigurationPage::SSHConfigurationPage(WizardForm* host)
 #else
     "~";
 #endif
-  _file_selector->initialize(homedir + "/.ssh/id_rsa", mforms::OpenFile, "", "...", 
+  _file_selector->initialize(homedir + "/.ssh/id_rsa", mforms::OpenFile, "", "...", true,
     sigc::mem_fun(this, &WizardPage::validate));
   use_ssh_key_changed();
   
@@ -974,7 +989,7 @@ WindowsManagementPage::WindowsManagementPage(WizardForm* host, wb::WBContext* co
 
   // Setup for configuration file browsing.
   _file_selector= mforms::manage(new FsObjectSelector(&_browse_button, &_config_path));
-  _file_selector->initialize("", mforms::OpenFile, "", "...", sigc::mem_fun(this, &WizardPage::validate));
+  _file_selector->initialize("", mforms::OpenFile, "", "...", true, sigc::mem_fun(this, &WizardPage::validate));
 
   add(&_layout_table, false, true);
 }
@@ -1006,6 +1021,7 @@ void WindowsManagementPage::enter(bool advancing)
   {
     wizard()->clear_problem();
     _config_paths.clear();
+    _service_names.clear();
     _service_selector.clear();
 
     std::string host = values().get_string("host_name");
@@ -1062,7 +1078,7 @@ void WindowsManagementPage::enter(bool advancing)
         arguments.ginsert(grt::StringRef(user));
         arguments.ginsert(grt::StringRef(password));
 
-        // this can take a long time
+        // This can take a few seconds.
         wmi_session = module->call_function("wmiOpenSession", arguments);
       }
 
@@ -1089,42 +1105,7 @@ void WindowsManagementPage::enter(bool advancing)
         std::string state = entry.get_string("State", "unknown");
         std::string start_mode = entry.get_string("StartMode", "unknown");
 
-        std::string config_file;
-        size_t position = path.find("--defaults-file");
-        if (position != std::string::npos)
-        {
-          position += 15; // Skip option name and find equal sign.
-          while (position < path.size() && path[position] != '=')
-            position++;
-
-          if (path[position] == '=')
-          {
-            position++;
-
-            // Skip any white space.
-            while (position < path.size() && path[position] == ' ')
-              position++;
-
-            char terminator;
-            if (path[position] == '"' || path[position] == '\'')
-              terminator = path[position++];
-            else
-              terminator = ' ';
-
-            size_t end_position = path.find(terminator, position);
-            if (end_position == std::string::npos)
-            {
-              // Terminator not found means the string was either not properly terminated (if quoted)
-              // or contains no space char. In this case take everything we can get.
-              if (terminator != ' ')
-                position++;
-              config_file = path.substr(position);
-            }
-            else
-              config_file = path.substr(position, end_position - position);
-          }
-        }
-
+        std::string config_file = base::extract_option_from_command_line("--defaults-file", path);
         if (!local_connection)
         {
           if (config_file.empty())
@@ -1133,6 +1114,7 @@ void WindowsManagementPage::enter(bool advancing)
           base::replace(config_file, ":", "$");
         }
         _config_paths.push_back(config_file);
+        _service_names.push_back(service_name);
         std::string selector_text = service_display_name + " (" + state + ", Start mode: " + start_mode + ")";
         _service_selector.add_item(selector_text);
       }
@@ -1143,8 +1125,13 @@ void WindowsManagementPage::enter(bool advancing)
     }
     catch (std::runtime_error e)
     {
-      _progress_label.set_text(base::strfmt(_("Could not initialize WMI: %s"), e.what()));
+      _progress_label.set_text(base::strfmt(_("Could not set up connection: %s"), e.what()));
       wizard()->set_problem(e.what());
+
+      // In case this error was caused by wrong credentials (we cannot be sure from the error message)
+      // we remove the already stored password to make it possible to enter the current one.
+      Utilities::forget_password("wmi@" + host, user);
+      values().gset("wmi_user_name", "");
     }
   }
 }
@@ -1153,9 +1140,9 @@ void WindowsManagementPage::enter(bool advancing)
 
 bool WindowsManagementPage::advance()
 {
-  std::string config_path = _config_paths[_service_selector.get_selected_index()];
-  values().gset("ini_path", config_path);
+  values().gset("ini_path", _config_path.get_string_value());
   values().gset("ini_section", "mysqld");
+  values().gset("service_name", _service_names[_service_selector.get_selected_index()]); 
 
   return true;
 }
@@ -1223,12 +1210,6 @@ void TestHostMachineSettingsPage::enter(bool advance)
 
 //--------------------------------------------------------------------------------------------------
 
-void TestHostMachineSettingsPage::leave(bool advance)
-{
-}
-
-//--------------------------------------------------------------------------------------------------
-
 bool TestHostMachineSettingsPage::connect_to_host()
 {
   // This will require the ssh or SSH key password, so it needs to be called from main thread.
@@ -1278,10 +1259,26 @@ bool TestHostMachineSettingsPage::skip_page()
 
 void TestHostMachineSettingsPage::tasks_finished(bool success)
 {
-  // if the template matches reality, we display the settings to the user and ask if they're ok.
-  // otherwise, just go straight to customization
-  values().gset("template_is_suitable", success);
-  values().gset("customize", success ? 0 : 1);
+  values().gset("host_tests_succeeded", success);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void TestHostMachineSettingsPage::leave(bool advancing)
+{
+  if (advancing)
+  {
+    bool require_review = false;
+    if (values().get_int("host_tests_succeeded") == 1)
+    {
+      require_review = Utilities::show_message(_("Review settings"), _("The tests returned without error, so it appears the connection "
+        "settings are correct. Do you still want to review them for changes that may be required?"), _("Review"), _("Continue without review"))
+        == mforms::ResultOk;
+    }
+    else
+      require_review = true;
+    values().gset("review_required", require_review);
+  }
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1296,12 +1293,13 @@ NewServerInstanceWizard* TestHostMachineSettingsPage::wizard()
 ReviewPage::ReviewPage(WizardForm* host)
   : NewServerInstancePage(host, "review"), _text(VerticalScrollBar)
 {
-  set_short_title(_("Review Parameters"));
-  set_title(_("Review MySQL Management Parameters"));
+  set_short_title(_("Review Settings"));
+  set_title(_("Review Server Instance Settings"));
   
-  _label.set_text(_("Check if the following default settings match your actual settings and toggle 'Change Parameters' "
-                  "if you need to make any changes.\n"
-                  "Be extra careful if you run more than one instance of MySQL in the same machine."));
+  _label.set_text(_("Below is a list of all settings collected so far. This includes also values taken "
+    "from templates or default values. Check if they match your actual settings and toggle 'Change Parameters' "
+    "if you need to make any changes to default values. For any other change go back to the appropriate wizard page.\n\n"
+    "Pay special attention if you run more than one instance of MySQL on the same machine."));
   _label.set_wrap_text(true);
 
   _text.set_read_only(true);
@@ -1330,18 +1328,36 @@ void ReviewPage::enter(bool advancing)
 
     grt::DictRef serverInfo(wizard()->assemble_server_instance()->serverInfo());
     
-    std::string host_name= values().get_string("host_name", "localhost");
-    std::string ssh_port= values().get_string("ssh_port", "22");
-    std::string ssh_user_name= values().get_string("ssh_user_name");
-    std::string ssh_key_path= values().get_string("ssh_key_path");
     bool ssh_management= values().get_int("remoteAdmin") != 0;
-    summary.append(strfmt(_("SSH Based Adminstration:  %s\n"), ssh_management? _("Yes") : _("No")));
+    bool wmi_management= values().get_int("windowsAdmin") != 0;
+    
+    std::string host_name= values().get_string("host_name", "localhost");
     if (ssh_management)
     {
+      std::string ssh_port= values().get_string("ssh_port", "22");
+      std::string ssh_user_name= values().get_string("ssh_user_name");
+      std::string ssh_key_path= values().get_string("ssh_key_path");
+
+      summary.append(_("SSH Based Adminstration enabled\n"));
       summary.append(strfmt(_("    SSH host:  %s:%s\n"), host_name.c_str(), ssh_port.c_str()));
       summary.append(strfmt(_("    SSH user:  %s\n"), ssh_user_name.c_str()));
       summary.append(strfmt(_("    SSH key file:  %s\n"), ssh_key_path.empty() ? "not set" : ssh_key_path.c_str()));
     }
+    else
+      if (wmi_management)
+      {
+        std::string user_name= values().get_string("wmi_user_name");
+        std::string service_name= values().get_string("service_name");
+
+        summary.append(_("Native Windows Adminstration enabled\n"));
+        summary.append(strfmt(_("    Windows host:  %s\n"), host_name.c_str()));
+        if (!wizard()->is_local())
+          summary.append(strfmt(_("    Windows user name:  %s\n"), user_name.c_str()));
+        summary.append(strfmt(_("    MySQL service name:  %s\n"), service_name.c_str()));
+      }
+      else
+        summary.append(_("Remote management is disabled\n"));
+
     summary.append("\n");
     std::string os_title= serverInfo.get_string("sys.system", "Unknown");
     std::string ini_path= serverInfo.get_string("sys.config.path");
@@ -1354,16 +1370,20 @@ void ReviewPage::enter(bool advancing)
     summary.append(strfmt(_("    Path to Configuration File:  %s\n"), ini_path.c_str()));
     summary.append(strfmt(_("    Instance Name in Configuration File:  %s\n"), ini_section.c_str()));
     summary.append("\n");
-    std::string start= serverInfo.get_string("sys.mysqld.start");
-    std::string stop= serverInfo.get_string("sys.mysqld.stop");
-    std::string status= serverInfo.get_string("sys.mysqld.status");
-    bool use_sudo= serverInfo.get_int("sys.usesudo") != 0;
-    summary.append(_("Commands for MySQL Management\n"));
-    summary.append(strfmt(_("    Start MySQL:  %s\n"), start.c_str()));
-    summary.append(strfmt(_("    Stop MySQL:  %s\n"), stop.c_str()));
-    summary.append(strfmt(_("    Check status:  %s\n"), status.c_str()));
-    if (os_title != "Windows")
-      summary.append(strfmt(_("    Use sudo:  %s\n"), use_sudo ? _("Yes") : _("No")));
+
+    if (!wmi_management)
+    {
+      std::string start= serverInfo.get_string("sys.mysqld.start");
+      std::string stop= serverInfo.get_string("sys.mysqld.stop");
+      std::string status= serverInfo.get_string("sys.mysqld.status");
+      bool use_sudo= serverInfo.get_int("sys.usesudo") != 0;
+      summary.append(_("Commands for MySQL Management\n"));
+      summary.append(strfmt(_("    Start MySQL:  %s\n"), start.c_str()));
+      summary.append(strfmt(_("    Stop MySQL:  %s\n"), stop.c_str()));
+      summary.append(strfmt(_("    Check status:  %s\n"), status.c_str()));
+      if (os_title != "Windows")
+        summary.append(strfmt(_("    Use sudo:  %s\n"), use_sudo ? _("Yes") : _("No")));
+    }
     
     _text.set_value(summary);
   }
@@ -1373,8 +1393,7 @@ void ReviewPage::enter(bool advancing)
 
 bool ReviewPage::skip_page()
 {
-  // if admin is disabled or the defaults failed, go straight to customize
-  return !(wizard()->is_admin_enabled()) || values().get_int("template_is_suitable", 0);
+  return values().get_int("review_required", 0) == 0;
 }
 
 //----------------- PathsPage ----------------------------------------------------------------------
@@ -1416,7 +1435,7 @@ PathsPage::PathsPage(WizardForm* host, wb::WBContext* context)
   
   // Setup for local config file browsing. This will be adjusted if we are at a remote location.
   _file_selector= mforms::manage(new FsObjectSelector(&_browse_button, &_config_path));
-  _file_selector->initialize("", mforms::OpenFile, "", "...", sigc::mem_fun(this, &WizardPage::validate));
+  _file_selector->initialize("", mforms::OpenFile, "", "...", true, sigc::mem_fun(this, &WizardPage::validate));
 
   _test_config_path_button.set_text(_("Check Path"));
   _test_config_path_button.signal_clicked().connect(sigc::mem_fun(this, &PathsPage::test_path));
@@ -1786,10 +1805,8 @@ NewServerInstanceWizard::NewServerInstanceWizard(wb::WBContext* context)
   _ssh_configuration_page= new SSHConfigurationPage(this);
   add_page(manage(_ssh_configuration_page));
     
-#ifdef NOT_READY
   _windows_connection_page= new WindowsManagementPage(this, _context);
   add_page(manage(_windows_connection_page));
-#endif
 
   _test_host_machine_settings_page= new TestHostMachineSettingsPage(this);
   add_page(manage(_test_host_machine_settings_page));
@@ -1874,9 +1891,11 @@ db_mgmt_ServerInstanceRef NewServerInstanceWizard::assemble_server_instance()
   {
     std::string ini_path= values().get_string("ini_path");
     std::string ini_section= values().get_string("ini_section");
+    std::string service_name= values().get_string("service_name");
   
     _instance->serverInfo().gset("sys.config.path", ini_path);
     _instance->serverInfo().gset("sys.config.section", ini_section);
+    _instance->serverInfo().gset("sys.mysqld.service_name", service_name);
   }
   if (get_active_page_number() >= 8
       && values().get_int("customize")) // don't overwrite defaults unless user has had time to enter them
