@@ -744,7 +744,7 @@ void ListModelWrapper::get_value_vfunc(const iterator& iter, int column, Glib::V
             sv= _tm->get_field_description(node, -column);
           else
             _tm->get_field_repr(node, column, sv);
-          set_glib_string(value, sv);
+          set_glib_string(value, sv, true);
           lmwdprint("LMW::get_value_vfunc: %s: node %s, col %i: value: '%s'\n", _name.c_str(), node.repr().c_str(), column, sv.c_str());
           break;
         }
