@@ -269,7 +269,16 @@ NativeHandle MainForm::open_plugin_becb(bec::GRTManager* mgr, grt::Module *grtmo
       GUIPluginBase* guiplugin = reinterpret_cast<GUIPluginBase*>(handles[0]);
       // try to reuse the plugin
       PluginEditorBase *editor= dynamic_cast<PluginEditorBase*>(guiplugin);
-            
+ 
+      if (flags & bec::StandaloneWindowFlag)
+      {
+        add_plugin_form(editor);
+      }
+      else
+      {
+        // add it to the bottom panel of the current tab
+        bring_plugin_pane(editor);
+      }
       if (editor && editor->can_close() && editor->switch_edited_object(mgr, args))
       {
         mgr->get_plugin_manager()->forget_gui_plugin_handle(handles[0]);
@@ -869,7 +878,7 @@ void MainForm::perform_command_becb(const std::string& command)
 }
 
 //------------------------------------------------------------------------------
-mdc::CanvasView* MainForm::create_view_becb(const std::string &id, const std::string &name)
+mdc::CanvasView* MainForm::create_view_becb(const model_DiagramRef &diagram)
 {
   ModelDiagramPanel *model_panel= Gtk::manage(ModelDiagramPanel::create(_wbui_context, _toolbar_manager));
 
@@ -877,7 +886,7 @@ mdc::CanvasView* MainForm::create_view_becb(const std::string &id, const std::st
 
   _signal_close_editor.connect(sigc::hide_return(sigc::mem_fun(model_panel, &ModelDiagramPanel::close_editors_for_object)));
 
-  model_panel->init(id);
+  model_panel->init(diagram.id());
 
   add_form_pane(model_panel, !model_panel->get_diagram_form()->is_closed());
   model_panel->set_data("canvas", model_panel->get_canvas());
@@ -1311,10 +1320,49 @@ void MainForm::add_plugin_pane(PluginEditorBase *pane)
     {
       // add the editor to the active tab
       form->add_plugin_tab(pane);
+
+      pane->set_data("ContainerForm", form);
     }
   }
 }
 
+
+void MainForm::bring_plugin_pane(PluginEditorBase *pane)
+{
+  // locate the active main tab
+  Gtk::Notebook *note = get_upper_note(); 
+  int pagenum= note->get_current_page();
+
+  if (pagenum < 0)
+  {
+    g_warning("Cannot add plugin with no active tab");
+    return;
+  }
+  
+
+  FormViewBase *old_form = reinterpret_cast<FormViewBase*>(pane->get_data("ContainerForm"));
+
+  Gtk::Widget *page = note->get_nth_page(pagenum);
+  if (page)
+  {
+    FormViewBase *form = reinterpret_cast<FormViewBase*>(page->get_data("FormViewBase"));
+    
+    if (form && old_form != form)
+    {
+      pane->reference();
+      if (old_form)
+        old_form->remove_plugin_tab(pane);
+      // remove from where it is now
+      // add the editor to the active tab
+      form->add_plugin_tab(pane);
+      pane->unreference();
+      pane->set_data("ContainerForm", form);
+    }
+  }
+}
+
+
+
 static bool close_plugin_form(GdkEventAny *ev, PluginEditorBase *frame)
 {
   frame->close_live_object_editor();
@@ -1611,13 +1659,14 @@ static std::string get_resource_path(mforms::App* app, const std::string& file)
   if (file.empty()) return self->get_wbui_context()->get_wb()->get_grt_manager()->get_data_file_path("");
   if (file[0] == '/') return file;
 
-  if (g_str_has_suffix(file.c_str(), ".png"))
+  if (g_str_has_suffix(file.c_str(), ".png") || g_str_has_suffix(file.c_str(), ".xpm"))
     return bec::IconManager::get_instance()->get_icon_path(file);
   else if (g_str_has_suffix(file.c_str(), ".vbs"))
   {
     return self->get_wbui_context()->get_wb()->get_grt_manager()->get_data_file_path(file);
   }
-  return "";
+
+  return self->get_wbui_context()->get_wb()->get_grt_manager()->get_data_file_path(file);
 }
 
 
@@ -1648,6 +1697,32 @@ static void end_thread_loop()
 {
 }
 
+
+static std::list<std::pair<int,int> > event_loop_exit_codes;
+
+static int begin_event_loop(mforms::App*,float timeout)
+{
+  Gtk::Main::run();
+  if (event_loop_exit_codes.empty() || event_loop_exit_codes.back().first != Gtk::Main::level())
+  {
+    // means something other than end_event_loop() called quit(), could mean the app is being quit.. 
+    Gtk::Main::quit();
+    return -1;
+  }
+  else
+  {
+    int rc = event_loop_exit_codes.back().second;
+    event_loop_exit_codes.pop_back();
+    return rc;
+  }
+}
+
+static void end_event_loop(mforms::App *,int rc)
+{
+  Gtk::Main::quit();
+  event_loop_exit_codes.push_back(std::make_pair(Gtk::Main::level()-1, rc));
+}
+
 void MainForm::setup_mforms_app()
 {
   mforms::ControlFactory *cf = mforms::ControlFactory::get_instance();
@@ -1664,6 +1739,8 @@ void MainForm::setup_mforms_app()
   cf->_app_impl.get_bounds= &get_main_window_bounds;
   cf->_app_impl.begin_thread_loop= &begin_thread_loop;
   cf->_app_impl.end_thread_loop= &end_thread_loop;
+  cf->_app_impl.enter_event_loop= &begin_event_loop;
+  cf->_app_impl.exit_event_loop= &end_event_loop;
 }
 
 
