@@ -274,7 +274,7 @@ void DbSqlEditorView::close_editor_tab(boost::shared_ptr<SqlEditorFE> editor)
 
   // auto-add a new buffer if all of them are closed
   if (_be->sql_editor_count() == 0)
-    _be->new_sql_script_file();
+    _be->new_sql_scratch_area();
 }
 
 void DbSqlEditorView::add_editor_tab()
@@ -288,7 +288,7 @@ void DbSqlEditorView::add_editor_tab()
   editor->set_font(_be->wbsql()->get_wbui()->get_wb()->get_wb_options().get_string("workbench.general.Editor:Font"));
   
   Gtk::Label *label;
-  _editor_note->append_page(editor->widget(), *create_closeable_tab("SQL Query", 
+  _editor_note->append_page(editor->widget(), *create_closeable_tab(_be->sql_editor_caption(_be->active_sql_editor_index()),
                                                                     sigc::bind(sigc::mem_fun(this, &DbSqlEditorView::close_editor_tab),
                                                                                editor),
                                                                    &label));
@@ -570,8 +570,6 @@ bool DbSqlEditorView::on_sql_editor_key_press_event(GdkEventKey *event)
 
 void DbSqlEditorView::on_sql_editor_selection_change()
 {
-  active_editor()->set_selected_range();
-
   _be->request_refresh_of_menu_and_toolbar();
 }
 
@@ -588,7 +586,7 @@ bool DbSqlEditorView::validate_explain_sql()
 
 void DbSqlEditorView::explain_sql()
 {
-  active_editor()->set_selected_range();
+  // active_editor()->set_selected_range();
   _be->explain_sql(false);
 }
 
@@ -627,7 +625,6 @@ void DbSqlEditorView::toggle_sidebar()
 void DbSqlEditorView::editor_caption_changed()
 {
   Gtk::Label *label = reinterpret_cast<Gtk::Label*>(active_editor()->widget().get_data("tab_label"));
-  
   label->set_text(_be->sql_editor_caption());
 }
 
