@@ -1,20 +1,20 @@
 /* 
- * (c) 2009-2010 Sun Microsystems, Inc.
+ * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
 #import "sigobjc++.h"
@@ -52,6 +52,7 @@ static NSImage *descendingSortIndicator= nil;
     
     [column setDataCell: [[[MQResultSetCell alloc] init] autorelease]];
     [[column dataCell] setEditable: YES];
+    [[column dataCell] setLineBreakMode: NSLineBreakByClipping];
     
     [mTableView addTableColumn: column];
   }
@@ -451,10 +452,15 @@ static int onRefresh(MResultsetViewer *self)
 {
   if (mData && [aTableColumn identifier] != nil)
   {
-    std::string text;
-    (*mData)->get_field_repr(rowIndex, [[aTableColumn identifier] intValue], text);
-    text = bec::replace_string(text, "\n", " ");
-    return [NSString stringWithUTF8String: text.c_str()];
+    int columnIndex = [[aTableColumn identifier] intValue];
+    if ((*mData)->get_column_type(columnIndex) != bec::GridModel::BlobType)
+    {
+      std::string text;
+      (*mData)->get_field_repr(rowIndex, columnIndex, text);
+      text = bec::replace_string(text, "\n", " ");
+      return [NSString stringWithCPPString: text];
+    }
+    return @"";
   }
   return nil;
 }
