@@ -275,7 +275,7 @@ objectValueForTableColumn: (NSTableColumn*) aTableColumn
     [mResultsetViews addObject: rsview];
     
     NSTabViewItem *tabViewItem= 
-    [[NSTabViewItem alloc] initWithIdentifier: [NSString stringWithFormat:@"rset%li", rset->key()]];
+        [[[NSTabViewItem alloc] initWithIdentifier: [NSString stringWithFormat:@"rset%li", rset->key()]] autorelease];
     
     [tabViewItem setLabel: [NSString stringWithCPPString: rset->caption()]];
     [tabViewItem setView: [rsview view]];
@@ -309,6 +309,12 @@ static int processTaskFinish(WBSQLQueryPanel *self)
   [self->mMessagesTable reloadData];
   [self displayResultsets];
   
+  if (self->mBackEnd->exec_sql_error_count() > 0)
+  {
+    [self->mOutputTabView selectTabViewItemWithIdentifier: @"actions"];
+    [self->mOutputSegmented selectSegmentWithTag: 0];
+  }
+  
   return 0;
 }
 
@@ -468,7 +474,7 @@ draggedHandleAtOffset: (NSPoint) offset
 
 - (void)addEditorTab
 {
-  int editor_index = mBackEnd->add_sql_editor();
+  int editor_index = mBackEnd->add_sql_editor(true);
 
   [self addEditorTabWithBackEndIndex: editor_index];  
 }
