@@ -6,6 +6,7 @@
 //  Copyright 2008 Sun Microsystems Inc. All rights reserved.
 //
 
+#import "string_utilities.h"
 #import "WBMainController.h"
 #import "WBMainWindow.h"
 #import "MCppUtilities.h"
@@ -49,7 +50,7 @@ static GThread *mainthread= 0;
 static std::string showFileDialog(const std::string &type, const std::string &title, const std::string &extensions)
 {   
   NSMutableArray *fileTypes= [NSMutableArray array];
-  std::vector<std::string> exts(bec::split_string(extensions,","));
+  std::vector<std::string> exts(base::split(extensions,","));
    
   for (std::vector<std::string>::const_iterator iter= exts.begin();
        iter != exts.end(); ++iter)
@@ -321,10 +322,10 @@ static void windowPerformCommand(const std::string &command, WBMainWindow *windo
 }
 
 
-static mdc::CanvasView* windowCreateView(const std::string &oid, const std::string &name, WBMainWindow *window)
+static mdc::CanvasView* windowCreateView(const model_DiagramRef &diagram, WBMainWindow *window)
 {  
-  return [window createView:oid.c_str()
-                       name:name.c_str()];
+  return [window createView:diagram.id().c_str()
+                       name:diagram->name().c_str()];
 }
 
 
@@ -521,11 +522,14 @@ static bool validate_cut(wb::WBContextUI *wbui)
 
 static void call_closetab(WBMainWindow *mainWindow)
 {
-  id activePanel = [mainWindow activePanel];
-  
-  if (![activePanel respondsToSelector: @selector(closeActiveEditorTab)]
-      || ![activePanel closeActiveEditorTab])
-    [mainWindow closePanel: activePanel];
+  if ([[mainWindow window] isKeyWindow])
+  {
+    id activePanel = [mainWindow activePanel];
+    
+    if (![activePanel respondsToSelector: @selector(closeActiveEditorTab)]
+        || ![activePanel closeActiveEditorTab])
+      [mainWindow closePanel: activePanel];
+  }
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -944,16 +948,19 @@ static NSString *applicationSupportFolder()
     //wboptions.open_at_startup = "";
     
     // add shipped python module search path to PYTHONPATH
+    // also include mysql utilities module zip
     {
       char *path = getenv("PYTHONPATH");
       if (path)
       {
-        path = g_strdup_printf("PYTHONPATH=%s:%s", path, _options->library_search_path.c_str());
+        path = g_strdup_printf("PYTHONPATH=%s:%s:%s/mysql_libs.zip", path, _options->library_search_path.c_str(),
+                               _options->library_search_path.c_str());
         putenv(path);  // path should not be freed
       }
       else
       {
-        path = g_strdup_printf("PYTHONPATH=%s", _options->library_search_path.c_str());
+        path = g_strdup_printf("PYTHONPATH=%s:%s/mysql_libs.zip", _options->library_search_path.c_str(),
+                               _options->library_search_path.c_str());
         putenv(path); // path should not be freed
       }
       
