@@ -420,6 +420,14 @@ static void refresh_layer_tree(const bec::NodeId &node, int old_count, WBModelDi
   return YES;
 }
 
+- (BOOL)canvasMouseDoubleClick:(mdc::MouseButton)button
+               location:(NSPoint)pos
+                  state:(mdc::EventState)state
+{
+  _formBE->handle_mouse_double_click(button, pos.x, pos.y, state);  
+  return YES;
+}
+
 - (BOOL)canvasMouseUp:(mdc::MouseButton)button
              location:(NSPoint)pos
                 state:(mdc::EventState)state
