@@ -87,6 +87,11 @@ namespace MySQL.Controls
     private Form activationTracker = null;
 
     private Color backColor = Color.FromArgb(0xff, 0x28, 0x37, 0x52);
+    private Color topHangingDarkBackgroundColor = Color.FromArgb(0xff, 0x18, 0x25, 0x3b);
+    private Color topNormalContentBackgroundColor = Color.FromArgb(0xff, 0x4a, 0x61, 0x84);
+    private Color topStyleFocusedTextColor = Color.FromArgb(0xff, 0xcc, 0xd3, 0xdf);
+    private Color topNormalFocusedTabGradientStart = Color.FromArgb(0xff, 0xff, 0xe9, 0xb6);
+
     private bool showCloseButton = true; // Central flag for close buttons. Can be overridden by each page.
     private bool showFocusState = true;
     private bool canCloseLastTab = false;
@@ -389,13 +394,13 @@ namespace MySQL.Controls
             // In top hanging style we need a background shade simulating a shadow.
             using (LinearGradientBrush gradientBrush = new LinearGradientBrush(
               new Rectangle(0, Margin.Top, 1, ItemSize.Height),
-              Color.FromArgb(255, 0x18, 0x25, 0x3b),
-              BackgroundColor, LinearGradientMode.Vertical))
+              topHangingDarkBackgroundColor, BackgroundColor, LinearGradientMode.Vertical))
               g.FillRectangle(gradientBrush, 0, Margin.Top, clientArea.Width, ItemSize.Height);
           }
           break;
+
         case TabStyleType.TopNormal:
-          // Next fill the inner rectangle (tab control area within margins but not covered by content)
+          // Fill the inner rectangle (tab control area within margins but not covered by content)
           // with the inner color but keep the top padding area the same color as that of the tab.
           if (SelectedTab != null)
           {
@@ -405,20 +410,37 @@ namespace MySQL.Controls
             innerArea.Y += Margin.Top + ItemSize.Height + ContentPadding.Top;
             innerArea.Height -= Margin.Vertical + ItemSize.Height + ContentPadding.Top;
 
-            SolidBrush brush = new SolidBrush(Color.FromArgb(0xff, 0x4a, 0x61, 0x84));
+            SolidBrush brush = new SolidBrush(topNormalContentBackgroundColor);
             g.FillRectangle(brush, innerArea);
             brush.Dispose();
 
             innerArea.Y -= ContentPadding.Top;
             innerArea.Height = ContentPadding.Top;
             if (drawFocused)
-              brush = new SolidBrush(Color.FromArgb(0xff, 0xff, 0xe9, 0xb6));
+              brush = new SolidBrush(topNormalFocusedTabGradientStart);
             else
-              brush = new SolidBrush(Color.FromArgb(0xff, 0x4a, 0x61, 0x84));
+              brush = new SolidBrush(topNormalContentBackgroundColor);
             g.FillRectangle(brush, innerArea);
             brush.Dispose();
           }
 
+          break;
+
+        case TabStyleType.BottomNormal:
+          // Fill the inner rectangle (tab control area within margins but not covered by content)
+          // with the inner color but keep the top padding area the same color as that of the tab.
+          if (SelectedTab != null)
+          {
+            Rectangle innerArea = clientArea;
+            innerArea.X += Margin.Left;
+            innerArea.Width -= Margin.Horizontal;
+            innerArea.Y += Margin.Top;
+            innerArea.Height -= Margin.Vertical + ItemSize.Height;
+
+            using (SolidBrush brush = new SolidBrush(SelectedTab.BackColor))
+              g.FillRectangle(brush, innerArea);
+          }
+
           break;
       }
 
@@ -512,7 +534,7 @@ namespace MySQL.Controls
             if (drawFocused)
               brush = new SolidBrush(Color.Black);
             else
-              brush = new SolidBrush(Color.FromArgb(255, 0xcc, 0xd3, 0xdf));
+              brush = new SolidBrush(topStyleFocusedTextColor);
           }
           else
             brush = new SolidBrush(Color.Black);
@@ -599,7 +621,9 @@ namespace MySQL.Controls
           if (drawFocused)
           {
             result.Colors = new Color[] {
-                Color.FromArgb(0xff, 0xff, 0xe9, 0xb6),
+                topNormalFocusedTabGradientStart,
+                // Other color constants depend on the actual gradient definition and are hence
+                // not worth to be kept in a class local member.
                 Color.FromArgb(0xff, 0xff, 0xe9, 0xb7),
                 Color.FromArgb(0xff, 0xff, 0xf3, 0xd9),
                 Color.FromArgb(0xff, 0xff, 0xf3, 0xd9),
@@ -614,8 +638,8 @@ namespace MySQL.Controls
           else
           {
             result.Colors = new Color[] {
-                Color.FromArgb(0xff, 0x4a, 0x61, 0x84),
-                Color.FromArgb(0xff, 0x4a, 0x61, 0x84),
+                topNormalContentBackgroundColor,
+                topNormalContentBackgroundColor,
                 Color.FromArgb(0xff, 0x3a, 0x4c, 0x68),
                 Color.FromArgb(0xff, 0x1f, 0x29, 0x37)
               };
@@ -629,7 +653,7 @@ namespace MySQL.Controls
           if (drawFocused)
           {
             result.Colors = new Color[] {
-                Color.FromArgb(0xff, 0xff, 0xe9, 0xb6),
+                topNormalFocusedTabGradientStart,
                 Color.FromArgb(0xff, 0xff, 0xe9, 0xb7),
                 Color.FromArgb(0xff, 0xff, 0xf3, 0xd9),
                 Color.FromArgb(0xff, 0xff, 0xfa, 0xf4)
@@ -642,8 +666,8 @@ namespace MySQL.Controls
           else
           {
             result.Colors = new Color[] {
-                Color.FromArgb(0xff, 0x4a, 0x61, 0x84),
-                Color.FromArgb(0xff, 0x4a, 0x61, 0x84)
+                topNormalContentBackgroundColor,
+                topNormalContentBackgroundColor
               };
 
             result.Positions = new float[] {
@@ -1177,10 +1201,9 @@ namespace MySQL.Controls
 
           OnTabClosed(new TabClosedEventArgs(page));
 
-          // this change cuases crashes
           // In the case OnTabClosed did not dispose of the page do it now.
-          //if (!page.IsDisposed)
-          //  page.Dispose();
+          if (!page.IsDisposed)
+            page.Dispose();
 
           return true;
         }
