@@ -6,6 +6,9 @@ using System.Reflection;
 using System.Threading;
 using System.Windows.Forms;
 
+using ScintillaNet;
+using Aga.Controls.Tree;
+
 using MySQL.Controls;
 using MySQL.Forms;
 using MySQL.Grt;
@@ -14,7 +17,6 @@ using MySQL.GUI.Workbench.Plugins;
 using MySQL.GUI.Workbench.Properties;
 using MySQL.Utilities;
 using MySQL.Workbench;
-using ScintillaNet;
 
 namespace MySQL.GUI.Workbench
 {
@@ -443,9 +445,9 @@ namespace MySQL.GUI.Workbench
     /// <summary>
     /// Creates a new diagram form with a given name and id. UI handling (e.g. docking) is done later.
     /// </summary>
-    public BaseWindowsCanvasView CreateNewDiagram(string viewId, string name)
+    public BaseWindowsCanvasView CreateNewDiagram(string viewId, string name, IntPtr payload)
     {
-      ModelDiagramForm modelDiagramForm = new ModelDiagramForm(wbContext, viewId);
+      ModelDiagramForm modelDiagramForm = new ModelDiagramForm(wbContext, viewId, payload);
 
       modelDiagramForm.Text = name;
       modelDiagramForm.TabText = name;
@@ -735,7 +737,10 @@ namespace MySQL.GUI.Workbench
         if (activeControl is Scintilla)
           return (activeControl as Scintilla).Selection.Length > 0;
         else
-          return wbContext.edit_can_copy();
+          if ((activeControl is TreeViewAdv) || (activeControl != null && activeControl.Parent is TreeViewAdv))
+            return false; // Returning false tells the backend not to take over the operation.
+          else
+            return wbContext.edit_can_copy();
     }
 
     private void EditCut()
@@ -760,7 +765,10 @@ namespace MySQL.GUI.Workbench
         if (activeControl is Scintilla)
           return (activeControl as Scintilla).Selection.Length > 0;
         else
-          return wbContext.edit_can_cut();
+          if ((activeControl is TreeViewAdv) || (activeControl != null && activeControl.Parent is TreeViewAdv))
+            return false; // Returning false tells the backend not to take over the operation.
+          else
+            return wbContext.edit_can_cut();
     }
 
     private void EditPaste()
@@ -1619,10 +1627,21 @@ namespace MySQL.GUI.Workbench
         else
           if (document is MySQL.Forms.AppViewDockContent)
           {
-            // This type of document is already closed (in TabClosing).
-            // TODO: adjust code to support OnClosing/OnClose duality also in AppView.
+            // Remove the appview from the page, otherwise it gets disposed with the page,
+            // what we don't want.
+            e.page.Controls.Clear();
           }
     }
 
+    /// <summary>
+    /// Restores the form to a usable state and activates it.
+    /// </summary>
+    public void Activate(bool restoreIfMinimized)
+    {
+      if (restoreIfMinimized && WindowState == FormWindowState.Minimized)
+        WindowState = FormWindowState.Normal;
+      Activate();
+    }
+
   }
 }
