@@ -56,20 +56,21 @@ namespace MySQL.GUI.Workbench
 
     #region Constructors
 
-    public ModelDiagramForm(WbContext WbContext, String id)
+    public ModelDiagramForm(WbContext WbContext, String id, IntPtr payload)
     {
       InitializeComponent();
       
       wbContext= WbContext;
 
       if (wbContext.use_opengl())
-        InitializeGLCanvas(id);
+        InitializeGLCanvas(id, payload);
       else
-        InitializeGDICanvas(id);
+        InitializeGDICanvas(id, payload);
 
       openGlCanvasViewer.CanvasPanel.MouseMove += new MouseEventHandler(CanvasPanel_MouseMove);
       openGlCanvasViewer.CanvasPanel.MouseDown += new MouseEventHandler(CanvasPanel_MouseDown);
       openGlCanvasViewer.CanvasPanel.MouseUp += new MouseEventHandler(CanvasPanel_MouseUp);
+      openGlCanvasViewer.CanvasPanel.MouseDoubleClick += new MouseEventHandler(CanvasPanel_MouseDoubleClick);
       openGlCanvasViewer.CanvasPanel.KeyDown += new KeyEventHandler(CanvasPanel_KeyDown);
       openGlCanvasViewer.CanvasPanel.KeyUp += new KeyEventHandler(CanvasPanel_KeyUp);
       openGlCanvasViewer.CanvasPanel.MouseLeave += new EventHandler(CanvasPanel_MouseLeave);
@@ -328,6 +329,12 @@ namespace MySQL.GUI.Workbench
       formBE.OnMouseDown(e, p.X, p.Y, ModifierKeys, MouseButtons);
     }
 
+    void CanvasPanel_MouseDoubleClick(object sender, MouseEventArgs e)
+    {
+      Point p = e.Location;
+      formBE.OnMouseDoubleClick(e, p.X, p.Y, ModifierKeys, MouseButtons);
+    }
+
     void CanvasPanel_MouseMove(object sender, MouseEventArgs e)
     {
       Point p = e.Location;
@@ -503,18 +510,18 @@ namespace MySQL.GUI.Workbench
       }
     }
 
-    public BaseWindowsCanvasView InitializeGLCanvas(String id)
+    public BaseWindowsCanvasView InitializeGLCanvas(String id, IntPtr payload)
     {
-      BaseWindowsCanvasView canvas = openGlCanvasViewer.InitializeGLCanvas(this, false);
+      BaseWindowsCanvasView canvas = openGlCanvasViewer.InitializeGLCanvas(this, payload, false);
 
       formBE = wbContext.get_diagram_form_for_diagram(id);
 
       return canvas;
     }
 
-    public BaseWindowsCanvasView InitializeGDICanvas(String id)
+    public BaseWindowsCanvasView InitializeGDICanvas(String id, IntPtr payload)
     {
-      BaseWindowsCanvasView canvas = openGlCanvasViewer.InitializeGDICanvas(this, false);
+      BaseWindowsCanvasView canvas = openGlCanvasViewer.InitializeGDICanvas(this, payload, false);
 
       formBE = wbContext.get_diagram_form_for_diagram(id);
 
