@@ -1,18 +1,18 @@
 /* 
  * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301  USA
  */
@@ -1215,7 +1215,7 @@ void LineDiagramWidget::repaint(cairo_t *cr, int areax, int areay, int areaw, in
 
   cairo_push_group(cr); // Prepare composing.
   
-  if (_sleep_mode != Sleeping)
+  //if (_sleep_mode != Sleeping)
   {
     cairo_set_line_width(cr, 1);
     cairo_set_source(cr, _value_gradient);
@@ -1422,14 +1422,14 @@ void LineDiagramWidget::get_minmax_values(double* min, double* max)
 
 //--------------------------------------------------------------------------------------------------
 
-#define DATA_TIMEOUT 5 // 5 seconds after which the diagram will go to sleep if no data comes in.
+#define DATA_TIMEOUT 15 // 15 seconds after which the diagram will go to sleep if no data comes in.
 
 void LineDiagramWidget::step()
 {
   double timestamp= g_timer_elapsed(_clock, NULL);
   
   bool needs_repaint= false;
-  if (timestamp - _last_shift >= 0.5)
+  if (_sleep_mode == Awake && timestamp - _last_shift >= 0.5)
   {
     // Two shifts per second.
     _last_shift= timestamp;
@@ -1453,8 +1453,10 @@ void LineDiagramWidget::step()
   {
     // We are sleeping. Check if we can awake.
     if (_sleep_mode == Sleeping && (timestamp - _timestamp[LINE_SERIES_DATA_SIZE - 1] < DATA_TIMEOUT))
+    {
       end_sleeping(timestamp);
-    needs_repaint= true;
+      needs_repaint= true;
+    }
   }
   if (feedback_step())
     needs_repaint= true;
@@ -1543,20 +1545,21 @@ bool LineDiagramWidget::feedback_step()
         if (timestamp - _sleep_start < 3)
         {
           // Compute content alpha value.
-          _content_alpha= 0.5 * (1 + cos(time_point));
+          // Minimum value is 25%. Divide by 2 as we shift the coordinate system 1 up (to
+          // have no negative values).
+          _content_alpha= 0.25 + 0.75 * 0.5 * (1 + cos(time_point));
         }
         else
           _sleep_mode= Sleeping;
-        // Fall through.
-      case Sleeping:
-          // Same for warning alpha.
+
+        // Same computation for warning alpha (just the opposite direction).
         _warning_alpha= 0.25 + 0.75 * 0.5 * (1 + sin(time_point - M_PI / 2));
         
         // Just in case the GoSleeping phase could not be executed (the timer thread could not
         // trigger within the first second of the animation cycle) check the content alpha value
         // and set it to its endpoint to have a proper display.
-        if (_sleep_mode == Sleeping && _content_alpha > 0)
-          _content_alpha= 0;
+        if (_sleep_mode == Sleeping && _content_alpha > 0.25)
+          _content_alpha= 0.25;
         
         result= true;
         break;
