@@ -1,22 +1,23 @@
 /* 
- * (c) 2009-2010 Sun Microsystems, Inc.
+ * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
-/**
+ 
+ /**
  * Implementation of the treeview wrapper for the mforms library. We use a third-party tree view control
  * here (TreeViewAdv) which supports columns.
  */
@@ -27,43 +28,34 @@
 
 using namespace MySQL;
 using namespace MySQL::Forms;
+using namespace MySQL::Utilities;
 
-//--------------------------------------------------------------------------------------------------
+//----------------- TreeNode -----------------------------------------------------------------------
 
 MySQL::Forms::TreeNode::TreeNode(TreeViewAdv^ tree)
   : _tree(tree)
 {
 }
 
-//--------------------------------------------------------------------------------------------------
+//----------------- ColumnComparer -----------------------------------------------------------------
 
-/**
- * Returns the text for a column given by index.
- *
- * @param index The index of the column for which the text is to be returned. If this index is not in the range of
- *              stored values then an empty string is returned (sparse list).
- * @return The requested string or "" if the index is out of range.
- */
-String^ MySQL::Forms::TreeNode::GetText(int index)
+ColumnComparer::ColumnComparer(int column, SortOrder direction)
 {
-  if (index < 0 || index >= _text.Count)
-    return "";
-  return _text[index] == nullptr ? "" : _text[index];
+  _column = column;
+  _direction = direction;
+  _inner = gcnew CaseInsensitiveComparer();
 }
 
 //--------------------------------------------------------------------------------------------------
 
-/**
- * Stores the given text at the given index for later retrieval.
- *
- * @param index The index at which to store the text (corresponds to a column in the tree). Must be >= 0. 
- *              The index can larger than the current count in the list. If so a number of empty entries are added.
- */
-void MySQL::Forms::TreeNode::SetText(int index, String^ text)
+int ColumnComparer::Compare(System::Object^ object1, System::Object^ object2)
 {
-  while (index >= _text.Count)
-    _text.Add(nullptr);
-  _text[index]= text;
+  TreeNode^ node1 = dynamic_cast<TreeNode^>(object1);
+  TreeNode^ node2 = dynamic_cast<TreeNode^>(object2);
+  if (_direction == SortOrder::Ascending)
+    return _inner->Compare(node1->Text[_column], node2->Text[_column]);
+  else
+    return _inner->Compare(node2->Text[_column], node1->Text[_column]);
 }
 
 //----------------- TreeViewImpl static interface functions ----------------------------------------
@@ -83,6 +75,9 @@ bool TreeViewImpl::create(::mforms::TreeView *self, ::mforms::TreeOptions option
     t->ShowLines= options & ::mforms::TreeShowLines ? true : false;
     t->ShowPlusMinus= options & ::mforms::TreeShowPlusMinus ? true : false;
     t->MouseDown += gcnew MouseEventHandler(&TreeViewImpl::OnMouseDown);
+    t->ColumnClicked += gcnew System::EventHandler<TreeColumnEventArgs^>(treeview, &TreeViewImpl::OnColumnClicked);
+    if (options & ::mforms::TreeNoBorder) // Default is 3d border.
+      t->BorderStyle = BorderStyle::None;
     return true;
   }
   return false;
@@ -157,6 +152,15 @@ void TreeViewImpl::set_selected(::mforms::TreeView *self, const int idx)
 
 //--------------------------------------------------------------------------------------------------
 
+void TreeViewImpl::set_allow_sorting(mforms::TreeView *self, bool flag)
+{
+  TreeViewImpl^ treeview= (TreeViewImpl^)ObjectImpl::FromUnmanaged(self);
+  if (treeview != nullptr)
+    treeview->allow_column_sorting(flag);
+}
+
+//--------------------------------------------------------------------------------------------------
+
 int TreeViewImpl::count(::mforms::TreeView *self)
 {
   TreeViewImpl^ treeview= (TreeViewImpl^)ObjectImpl::FromUnmanaged(self);
@@ -241,12 +245,24 @@ bool TreeViewImpl::get_check(::mforms::TreeView *self, int row, int column)
   return false;
 }
 
+//--------------------------------------------------------------------------------------------------
+
+void TreeViewImpl::freeze_refresh(mforms::TreeView *self, bool flag)
+{
+  TreeViewImpl^ treeview= (TreeViewImpl^)ObjectImpl::FromUnmanaged(self);
+  if (treeview != nullptr)
+    return treeview->freeze_refresh(flag);
+}
+
 //----------------- Actual implementation ----------------------------------------------------------
 
 TreeViewImpl::TreeViewImpl(::mforms::TreeView *self)
   : ViewImpl(self)
 {
-  _model= gcnew Aga::Controls::Tree::TreeModel();
+  _model = gcnew SortableTreeModel();
+  _current_sort_column = -1;
+  _can_sort_column = false;
+  _current_sort_order = SortOrder::None;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -305,7 +321,7 @@ void TreeViewImpl::TreeValueNeeded(System::Object^ sender, NodeControlValueEvent
   // The passed in TreeNodeAdv can be the hidden root node, so better check it.
   if (ourNode != nullptr)
   {
-    String^ value= ourNode->GetText(control->ParentColumn->Index);
+    String^ value= ourNode->Text[control->ParentColumn->Index];
     if (control->GetType() == NodeCheckBox::typeid)
       e->Value = (value == "Checked") || (value == "1") ? true : false;
     else
@@ -338,7 +354,7 @@ void TreeViewImpl::TreeValuePushed(System::Object^ sender, NodeControlValueEvent
   {
     // Backend says ok, so update the model.
     if (ourNode != nullptr)
-      ourNode->SetText(control->ParentColumn->Index, e->Value->ToString());
+      ourNode->Text[control->ParentColumn->Index] = e->Value->ToString();
   }
 }
 
@@ -384,7 +400,7 @@ void TreeViewImpl::OnMouseDown(System::Object^ sender, MouseEventArgs^ e)
   {
     ::mforms::TreeView* t= ViewImpl::get_backend_control<::mforms::TreeView>(tree);
 
-    // udpate the associated context menu
+    // update the associated context menu
     if (t && t->get_context_menu())
     {
       ContextMenu^ menu= MenuImpl::get(t->get_context_menu());
@@ -401,6 +417,40 @@ void TreeViewImpl::OnMouseDown(System::Object^ sender, MouseEventArgs^ e)
 
 //--------------------------------------------------------------------------------------------------
 
+void TreeViewImpl::OnColumnClicked(System::Object^ sender, TreeColumnEventArgs^ e)
+{
+  if (_can_sort_column)
+  {
+    // Note: TreeViewAdv^ uses sort indicators wrongly. They are used as if they were arrows pointing
+    //       to the greater value. However common usage is to view them as normal triangles with the
+    //       tip showing the smallest value and the base side the greatest (Windows explorer and many
+    //       other controls use it that way). Hence I switch the order here for the columns
+    //       (though not for the comparer).
+    TreeViewAdv^ tree= (TreeViewAdv^)sender;
+    if (e->Column->Index != _current_sort_column)
+    {
+      _current_sort_column = e->Column->Index;
+
+      // Initial sort order is always ascending.
+      e->Column->SortOrder = SortOrder::Descending;
+      _current_sort_order = SortOrder::Ascending;
+    }
+    else
+    {
+      if (e->Column->SortOrder == SortOrder::Ascending)
+        e->Column->SortOrder = SortOrder::Descending;
+      else
+        e->Column->SortOrder = SortOrder::Ascending;
+
+      // Revert the sort order for our comparer, read above why.
+      _current_sort_order = (e->Column->SortOrder == SortOrder::Ascending) ? SortOrder::Descending : SortOrder::Ascending;
+    }
+    _model->Comparer = gcnew ColumnComparer(_current_sort_column, _current_sort_order);
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
 void TreeViewImpl::end_columns()
 {
 }
@@ -446,13 +496,25 @@ void TreeViewImpl::set_selected(const int idx)
   TreeNodeAdv^ node= tree->Root;
 
   // Currently the tree is managed as a simple list with columns. Hence setting the selected index
-  // means to select the index'ed child node of the root node.
+  // means to select the indexed child node of the root node.
   if (tree->Root->Children->Count > idx)
     tree->SelectedNode= tree->Root->Children[idx];
 }
 
 //--------------------------------------------------------------------------------------------------
 
+void TreeViewImpl::allow_column_sorting(bool flag)
+{
+  _can_sort_column = flag;
+  if (_current_sort_column > -1)
+  {
+    TreeViewAdv^ tree= get_control<TreeViewAdv>();
+    tree->Columns[_current_sort_column]->SortOrder = _current_sort_order;
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
 int TreeViewImpl::count()
 {
   return _model->Nodes->Count;
@@ -471,7 +533,7 @@ void TreeViewImpl::set_row_tag(int row, const string &value)
 void TreeViewImpl::set_string(int row, int column, const string &value)
 {
   MySQL::Forms::TreeNode^ node= (MySQL::Forms::TreeNode^) _model->Nodes[row];
-  node->SetText(column, CppStringToNative(value));
+  node->Text[column] = CppStringToNative(value);
 
   get_control<TreeViewAdv>()->Invalidate(); // Might lead to flickering, investigate if necessary.
 }
@@ -481,7 +543,7 @@ void TreeViewImpl::set_string(int row, int column, const string &value)
 void TreeViewImpl::set_int(int row, int column, int value)
 {
   MySQL::Forms::TreeNode^ node= (MySQL::Forms::TreeNode^) _model->Nodes[row];
-  node->SetText(column, Convert::ToString(value));
+  node->Text[column] = Convert::ToString(value);
 
   get_control<TreeViewAdv>()->Invalidate();
 }
@@ -491,7 +553,7 @@ void TreeViewImpl::set_int(int row, int column, int value)
 void TreeViewImpl::set_check(int row, int column, bool value)
 {
   MySQL::Forms::TreeNode^ node= (MySQL::Forms::TreeNode^) _model->Nodes[row];
-  node->SetText(column , value ? "1" : "0");
+  node->Text[column] = value ? "1" : "0";
 
   get_control<TreeViewAdv>()->Invalidate();
 }
@@ -517,7 +579,7 @@ string TreeViewImpl::get_string(int row, int column)
   if (_model->Nodes->Count > row && row >= 0)
   {
     MySQL::Forms::TreeNode^ node= (MySQL::Forms::TreeNode^) _model->Nodes[row];
-    ret = NativeToCppString(node->GetText(column));
+    ret = NativeToCppString(node->Text[column]);
   }
   return ret;
 }
@@ -527,7 +589,7 @@ string TreeViewImpl::get_string(int row, int column)
 int TreeViewImpl::get_int(int row, int column)
 {
   MySQL::Forms::TreeNode^ node= (MySQL::Forms::TreeNode^) _model->Nodes[row];
-  return Convert::ToInt32(node->GetText(column));
+  return Convert::ToInt32(node->Text[column]);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -535,8 +597,21 @@ int TreeViewImpl::get_int(int row, int column)
 bool TreeViewImpl::get_check(int row, int column)
 {
   MySQL::Forms::TreeNode^ node= (MySQL::Forms::TreeNode^) _model->Nodes[row];
-  return (node->GetText(column) == "Checked") || (node->GetText(column) == "1") ? true : false;
+  return (node->Text[column] == "Checked") || (node->Text[column] == "1") ? true : false;
 }
 
 //--------------------------------------------------------------------------------------------------
 
+void TreeViewImpl::freeze_refresh(bool flag)
+{
+  TreeViewAdv^ tree= get_control<TreeViewAdv>();
+  if (flag)
+    ControlUtilities::SuspendDrawing(tree);
+  else
+  {
+    ControlUtilities::ResumeDrawing(tree);
+    tree->Refresh();
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
