@@ -1,18 +1,18 @@
 /* 
  * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301  USA
  */
@@ -26,6 +26,7 @@ using namespace System::Drawing;
 using namespace MySQL;
 using namespace MySQL::Forms;
 using namespace MySQL::Utilities;
+using namespace MySQL::Controls;
 
 using namespace Aga::Controls::Tree;
 using namespace ScintillaNet;
@@ -64,7 +65,10 @@ bool ViewImpl::use_min_width_for_layout(Control^ control)
     System::Type^ type = control->GetType();
     needsMinWidth= (type == TabControl::typeid) || (type == Panel::typeid) || (type == ComboBox::typeid)
       || (type == TreeViewAdv::typeid) || (type == ListBox::typeid) || (type == TextBox::typeid)
-      || (type == SplitContainer::typeid) || (type == Scintilla::typeid);
+      || (type == SplitContainer::typeid) || (type == Scintilla::typeid)
+      
+      // TODO: investigate if we can remove this and override GetPreferredSize in FlatTabControl instead.
+      || (type == FlatTabControl::typeid);
   }
 
   return needsMinWidth;
@@ -80,7 +84,10 @@ bool ViewImpl::use_min_height_for_layout(Control^ control)
   {
     System::Type^ type = control->GetType();
     needsMinHeight= (type == TabControl::typeid) || (type == Panel::typeid) || (type == ListBox::typeid)
-      || (type == TreeViewAdv::typeid) || (type == SplitContainer::typeid) || (type == Scintilla::typeid);
+      || (type == TreeViewAdv::typeid) || (type == SplitContainer::typeid) || (type == Scintilla::typeid)
+
+    // TODO: investigate if we can remove this and override GetPreferredSize in FlatTabControl instead.
+    || (type == FlatTabControl::typeid);
   }
 
   return needsMinHeight;
@@ -291,10 +298,7 @@ void ViewImpl::set_size(::mforms::View *self, int w, int h)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
-  {
-    self->set_layout_dirty(true);
     view->set_size(w, h);
-  }
 }
 
 //-------------------------------------------------------------------------------------------------
@@ -487,11 +491,10 @@ void ViewImpl::set_size(int width, int height)
     newSize.Width= width;
   if (height >= 0)
     newSize.Height= height;
-  if (!control->Size.Equals(newSize))
-  {
-    control->MinimumSize = newSize;
-    control->Size = newSize;
-  }
+  
+  control->MinimumSize = newSize; // TODO: wrong place for this, when we restore loaded settings we don't want to set a min size.
+                                  //       Implement an own set_min_size function.
+  control->Size = newSize;
 }
 
 //-------------------------------------------------------------------------------------------------
@@ -514,7 +517,13 @@ void ViewImpl::set_position(int x, int y)
 void ViewImpl::set_tooltip(const std::string& text)
 {
   if (tooltip == nullptr)
+  {
     tooltip= gcnew ToolTip();
+    tooltip->AutoPopDelay = 10000; // 10 sec. show time
+    tooltip->InitialDelay = 500;   // 500 ms before showing the tooltip if none was visible before.
+    tooltip->ReshowDelay = 250;    // 250 ms before showing the tooltip if another one was active.
+    tooltip->ShowAlways = true;    // Show tooltip even if the control is not active.
+  }
   tooltip->SetToolTip(get_control<Control>(), CppStringToNative(text));
 }
 
@@ -522,7 +531,19 @@ void ViewImpl::set_tooltip(const std::string& text)
 
 void ViewImpl::set_font(const std::string& fontDescription)
 {
-  get_control<Control>()->Font = ControlUtilities::getFont(CppStringToNativeRaw(fontDescription));
+  std::string font;
+  int size;
+  bool bold;
+  bool italic;
+  base::parse_font_description(fontDescription, font, size, bold, italic);
+
+  FontStyle style = FontStyle::Regular;
+  if (bold)
+    style = (FontStyle) (style | FontStyle::Bold);
+  if (italic)
+    style = (FontStyle) (style | FontStyle::Italic);
+  get_control<Control>()->Font = ControlUtilities::getFont(CppStringToNativeRaw(font),
+    (float) size, style);
 }
 
 //-------------------------------------------------------------------------------------------------
@@ -590,19 +611,36 @@ void ViewImpl::resize_with_docking(Control^ control, Drawing::Size& size)
     return;
 
   if (control->Dock == DockStyle::Left || control->Dock == DockStyle::Right)
-    size.Height= control->ClientSize.Height;
+    size.Height= control->Size.Height;
   else
     if (control->Dock == DockStyle::Top || control->Dock == DockStyle::Bottom)
-      size.Width= control->ClientSize.Width;
+      size.Width= control->Size.Width;
 
-  control->ClientSize= size;
+  control->Size= size;
 }
 
 //-------------------------------------------------------------------------------------------------
 
 /**
+ * Removes auto resizing flags if a control is limited in resizing by docking.
  */
+void ViewImpl::adjust_auto_resize_from_docking(Control^ control)
+{
+  if (control->Dock == DockStyle::Fill)
+    set_auto_resize(control, mforms::ResizeNone);
+  else
+    if (control->Dock == DockStyle::Left || control->Dock == DockStyle::Right)
+      remove_auto_resize(control, mforms::ResizeHorizontal);
+    else
+      if (control->Dock == DockStyle::Top || control->Dock == DockStyle::Bottom)
+        remove_auto_resize(control, mforms::ResizeVertical);
+}
+
+//-------------------------------------------------------------------------------------------------
+
 void ViewImpl::flush_events(::mforms::View *)
 {
   Application::DoEvents();
 }
+
+//-------------------------------------------------------------------------------------------------
