@@ -912,18 +912,20 @@ void Object::__set_id(const std::string &id)
 
 bool process_reset_references_for_member(const MetaClass::Member* m, Object* obj)
 {
-  if ( m && m->owned_object )
+  if (m && !m->calculated && !grt::is_simple_type(m->type.base.type))
   {
     //g_log("grt", G_LOG_LEVEL_DEBUG, "\tprocess_reset_references_for_member'%s':'%s':'%s'", obj->class_name().c_str(), obj->id().c_str(), m->name.c_str());
-    
     grt::ValueRef member_value = obj->get_member(m->name);
     if ( member_value.is_valid() )
     {
-      member_value.valueptr()->reset_references();
+      // if the member is owned, then recursively reset references in it
+      if (m->owned_object)
+        member_value.valueptr()->reset_references();
+      
+      // set the member value to null
       obj->get_metaclass()->set_member_internal(obj, m->name, grt::ValueRef(), true);
     }
   }
-  
   return true;
 }
 
