@@ -1,20 +1,20 @@
 /* 
  * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
- * Boston, MA 02110-1301  USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
  
@@ -97,9 +97,12 @@ PythonContext::PythonContext(GRT *grt)
   PyDict_SetItemString(PyModule_GetDict(main),
                        "grt", PyImport_ImportModule("grt"));
   
-  // make sys.stdout and sys.stderr send stuff to GRT  
+  // make sys.stdout and sys.stderr send stuff to GRT
   PySys_SetObject((char*)"stdout", get_grt_module());
   PySys_SetObject((char*)"stderr", get_grt_module());
+  
+  // set stdin to the GRT shell console
+  PySys_SetObject((char*)"stdin", get_grt_module());
 
   PyEval_SaveThread();
 }
@@ -137,21 +140,21 @@ void PythonContext::add_module_path(const std::string &modpath)
 }
 
 
-void PythonContext::set_python_error(const grt::type_error &exc)
+void PythonContext::set_python_error(const grt::type_error &exc, const std::string &location)
 {
-  PyErr_SetString(PyExc_TypeError, exc.what());
+  PyErr_SetString(PyExc_TypeError, (location.empty() ? exc.what() : location + ": " + exc.what()).c_str());
 }
 
 
-void PythonContext::set_python_error(const grt::bad_item &exc)
+void PythonContext::set_python_error(const grt::bad_item &exc, const std::string &location)
 {
-  PyErr_SetString(PyExc_IndexError, exc.what());
+  PyErr_SetString(PyExc_IndexError, (location.empty() ? exc.what() : location + ": " + exc.what()).c_str());
 }
 
 
-void PythonContext::set_python_error(const std::exception &exc)
+void PythonContext::set_python_error(const std::exception &exc, const std::string &location)
 {
-  PyErr_SetString(PyExc_SystemError, exc.what());
+  PyErr_SetString(PyExc_SystemError, (location.empty() ? exc.what() : location + ": " + exc.what()).c_str());
 }
 
 
@@ -200,18 +203,30 @@ PythonContext *PythonContext::get_and_check()
 static PyObject *grt_print(PyObject *self, PyObject *args)
 {
   PythonContext *ctx;
-  const char *text= "";
+  std::string text;
     
   if (!(ctx= PythonContext::get_and_check()))
     return NULL;
-  
-  if (!PyArg_ParseTuple(args, "z", &text))
-    return NULL;
 
+  PyObject *o;
+  if (!PyArg_ParseTuple(args, "O", &o))
+  {
+    if (PyTuple_Size(args) == 1 && PyTuple_GetItem(args, 0) == Py_None)
+    {
+      PyErr_Clear();
+      text = "None";
+    }
+    else
+      return NULL;
+  }  
+  else
+    if (!ctx->pystring_to_string(o, text))
+      return NULL;
+  
 #ifdef _WIN32
-  OutputDebugStringA(text);
+  OutputDebugStringA(text.c_str());
 #else
-  g_print("%s", text); // g_print is not routed to g_log
+  g_print("%s", text.c_str()); // g_print is not routed to g_log
 #endif
   ctx->get_grt()->send_output(text);
   
@@ -223,13 +238,25 @@ static PyObject *grt_print(PyObject *self, PyObject *args)
 static PyObject *grt_send_output(PyObject *self, PyObject *args)
 {
   PythonContext *ctx;
-  const char *text= "";
+  std::string text;
     
   if (!(ctx= PythonContext::get_and_check()))
     return NULL;
   
-  if (!PyArg_ParseTuple(args, "z", &text))
-    return NULL;
+  PyObject *o;
+  if (!PyArg_ParseTuple(args, "O", &o))
+  {
+    if (PyTuple_Size(args) == 1 && PyTuple_GetItem(args, 0) == Py_None)
+    {
+      PyErr_Clear();
+      text = "None";
+    }
+    else
+      return NULL;
+  }  
+  else
+    if (!ctx->pystring_to_string(o, text))
+      return NULL;
 
   ctx->get_grt()->send_output(text);
   
@@ -241,13 +268,25 @@ static PyObject *grt_send_output(PyObject *self, PyObject *args)
 static PyObject *grt_send_warning(PyObject *self, PyObject *args)
 {
   PythonContext *ctx;
-  const char *text= "";
+  std::string text;
   if (!(ctx= PythonContext::get_and_check()))
     return NULL;
   
-  if (!PyArg_ParseTuple(args, "z", &text))
-    return NULL;
-  
+  PyObject *o;
+  if (!PyArg_ParseTuple(args, "O", &o))
+  {
+    if (PyTuple_Size(args) == 1 && PyTuple_GetItem(args, 0) == Py_None)
+    {
+      PyErr_Clear();
+      text = "None";
+    }
+    else
+      return NULL;
+  }  
+  else
+    if (!ctx->pystring_to_string(o, text))
+      return NULL;
+
   ctx->get_grt()->send_warning(text);
   
   Py_INCREF(Py_None);
@@ -257,13 +296,25 @@ static PyObject *grt_send_warning(PyObject *self, PyObject *args)
 static PyObject *grt_send_info(PyObject *self, PyObject *args)
 {
   PythonContext *ctx;
-  const char *text= "";
+  std::string text;
   if (!(ctx= PythonContext::get_and_check()))
     return NULL;
   
-  if (!PyArg_ParseTuple(args, "z", &text))
-    return NULL;
-  
+  PyObject *o;
+  if (!PyArg_ParseTuple(args, "O", &o))
+  {
+    if (PyTuple_Size(args) == 1 && PyTuple_GetItem(args, 0) == Py_None)
+    {
+      PyErr_Clear();
+      text = "None";
+    }
+    else
+      return NULL;
+  }  
+  else
+    if (!ctx->pystring_to_string(o, text))
+      return NULL;
+
   ctx->get_grt()->send_info(text);
   
   Py_INCREF(Py_None);
@@ -273,13 +324,25 @@ static PyObject *grt_send_info(PyObject *self, PyObject *args)
 static PyObject *grt_send_error(PyObject *self, PyObject *args)
 {
   PythonContext *ctx;
-  const char *text= "";
+  std::string text;
   if (!(ctx= PythonContext::get_and_check()))
     return NULL;
   
-  if (!PyArg_ParseTuple(args, "z", &text))
-    return NULL;
-  
+  PyObject *o;
+  if (!PyArg_ParseTuple(args, "O", &o))
+  {
+    if (PyTuple_Size(args) == 1 && PyTuple_GetItem(args, 0) == Py_None)
+    {
+      PyErr_Clear();
+      text = "None";
+    }
+    else
+      return NULL;
+  }  
+  else
+    if (!ctx->pystring_to_string(o, text))
+      return NULL;
+
   ctx->get_grt()->send_error(text);
   
   Py_INCREF(Py_None);
@@ -336,6 +399,21 @@ static PyObject *grt_get_by_path(PyObject *self, PyObject *args)
 }
 
 
+static PyObject *grt_readline(PyObject *self, PyObject *args)
+{
+  PythonContext *ctx;
+  
+  if (!(ctx= PythonContext::get_and_check()))
+    return NULL;
+  
+  if (!PyArg_ParseTuple(args, ""))
+    return NULL;
+  
+  std::string line = ctx->stdin_readline_slot();
+  return Py_BuildValue("s", line.c_str());
+}
+
+
 /** Register grt related functionality as a module
 
  Stuff made available in the module include:
@@ -356,6 +434,9 @@ static PyMethodDef GrtModuleMethods[] = {
 {"send_info",  grt_send_info, METH_VARARGS,
 "Write a info message to the GRT shell."},
 
+{"readline", grt_readline, METH_VARARGS,
+"Waits for a line of text to be input to the scripting shell prompt."},
+
 {"get", grt_get_by_path, METH_VARARGS,
 "Gets a value from a GRT dict or object (or from the global tree) by path."},
 
@@ -389,12 +470,12 @@ void PythonContext::register_grt_module()
   init_grt_object_type();
   
   _grt_modules_module = Py_InitModule("grt.modules", NULL);
-  if (_grt_modules_module == NULL)
+  if (!_grt_modules_module)
     throw std::runtime_error("Error initializing grt.modules module in Python support");
   PyModule_AddObject(_grt_module, "modules", _grt_modules_module);
   
   _grt_classes_module = Py_InitModule("grt.classes", NULL);
-  if (_grt_classes_module == NULL)
+  if (!_grt_classes_module)
     throw std::runtime_error("Error initializing grt.classes module in Python support");
   PyModule_AddObject(_grt_module, "classes", _grt_classes_module);
   
@@ -465,6 +546,28 @@ PyObject *PythonContext::get_global(const std::string &value)
 }
 
 
+bool PythonContext::set_global(const std::string &name, PyObject *value)
+{
+  PyObject *mainmod= PyImport_AddModule("__main__");
+  if (!mainmod)
+  {
+    PyErr_Print();
+    PyErr_Clear();
+    return false;
+  }
+  PyObject *globals= PyModule_GetDict(mainmod);
+  if (!globals)
+  {
+    PyErr_Print();
+    PyErr_Clear();
+    return false;
+  }
+
+  PyDict_SetItemString(globals, name.c_str(), value); 
+  return true;
+}
+
+
 static void release_value(void *value, void *desc)
 {
   internal::Value *v= reinterpret_cast<internal::Value*>(value);
