@@ -1,20 +1,20 @@
 /* 
- * (c) 2009-2010 Sun Microsystems, Inc.
+ * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
 #include "stdafx.h"
@@ -37,6 +37,7 @@
 #include "db_mysql_catalog_report.h"
 #include "string_utilities.h"
 
+#include "grtsqlparser/sql_specifics.h"
 #include "sqlide/recordset_table_inserts_storage.h"
 #include "sqlide/recordset_be.h"
 
@@ -434,6 +435,7 @@ class ActionGenerateSQL : public DiffSQLGeneratorBEActionInterface
   size_t empty_length;
   bool first_column, first_index,
   first_change, first_fk_create, first_fk_drop;
+  std::string _non_std_sql_delimiter;
 
   std::string fk_add_sql;
   std::string fk_drop_sql;
@@ -452,7 +454,7 @@ class ActionGenerateSQL : public DiffSQLGeneratorBEActionInterface
   void alter_table_property(std::string& to, const std::string& name, const std::string& value);
 
 public:
-  ActionGenerateSQL(grt::ValueRef target, grt::ListRef<GrtNamedObject> obj_list);
+  ActionGenerateSQL(grt::ValueRef target, grt::ListRef<GrtNamedObject> obj_list,grt::GRT *grt);
   virtual ~ActionGenerateSQL();
 
   // create table
@@ -577,9 +579,13 @@ public:
   inline std::string get_name(GrtNamedObjectRef object) const{return ::get_name(object,_use_short_names);};
 };
 
-ActionGenerateSQL::ActionGenerateSQL(grt::ValueRef target, grt::ListRef<GrtNamedObject> obj_list)
+ActionGenerateSQL::ActionGenerateSQL(grt::ValueRef target, grt::ListRef<GrtNamedObject> obj_list,grt::GRT *grt)
   : padding(2)
 {
+  SqlFacade::Ref sql_facade = SqlFacade::instance_for_rdbms_name(grt, "Mysql");
+  Sql_specifics::Ref sql_specifics = sql_facade ->sqlSpecifics();
+  _non_std_sql_delimiter = sql_specifics->non_std_sql_delimiter();
+
   if(target.type() == DictType)
   {
     this->target_list= grt::StringListRef();
@@ -1320,7 +1326,7 @@ void ActionGenerateSQL::create_trigger(db_mysql_TriggerRef trigger, bool for_alt
 {
   std::string trigger_sql;
   if (!_use_short_names || _gen_use)
-    trigger_sql.append("USE `").append(trigger->owner()->owner()->name().c_str()).append("`$$\n");
+    trigger_sql.append("USE `").append(trigger->owner()->owner()->name().c_str()).append("`").append(_non_std_sql_delimiter).append("\n");
   trigger_sql.append(trigger->sqlDefinition().c_str());
 
   if(for_alter)
@@ -1333,7 +1339,7 @@ void ActionGenerateSQL::drop_trigger(db_mysql_TriggerRef trigger, bool for_alter
 {
   std::string trigger_sql;
   if (!_use_short_names || _gen_use)
-    trigger_sql.append("USE `").append(trigger->owner()->owner()->name().c_str()).append("`$$\n");
+    trigger_sql.append("USE `").append(trigger->owner()->owner()->name().c_str()).append("`").append(_non_std_sql_delimiter).append("\n");
   trigger_sql.append("DROP TRIGGER IF EXISTS ").append(get_name(trigger)).append(" ");
   if(for_alter)
     remember_alter(trigger, trigger_sql);
@@ -1401,13 +1407,14 @@ void ActionGenerateSQL::create_routine(db_mysql_RoutineRef routine, bool for_alt
 
   std::string routine_sql;
 //  if(!_use_short_names)
-  routine_sql = "\nDELIMITER $$\n";
+  routine_sql = "\nDELIMITER ";
+  routine_sql.append(_non_std_sql_delimiter).append("\n");
   if (!_use_short_names || _gen_use)
   {
     routine_sql.append("USE `");
-    routine_sql.append(routine->owner()->name()).append("`$$\n");
+    routine_sql.append(routine->owner()->name()).append("`").append(_non_std_sql_delimiter).append("\n");
   }
-  routine_sql.append(routine->sqlDefinition().c_str()).append("$$\n");
+  routine_sql.append(routine->sqlDefinition().c_str()).append(_non_std_sql_delimiter).append("\n");
 
   if (_use_short_names)
   {
@@ -1548,12 +1555,12 @@ int DbMySQLImpl::generateSQL(GrtNamedObjectRef org_object,
   
   if(result.type() == DictType)
   {
-    DiffSQLGeneratorBE(options, new ActionGenerateSQL(result, obj_list))
+    DiffSQLGeneratorBE(options, new ActionGenerateSQL(result, obj_list, get_grt()))
       .process_diff_change(org_object, p, grt::DictRef::cast_from(result));
   }
   else if(result.type() == ListType)
   {
-    DiffSQLGeneratorBE(options, new ActionGenerateSQL(result, obj_list))
+    DiffSQLGeneratorBE(options, new ActionGenerateSQL(result, obj_list, get_grt()))
       .process_diff_change(org_object, p, grt::StringListRef::cast_from(result), obj_list);
   }
 
@@ -1622,10 +1629,17 @@ static std::string generate_view_placeholder(db_mysql_ViewRef view, const bool s
     sql.append("`id` INT");
   else
   {
+    std::vector<std::string> used_colnames;
+    used_colnames.reserve(select_statement->select_items.size());
     SelectItems::const_iterator it = select_statement->select_items.begin();
     sql.append("`").append(it->effective_alias()).append("` INT" );
+    used_colnames.push_back(it->effective_alias());
     for (++it; it != select_statement->select_items.end(); ++it)
-      sql.append(", `").append(it->effective_alias()).append("` INT" );
+      if (std::find(used_colnames.begin(),used_colnames.end(),it->effective_alias()) == used_colnames.end())
+      {
+        sql.append(", `").append(it->effective_alias()).append("` INT" );
+        used_colnames.push_back(it->effective_alias());
+      }
   }
   sql.append(");\n");
 
@@ -1702,6 +1716,10 @@ static std::string get_export_sql(db_mysql_CatalogRef cat,
   std::string inserts_sql; // separate from main sql script & append to it as a last step,
                            // to separate creation of structures from data loading.
 
+    SqlFacade::Ref sql_facade = SqlFacade::instance_for_rdbms_name(grt, "Mysql");
+    Sql_specifics::Ref sql_specifics = sql_facade ->sqlSpecifics();
+    std::string non_std_sql_delimiter = sql_specifics->non_std_sql_delimiter();
+
   struct IntOption {
     static int get(grt::DictRef dict, const char *option, int defval= 0)
     {
@@ -1725,7 +1743,7 @@ static std::string get_export_sql(db_mysql_CatalogRef cat,
                                      grt::DictRef create_map, 
                                      grt::DictRef drop_map,
                                      grt::GRT *grt,
-									 std::string& inserts_sql)
+									                   std::string& inserts_sql)
     {
       std::string sql;
       const grt::ListRef<db_mysql_ForeignKey> fks= table->foreignKeys();
@@ -2017,7 +2035,7 @@ static std::string get_export_sql(db_mysql_CatalogRef cat,
       grt::ListRef<db_mysql_Trigger> triggers= table->triggers();
       if(table->triggers().count() > 0)
       {
-        objects_sql.append("\nDELIMITER $$\n");
+        objects_sql.append("\nDELIMITER ").append(non_std_sql_delimiter).append("\n");
 
         for(size_t c3= triggers.count(), k= 0; k < c3; k++)
         {
@@ -2037,13 +2055,13 @@ static std::string get_export_sql(db_mysql_CatalogRef cat,
           {
             std::string drop_trigger(string_from_map(trigger, drop_map));
             if(!drop_trigger.empty())
-              objects_sql.append("\n").append(drop_trigger).append("$$\n");
+              objects_sql.append("\n").append(drop_trigger).append(non_std_sql_delimiter).append("\n");
             if(show_warnings)
-              objects_sql.append("SHOW WARNINGS$$\n");
+              objects_sql.append("SHOW WARNINGS").append(non_std_sql_delimiter).append("\n");;
           }
-          objects_sql.append(string_from_map(trigger, create_map)).append("$$\n\n");
+          objects_sql.append(string_from_map(trigger, create_map)).append(non_std_sql_delimiter).append("\n\n");
           if(show_warnings)
-            objects_sql.append("SHOW WARNINGS$$\n");
+            objects_sql.append("SHOW WARNINGS").append(non_std_sql_delimiter).append("\n");
 
           if (grt) 
             grt->send_output(std::string("Processing Trigger ") 
@@ -2145,6 +2163,13 @@ int DbMySQLImpl::makeSQLSyncScript(const grt::DictRef& options,
   sql.append("SET @OLD_UNIQUE_CHECKS=@@UNIQUE_CHECKS, UNIQUE_CHECKS=0;\n");
   sql.append("SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0;\n");
   sql.append("SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='TRADITIONAL';\n\n");
+/*
+  SqlFacade::Ref sql_facade = SqlFacade::instance_for_rdbms_name(grt, "Mysql");
+  Sql_specifics::Ref sql_specifics = sql_facade ->sqlSpecifics();
+  std::string non_std_sql_delimiter = sql_specifics->non_std_sql_delimiter();
+  */
+  std::string non_std_sql_delimiter("$$");
+
 
   std::list<int> triggers_indices, views_indices, routines_indices;
 
@@ -2203,7 +2228,7 @@ int DbMySQLImpl::makeSQLSyncScript(const grt::DictRef& options,
   }
 
 //  if(!routines_indices.empty())
-//    sql.append("\nDELIMITER $$\n\n");
+//    sql.append("\nDELIMITER ").append(non_std_sql_delimiter).append("\n\n");
 
   for(std::list<int>::const_iterator e= routines_indices.end(), it= routines_indices.begin(); 
     it != e; it++)
@@ -2214,8 +2239,8 @@ int DbMySQLImpl::makeSQLSyncScript(const grt::DictRef& options,
   for(std::list<int>::const_iterator e= triggers_indices.end(), it= triggers_indices.begin(); 
     it != e; it++)
   {
-    sql.append("\nDELIMITER $$\n\n");
-    sql.append(sql_list.get(*it)).append("$$\n\n");
+    sql.append("\nDELIMITER ").append(non_std_sql_delimiter).append("\n\n");
+    sql.append(sql_list.get(*it)).append(non_std_sql_delimiter).append("\n\n");
     sql.append("\nDELIMITER ;\n\n");
   }
 
@@ -2246,10 +2271,18 @@ std::string DbMySQLImpl::makeAlterScriptForObject(GrtNamedObjectRef source, GrtN
   DiffChange* diff= diff_make(source, target, &omf, grt::NormalizedComparer<grt::GRT*>(get_grt()));
 
   std::string sql;
+  std::string non_std_sql_delimiter("$$");
+  if (ObjectRef::can_wrap(target))
+  {
+    SqlFacade::Ref sql_facade = SqlFacade::instance_for_rdbms_name(ObjectRef::cast_from(target)->get_grt(), "Mysql");
+    Sql_specifics::Ref sql_specifics = sql_facade ->sqlSpecifics();
+    non_std_sql_delimiter = sql_specifics->non_std_sql_delimiter();
+  }
+
 
   if (diff)
   {
-    ActionGenerateSQL* generator = new ActionGenerateSQL(result, grt::ListRef<GrtNamedObject>());
+    ActionGenerateSQL* generator = new ActionGenerateSQL(result, grt::ListRef<GrtNamedObject>(),get_grt());
     generator->set_put_if_exists(false);
     DiffSQLGeneratorBE *gen= new DiffSQLGeneratorBE(
       options, generator);
@@ -2321,8 +2354,8 @@ std::string DbMySQLImpl::makeAlterScriptForObject(GrtNamedObjectRef source, GrtN
                       std::string trigger_code = result.get_string(get_full_object_name_for_key(db_triggers.get(i)), "");
                       if (!trigger_code.empty())
                       {
-                        sql.append("USE ").append(db_table->owner()->name()).append(";\n").append("\nDELIMITER $$\n");
-                        sql.append(trigger_code).append("$$\nDELIMITER ;\n");
+                        sql.append("USE ").append(db_table->owner()->name()).append(";\n").append("\nDELIMITER ").append(non_std_sql_delimiter).append("\n\n");
+                        sql.append(trigger_code).append(non_std_sql_delimiter).append("\nDELIMITER ;\n");
                       }
                     }
                     
@@ -2337,10 +2370,10 @@ std::string DbMySQLImpl::makeAlterScriptForObject(GrtNamedObjectRef source, GrtN
         std::string trigger_code = result.get_string(get_full_object_name_for_key(triggers.get(i)), "");
         if (!trigger_code.empty())
         {
-          sql.append("USE ").append(table->owner()->name()).append(";\n").append("\nDELIMITER $$\n");
+          sql.append("USE ").append(table->owner()->name()).append(";\n").append("\nDELIMITER ").append(non_std_sql_delimiter).append("\n\n");
           sql.append("DROP TRIGGER IF EXISTS ").append(table->owner()->name()).append(".");
-          sql.append(triggers.get(i)->name()).append("$$\n");
-          sql.append(trigger_code).append("$$\nDELIMITER ;\n");
+          sql.append(triggers.get(i)->name()).append(non_std_sql_delimiter).append("\n");
+          sql.append(trigger_code).append(non_std_sql_delimiter).append("\nDELIMITER ;\n");
         }
       }
 
@@ -2351,7 +2384,6 @@ std::string DbMySQLImpl::makeAlterScriptForObject(GrtNamedObjectRef source, GrtN
         db_mysql_RoutineRef routine = db_mysql_RoutineRef::cast_from(obj); 
         if(routine->routineType().empty()) 
           routine->routineType("procedure"); 
-        //std::string routine_sql("\nDELIMITER $$\nUSE `"); 
         std::string routine_sql("USE `"); 
         routine_sql.append(routine->owner()->name()).append("`;\n"); 
         routine_sql.append("DROP ").append(routine->routineType().c_str()) 
@@ -2401,7 +2433,7 @@ std::string DbMySQLImpl::makeCreateScriptForObject(GrtNamedObjectRef object)
   if (diff)
   {
     DiffSQLGeneratorBE *gen= new DiffSQLGeneratorBE(
-      options, new ActionGenerateSQL(result, grt::ListRef<GrtNamedObject>()));
+      options, new ActionGenerateSQL(result, grt::ListRef<GrtNamedObject>(),get_grt()));
 
     gen->process_diff_change(ValueRef(), diff, result);
     delete diff;
@@ -2417,4 +2449,69 @@ grt::ListRef<db_mysql_StorageEngine> DbMySQLImpl::getKnownEngines()
   return dbmysql::get_known_engines(this->get_grt());
 }
 
+
+grt::ListRef<db_UserDatatype> DbMySQLImpl::getDefaultUserDatatypes(db_mgmt_RdbmsRef rdbms)
+{
+  static struct
+  {
+    const char *oid;
+    const char *name;
+    const char *sql_def;
+  }
+  type_init_data[] = {
+    // These are type aliases, not UDTs
+    {"com.mysql.rdbms.mysql.userdatatype.bool", "BOOL", "TINYINT(1)"},
+    {"com.mysql.rdbms.mysql.userdatatype.boolean", "BOOLEAN", "TINYINT(1)"},
+    {"com.mysql.rdbms.mysql.userdatatype.fixed", "FIXED", "DECIMAL(10,0)"},
+    {"com.mysql.rdbms.mysql.userdatatype.float4", "FLOAT4", "FLOAT"},
+    {"com.mysql.rdbms.mysql.userdatatype.float8", "FLOAT8", "DOUBLE"},
+    {"com.mysql.rdbms.mysql.userdatatype.int1", "INT1", "TINYINT(4)"},
+    {"com.mysql.rdbms.mysql.userdatatype.int2", "INT2", "SMALLINT(6)"},
+    {"com.mysql.rdbms.mysql.userdatatype.int3", "INT3", "MEDIUMINT(9)"},
+    {"com.mysql.rdbms.mysql.userdatatype.int4", "INT4", "INT(11)"},
+    {"com.mysql.rdbms.mysql.userdatatype.int8", "INT8", "BIGINT(20)"},
+    {"com.mysql.rdbms.mysql.userdatatype.integer", "INTEGER", "INT(11)"},
+    {"com.mysql.rdbms.mysql.userdatatype.longvarbinary", "LONG VARBINARY", "MEDIUMBLOB"},
+    {"com.mysql.rdbms.mysql.userdatatype.longvarchar", "LONG VARCHAR", "MEDIUMTEXT"},
+    {"com.mysql.rdbms.mysql.userdatatype.long", "LONG", "MEDIUMTEXT"},
+    {"com.mysql.rdbms.mysql.userdatatype.middleint", "MIDDLEINT", "MEDIUMINT(9)"},
+    {"com.mysql.rdbms.mysql.userdatatype.numeric", "NUMERIC", "DECIMAL(10,0)"},
+    {"com.mysql.rdbms.mysql.userdatatype.dec", "DEC", "DECIMAL(10,0)"},
+    {"com.mysql.rdbms.mysql.userdatatype.character", "CHARACTER", "CHAR(1)"}
+    // End type aliases
+  };
+  
+  grt::ListRef<db_UserDatatype> list(get_grt());
+  
+  for (size_t i= 0; i < sizeof(type_init_data)/sizeof(*type_init_data); i++)
+  {
+    std::string type= type_init_data[i].sql_def;
+    std::string::size_type paren= type.find('(');
+    if (paren != std::string::npos)
+      type= type.substr(0, paren);
+    
+    db_SimpleDatatypeRef simpletype(bec::CatalogHelper::get_datatype(rdbms->simpleDatatypes(), 
+                                                                     type));
+    
+    if (!simpletype.is_valid()) // unlikely
+    {
+      g_warning("Could not define built-in userdatatype <%s> %s (%s)", 
+                type_init_data[i].oid, type_init_data[i].name, type_init_data[i].sql_def);
+      continue;
+    }
+    db_UserDatatypeRef udata(get_grt());
+    
+    udata->__set_id(type_init_data[i].oid);
+
+    udata->name(type_init_data[i].name);
+    udata->sqlDefinition(type_init_data[i].sql_def);
+    udata->actualType(simpletype);
+    
+    list.insert(udata);
+  }
+  
+  return list;
+}
+
+
 GRT_MODULE_ENTRY_POINT(DbMySQLImpl);
