@@ -1,20 +1,20 @@
 /* 
- * (c) 2009-2010 Sun Microsystems, Inc.
+ * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
 #include "stdafx.h"
@@ -35,6 +35,41 @@ MySQLTableColumnsListBE::MySQLTableColumnsListBE(MySQLTableEditorBE *owner)
 {
 }
 
+bool MySQLTableColumnsListBE::set_field(const NodeId &node, int column, const std::string &value)
+{
+  db_mysql_ColumnRef col;
+
+  if (node.is_valid() && node[0] < real_count())
+  {
+    col= static_cast<MySQLTableEditorBE*>(_owner)->table()->columns().get(node[0]);
+    if (!col.is_valid())
+      return false;
+
+    db_SimpleDatatypeRef columnType;
+    switch ((MySQLColumnListColumns)column)
+    {
+    case Default:
+      // If a default value is set then auto increment for a column doesn't make sense.
+      if (!base::trim(value).empty())
+      {
+        AutoUndoEdit undo(_owner);
+
+        bool result = TableColumnsListBE::set_field(node, column, value);
+        col->autoIncrement(false);
+        undo.end(
+          strfmt(_("Set Default Value and Unset Auto Increment '%s.%s'"), _owner->get_name().c_str(),
+            col->name().c_str()
+          )
+        );
+
+        return result;
+      }
+
+      break;
+    }
+  }
+  return TableColumnsListBE::set_field(node, column, value);
+}
 
 bool MySQLTableColumnsListBE::set_field(const ::bec::NodeId &node, int column, int value)
 {
@@ -82,6 +117,10 @@ bool MySQLTableColumnsListBE::set_field(const ::bec::NodeId &node, int column, i
                     
           col->autoIncrement(value != 0);
 
+          // If auto increment is enabled then reset any default value.
+          if (col->autoIncrement() && !(*col->defaultValue()).empty())
+            bec::ColumnHelper::set_default_value(col, "");
+
           // if this is a primary key and auto-inc was set, then we should move this to the
           // beginning of the pk index 
           if (value && *_owner->get_table()->isPrimaryKeyColumn(col))
@@ -158,6 +197,18 @@ bool MySQLTableColumnsListBE::get_field_grt(const ::bec::NodeId &node, int colum
   return TableColumnsListBE::get_field_grt(node, column, value);
 }
 
+static bool can_be_timestamp(const char *value)
+{
+  if (*value == '\'')
+    return true;
+  
+  // accept anything that looks like a date
+  for (; *value; ++value)
+    if (!(isdigit(*value) || *value == ':' || *value == '-' || *value == '.' || *value == ' '))
+      return false;
+  return true;
+}
+
 
 bec::MenuItemList MySQLTableColumnsListBE::get_popup_items_for_nodes(const std::vector<bec::NodeId> &nodes)
 {
@@ -179,17 +230,30 @@ bec::MenuItemList MySQLTableColumnsListBE::get_popup_items_for_nodes(const std::
       
       if (type == "TIMESTAMP")
       {
-        bool is_first; // only the 1st TIMESTAMP column can have CURRENT_TIMESTAMP
+        bool seen_current_ts = false;
+        bool is_first_ts = false; // only the 1st TIMESTAMP column can have CURRENT_TIMESTAMP, unless all 
+        // previous ones are set to 0 or a constant
+        bool flag = false;
         
         GRTLIST_FOREACH(db_Column, columns, c)
         {
           if ((*c)->simpleType().is_valid() && (*c)->simpleType()->name() == "TIMESTAMP")
           {
-            is_first= *c == col;
-            break;
+            if (*c == col)
+            {
+              is_first_ts= !seen_current_ts;
+              flag = true;
+            }
+            
+            if (!((*c)->defaultValue() == "0" || can_be_timestamp((*c)->defaultValue().c_str())))
+              seen_current_ts = true;
+
+            // a column before some other TS column that is already marked as CURRENT_TIMESTAMP cannot become CURRENT_TS
+            if (*c != col && flag && strstr((*c)->defaultValue().c_str(), "TIMESTAMP"))
+              is_first_ts = false;
           }
         }
-        
+
         item.caption = "Default 0";
         item.name    = "TSToolStripMenuItem";
         item.enabled = true;
@@ -197,12 +261,12 @@ bec::MenuItemList MySQLTableColumnsListBE::get_popup_items_for_nodes(const std::
 
         item.caption = "Default CURRENT_TIMESTAMP";
         item.name    = "currentTSToolStripMenuItem";
-        item.enabled = is_first;
+        item.enabled = is_first_ts;
         items.push_back(item);
         
         item.caption = "Default CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP";
         item.name    = "currentTSOnUpdateToolStripMenuItem";
-        item.enabled = is_first;
+        item.enabled = is_first_ts;
         items.push_back(item);        
       }
       else if (col->simpleType()->group()->name() == "numeric" ||
@@ -502,11 +566,12 @@ std::string MySQLTableEditorBE::get_all_triggers_sql() const
       unordered_triggers.push_back(trigger);
   }
 
+  //XXX fix parser so that it skips newlines before the trigger
   for (OrderedTriggers::iterator i= ordered_triggers.begin(), i_end= ordered_triggers.end(); i != i_end; ++i)
-    retval.append(i->second->sqlDefinition()).append(_non_std_sql_delimiter).append("\n\n");
+    retval.append(base::strip_text(i->second->sqlDefinition(), true, false)).append(_non_std_sql_delimiter).append("\n\n");
 
   for (UnorderedTriggers::iterator i= unordered_triggers.begin(), i_end= unordered_triggers.end(); i != i_end; ++i)
-    retval.append((*i)->sqlDefinition()).append(_non_std_sql_delimiter).append("\n\n");
+    retval.append(base::strip_text((*i)->sqlDefinition(), true, false)).append(_non_std_sql_delimiter).append("\n\n");
 
   return retval;
 }
