@@ -1,20 +1,20 @@
 /* 
- * (c) 2009-2010 Sun Microsystems, Inc.
+ * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
 #include "stdafx.h"
@@ -490,11 +490,11 @@ DiffTreeBE *DbMySQLScriptSync::init_diff_tree(const std::vector<std::string>& sc
   {
     StringRef schema_name = SchemaSkipList.get(i);
     for(size_t schema_count = 0; schema_count < _org_cat->schemata().count(); schema_count++)
-      if (_org_cat->schemata().get(schema_count)->name() == schema_name)
+      while((schema_count < _org_cat->schemata().count())&& (_org_cat->schemata().get(schema_count)->name() == schema_name))
         _org_cat->schemata().remove(schema_count);
 
     for(size_t schema_count = 0; schema_count < _mod_cat_copy->schemata().count(); schema_count++)
-      if (_mod_cat_copy->schemata().get(schema_count)->name() == schema_name)
+      while((schema_count < _mod_cat_copy->schemata().count())&& (_mod_cat_copy->schemata().get(schema_count)->name() == schema_name))
         _mod_cat_copy->schemata().remove(schema_count);
   }
 
