@@ -1,20 +1,20 @@
 /* 
- * (c) 2009-2010 Sun Microsystems, Inc.
+ * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
 
 #include "stdafx.h"
@@ -43,7 +43,7 @@ ImportInputPage::ImportInputPage(WizardPlugin *form)
   
   _heading.set_style(WizardHeadingStyle);
   _heading.set_text(_("Select the script containing the schemata to reverse engineer"));
-  _table.add(&_heading, 0, 2, 0, 1, 0);
+  _table.add(&_heading, 0, 2, 0, 1, mforms::HFillFlag);
 
   _caption.set_text_align(mforms::WizardLabelAlignment);
   _caption.set_text(_("Select SQL script file:"));
@@ -52,7 +52,7 @@ ImportInputPage::ImportInputPage(WizardPlugin *form)
 
   std::string initial_filename= form->module()->document_string_data("input_filename", "");
   _file_selector.initialize(initial_filename, mforms::OpenFile, "SQL Files (*.sql)|*.sql", _("Browse..."),
-                            sigc::mem_fun(this, &WizardPage::validate));
+    false, sigc::mem_fun(this, &WizardPage::validate));
   _file_selector.signal_changed().connect(sigc::mem_fun(this, &ImportInputPage::file_changed));
 
   _file_codeset_caption.set_text(_("File encoding:"));
@@ -64,7 +64,7 @@ ImportInputPage::ImportInputPage(WizardPlugin *form)
   // Fill the encoding selector with a useful list of encodings.
   fill_encodings_list();
 
-  _table.add(&_autoplace_check, 1, 2, 3, 4, 0);
+  _table.add(&_autoplace_check, 1, 2, 3, 4, mforms::HExpandFlag | mforms::HFillFlag);
   _autoplace_check.set_text(_("Place imported objects on a diagram"));
   _autoplace_check.set_active(true);
 
