@@ -51,7 +51,7 @@ def wrap_for_sudo(command, sudo_prefix):
 
 class SSH(WbAdminSSH):
   def __init__(self, profile, password_delegate):
-    self.mtx = threading.RLock()
+    self.mtx = threading.Lock()
     self.wrapped_connect(profile, password_delegate)
 
   def __del__(self):
