@@ -98,16 +98,34 @@ class WbAdminConfiguration(mforms.Box):
           panel.add(bottom_box, False, False)
 
           accept_btn = newButton()
-          accept_btn.set_text("Apply...")
+          accept_btn.set_text("Apply ...")
           #accept_btn.set_size(80, -1)
 
           discard_btn = newButton()
           discard_btn.set_text("Discard")
           #discard_btn.set_size(80, -1)
 
+          search_label = newLabel("Locate option:")
+          self.option_lookup_entry  = newTextEntry()
+          search_btn = newButton()
+          search_btn.set_text("Find ...")
+          search_btn.set_size(60, -1)
+
+          search_box = newBox(True)
+          search_box.set_padding(2)
+          search_box.set_spacing(4)
+          search_box.set_size(300, -1)
+          search_box.add(search_label, False, False)
+          search_box.add(self.option_lookup_entry, True, True)
+          search_box.add(search_btn, False, False)
+          search_panel = newPanel(mforms.BorderedPanel)
+          search_panel.add(search_box)
+          search_btn.add_clicked_callback(self.lookup_option)
+
           bottom_box.add(newLabel("Configuration File:"), False, True)
           bottom_box.add(file_name, True, True)
           bottom_box.add(section, False, True)
+          bottom_box.add(search_panel, False, True)
 
           Utilities.add_end_ok_cancel_buttons(bottom_box, accept_btn, discard_btn)
 
@@ -190,6 +208,10 @@ class WbAdminConfiguration(mforms.Box):
 
      #   return self.connection_error
 
+    # To scroll to option
+    def lookup_option(self):
+      self.config_ui.locate_option(self.option_lookup_entry.get_string_value())
+
 
     def switch_to(self, tab_no):
       self.switcher.set_selected(tab_no)
@@ -246,16 +268,14 @@ class WbAdminConfiguration(mforms.Box):
       last_server_status = None
       only_status_check = 0
       while not self.closing:
-        if self.ctrl_be.server_control:
-          try:
-            status = self.ctrl_be.is_server_running(verbose=0)
-          except Exception, exc:
-            import traceback
-            traceback.print_exc()
-            print "exception getting server status: %s" % exc
-        else:
-          # server admin is disabled
-          status = "unknown"
+        status = "unknown"
+
+        try:
+          status = self.ctrl_be.is_server_running(verbose=0)
+        except Exception, exc:
+          import traceback
+          traceback.print_exc()
+          print "exception getting server status: %s" % exc
 
         control_event = None
         if last_server_status != status:
