@@ -100,6 +100,8 @@ class DumpThread(threading.Thread):
                 try:
                     p1 = subprocess.Popen(logstr,stdout=respipe,stdin=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
                 except OSError, exc:
+                    import traceback
+                    traceback.print_exc()
                     self.print_log_message("Error executing task: %s" % str(exc))
 
                 pwdfile = open(pwdfilename, 'w')
@@ -116,6 +118,8 @@ class DumpThread(threading.Thread):
                     info.wShowWindow = subprocess.SW_HIDE
                     p1 = subprocess.Popen(logstr,stdout=respipe or subprocess.PIPE,stdin=subprocess.PIPE, stderr=subprocess.PIPE,startupinfo=info,shell=logstr[0] != '"')
                 except OSError, exc:
+                    import traceback
+                    traceback.print_exc()
                     self.print_log_message("Error executing task: %s" % str(exc))
 #                finally:
 #                    pass
@@ -131,6 +135,8 @@ class DumpThread(threading.Thread):
 
 
         except Exception, exc:
+            import traceback
+            traceback.print_exc()
             self.print_log_message("Error executing task: %s" % str(exc))
 #        finally:
 #            pass
@@ -164,9 +170,10 @@ class DumpThread(threading.Thread):
                     self.print_log_message("kill task: %s" % str(exc))
 
     def print_log_message(self,message):
-        self.logging_lock.acquire()
-        self.log.append(str(message))
-        self.logging_lock.release()  
+        if message:
+            self.logging_lock.acquire()
+            self.log.append(message)
+            self.logging_lock.release()  
 
     def run(self):
         try:
@@ -202,6 +209,8 @@ class DumpThread(threading.Thread):
 #                import time
 #                time.sleep(1)
         except Exception, exc:
+            import traceback
+            traceback.print_exc()
             self.print_log_message("Error executing task %s" % str(exc) )
 #        finally:
         if not self.abort_requested:
@@ -209,13 +218,15 @@ class DumpThread(threading.Thread):
         self.done = True
 
 class TableListModel:
-    tables_by_schema = {}
+    def __init__(self):
+        self.tables_by_schema = {}
 
     def get_full_selection(self):
         result = []
         for schema, (tables, selection) in self.tables_by_schema.items():
             for table in selection:
                 result.append((schema,table))
+        result.sort()
         return result
 
     def get_schema_names(self):
@@ -241,14 +252,14 @@ class TableListModel:
     def get_count(self):
         return sum([len(item[1]) for item in self.tables_by_schema.values()])
 
-    def get_objects_to_dump(self):
+    def get_objects_to_dump(self, include_empty_schemas=False):
         schemas_to_dump = []
         names = self.tables_by_schema.keys()
         names.sort()
         for schema in names:
         #No tables selected for schema so skip it
             tables, selection = self.tables_by_schema[schema]
-            if not selection:
+            if not selection and not include_empty_schemas:
                 continue
             schemas_to_dump.append((schema, list(selection)))
         return schemas_to_dump
@@ -257,9 +268,10 @@ class TableListModel:
 
 
 class WbAdminSchemaListTab(mforms.Box):
-    savefile_path = None
-    savefolder_path = None
     def __init__(self, owner, instance_info, is_importing = False):
+        self.savefile_path = None
+        self.savefolder_path = None
+
         mforms.Box.__init__(self, False)
         
         self.logging_lock = thread.allocate_lock()
@@ -284,11 +296,13 @@ class WbAdminSchemaListTab(mforms.Box):
 
         self.schema_list.set_cell_edited_callback(self.schema_list_edit)
         self.schema_list.end_columns()
+        self.schema_list.set_allow_sorting(True)
 
         self.table_list = newTreeView(mforms.TreeShowHeader)
         self.table_list.add_column(mforms.CheckColumnType, is_importing and "Import" or "Export", 50, True)
         self.table_list.add_column(mforms.StringColumnType, "Tables in Schema", 300, False)
         self.table_list.end_columns()
+        self.table_list.set_allow_sorting(True)
 
         self.table_list.set_cell_edited_callback(self.table_list_edit)
         
@@ -327,7 +341,7 @@ class WbAdminSchemaListTab(mforms.Box):
           self.single_transaction_check = newCheckBox()
           self.dump_view_check = newCheckBox()
           self.dump_routines_check = newCheckBox()
-
+          
         self.filelabel.set_enabled(False)
         self.filelabel.set_style(mforms.SmallStyle)
 
@@ -392,7 +406,7 @@ class WbAdminSchemaListTab(mforms.Box):
             suboptionsbox.add(self.dump_view_check, False, True)
         if self.dump_routines_check:
             suboptionsbox.add(self.dump_routines_check, False, True)
-
+            
         if os.getenv("WB_DUMP_TEST_BUTTON"):
             self.test_button = newButton()
             self.test_button.set_text("Test")
@@ -504,6 +518,7 @@ class WbAdminSchemaListTab(mforms.Box):
           self.single_transaction_check.add_clicked_callback(self.single_transaction_clicked)
           self.dump_view_check.set_text("Dump Views")
           self.dump_routines_check.set_text("Dump Stored Routines (Procedures and Functions)")
+          
           self.folderradio.set_text("Export to Dump Project Folder")
           self.export_button.set_text("Start Export")
 
@@ -553,11 +568,12 @@ class WbAdminSchemaListTab(mforms.Box):
         schema = self.get_selected_schema()
         tables = self.table_list_model.get_tables(schema)
         selection = self.table_list_model.get_selection(schema)
-        for r in range(len(tables)):
+        for r in range(self.table_list.count()):
+            table_name = self.table_list.get_row_tag(r)
             if self.table_list.get_check(r, 0):
-                selection.add(tables[r])
+                selection.add(table_name)
             else:
-                selection.discard(tables[r])
+                selection.discard(table_name)
         self.statlabel.set_text("%i tables selected" % self.table_list_model.count_selected_tables())
 
     def get_selected_schema(self):
@@ -568,10 +584,12 @@ class WbAdminSchemaListTab(mforms.Box):
 
     def schema_selected(self):
         sel = self.schema_list.get_selected()
+        self.table_list.freeze_refresh()
         self.table_list.clear_rows()
         if sel < 0:
             self.unselect_all_btn.set_enabled(False)
             self.select_all_btn.set_enabled(False)
+            self.table_list.thaw_refresh()
             return
         schema = self.get_selected_schema()
         tables = self.table_list_model.get_tables(schema)
@@ -580,6 +598,8 @@ class WbAdminSchemaListTab(mforms.Box):
             r = self.table_list.add_row()
             self.table_list.set_bool(r, 0, table in selection)
             self.table_list.set_string(r, 1, table)
+            self.table_list.set_row_tag(r, table)
+        self.table_list.thaw_refresh()
         self.unselect_all_btn.set_enabled(True)
         self.select_all_btn.set_enabled(True)
         
@@ -637,7 +657,9 @@ class WbAdminSchemaListTab(mforms.Box):
             progress = self.dump_thread.progress
             self.dump_thread.logging_lock.acquire()
             while len(self.dump_thread.log) > 0:
-                self.progress_log.append_text_and_scroll(self.dump_thread.log.popleft()+"\n", True)
+                text = self.dump_thread.log.popleft()+"\n"
+                print text
+                self.progress_log.append_text_and_scroll(text, True)
 #                print self.dump_thread.log.popleft()
 #                print progress
             self.dump_thread.logging_lock.release() 
@@ -730,7 +752,8 @@ class WbAdminImportTab(WbAdminSchemaListTab):
 
     def refresh_table_list(self):
         def parse_name_from_single_table_dump(path):
-            f = open(path)
+            import codecs
+            f = codecs.open(path, encoding="utf-8")
             schema = None
             table = None
             is_view = False
@@ -757,6 +780,7 @@ class WbAdminImportTab(WbAdminSchemaListTab):
         # (schema, table) -> path
         self.tables_paths = {}
         self.views_paths = {}
+        self.schema_list.freeze_refresh()
         self.schema_list.clear_rows()
         try:
             save_to_folder = not self.fileradio.get_active()
@@ -795,8 +819,10 @@ class WbAdminImportTab(WbAdminSchemaListTab):
                         self.schema_list.set_string(row, 1, schema)
                 
                     self.export_button.set_enabled(True)
+            self.schema_list.thaw_refresh()
         except Exception, exc:
             import traceback
+            self.schema_list.thaw_refresh()
             traceback.print_exc()
             Utilities.show_error("Error Opening Dump", str(exc), "OK", "", "")
             self.failed(str(exc))
@@ -848,7 +874,8 @@ class WbAdminImportTab(WbAdminSchemaListTab):
                 # description, object_count, extra_args, objects, pipe_factory
                 if path != None: 
                     task = DumpThread.TaskData(logmsg, 1, [], [path], lambda:None)
-                    operations.insert(0,task) 
+                    #operations.insert(0,task) 
+                    operations.append(task)
                 else: 
                     path = self.views_paths.get((schema, table)) 
                     task = DumpThread.TaskData(logmsg, 1, [], [path], lambda:None)
@@ -856,8 +883,8 @@ class WbAdminImportTab(WbAdminSchemaListTab):
                         operations.append(task) 
         else:
             if not os.path.exists(self.path):
-                Utilities.show_message("Dump file not found", "File " + str(self.path) + " doesn't exist", "OK", "", "")
-                self.failed("Dump file not found: File " + str(self.path) + " doesn't exist")
+                Utilities.show_message("Dump file not found", u"File %s doesn't exist" % self.path, "OK", "", "")
+                self.failed(u"Dump file not found: File %s doesn't exist" % self.path)
                 return
             logmsg = "Restoring " + self.path
             # description, object_count, pipe_factory, extra_args, objects
@@ -927,7 +954,7 @@ class WbAdminImportTab(WbAdminSchemaListTab):
 
 
     def tasks_completed(self):
-        logmsg = time.strftime('%X ') + "Import of %s has finished" % str(self.path)
+        logmsg = time.strftime('%X ') + u"Import of %s has finished" % self.path
         if self.dump_thread.error_count > 0:
             self.hintlabel.set_text("Import Completed With %i Errors" % self.dump_thread.error_count)
             logmsg += " with %i errors" % self.dump_thread.error_count
@@ -981,9 +1008,9 @@ class WbAdminImportTab(WbAdminSchemaListTab):
 
     def check_result(self):
         def check_val(result,name,expected_val):
-            resval = result.stringByName(name)
-            expected_val = str(expected_val)
-            return (str(resval) == str(expected_val) and [None] or ["Wrong data got %s for %s instead of %s"%(resval,name,expected_val)])[0]
+            resval = result.unicodeByName(name)
+            expected_val = unicode(expected_val)
+            return (resval == expected_val and [None] or ["Wrong data got %s for %s instead of %s"%(resval,name,expected_val)])[0]
 
         db_connection = self.owner.ctrl_be.sql
         try:
@@ -1057,6 +1084,7 @@ class WbAdminExportTab(WbAdminSchemaListTab):
 
     class ExportTableListModel(TableListModel):
         def __init__(self):
+            TableListModel.__init__(self)
             self.views_by_schema = {}
             self.schemasqls = {}
             self.schemas_to_load = {}
@@ -1132,31 +1160,31 @@ class WbAdminExportTab(WbAdminSchemaListTab):
         self.refresh_canceled = True
 
     def load_schema_tables(self,schema):
-            schematables = []
-            viewlist = []
-            dbsql = ""
-            try:
-                self.refresh_state= "Retriving tables data for schema " + schema
-                dbcreate = self.owner.ctrl_be.exec_query("SHOW CREATE DATABASE `"+escape_sql_identifier(schema)+"`")
-                tableset = self.owner.ctrl_be.exec_query("SHOW FULL TABLES FROM `"+escape_sql_identifier(schema)+"`")
-                dbcreate.nextRow()
-                dbsql = dbcreate.stringByName("Create Database")
-                parts = dbsql.partition("CREATE DATABASE ")
-                dbsql = parts[0] + parts[1] + " IF NOT EXISTS " + parts[2] + ";\nUSE `" + escape_sql_identifier(schema) + "`;\n"
-                while tableset.nextRow():
-                    tabletype = tableset.stringByName("Table_type")
-                    tablename = tableset.stringByName("Tables_in_"+schema)
-                    if type(tablename) is unicode:
-                        tablename = tablename.encode("utf8")
-                    if tabletype == "VIEW":
-                        viewlist.append(tablename)
-                        continue
-                    schematables.append(tablename)
-                del tableset
-            except Exception, exc:
-                print "Error retriving table list form schema '",schema,"'"
-                self.print_log_message("Error Fetching Table List From %s (%s)" % (schema, str(exc)) )
-            return schema,schematables,viewlist,dbsql
+        schematables = []
+        viewlist = []
+        dbsql = ""
+        try:
+            self.refresh_state= "Retriving tables data for schema " + schema
+            dbcreate = self.owner.ctrl_be.exec_query("SHOW CREATE DATABASE `"+escape_sql_identifier(schema)+"`")
+            tableset = self.owner.ctrl_be.exec_query("SHOW FULL TABLES FROM `"+escape_sql_identifier(schema)+"`")
+            dbcreate.nextRow()
+            dbsql = dbcreate.unicodeByName("Create Database")
+            parts = dbsql.partition("CREATE DATABASE ")
+            dbsql = u"%s%s IF NOT EXISTS %s;\nUSE `%s`;\n" % (parts[0], parts[1], parts[2], escape_sql_identifier(schema))
+            while tableset.nextRow():
+                tabletype = tableset.unicodeByName("Table_type")
+                tablename = tableset.unicodeByName("Tables_in_"+schema)
+                if tabletype == "VIEW":
+                    viewlist.append(tablename)
+                    continue
+                schematables.append(tablename)
+            del tableset
+        except Exception, exc:
+            import traceback
+            traceback.print_exc()
+            print "Error retriving table list form schema '",schema,"'"
+            self.print_log_message("Error Fetching Table List From %s (%s)" % (schema, str(exc)) )
+        return schema,schematables,viewlist,dbsql
 
 
     def refresh_table_list_thread(self):
@@ -1165,9 +1193,7 @@ class WbAdminExportTab(WbAdminSchemaListTab):
             result = self.owner.ctrl_be.exec_query("SHOW DATABASES")
             schema_names = []
             while result.nextRow():
-                value = result.stringByName("Database")
-                if type(value) is unicode:
-                    value = value.encode("utf8")
+                value = result.unicodeByName("Database")
                 if value == "information_schema":
                     continue
                 schema_names.append(value)            
@@ -1223,10 +1249,12 @@ class WbAdminExportTab(WbAdminSchemaListTab):
         self.dump_progressbar.set_value(0)
         names = self.table_list_model.get_schema_names()
         names.sort()
+        self.schema_list.freeze_refresh()
         for schema in names:
             r = self.schema_list.add_row()
             self.schema_list.set_string(r, 1, schema)
             self.schema_list.set_bool(r, 0, 0)
+        self.schema_list.thaw_refresh()
         self.refresh_button.set_enabled(True)
         self.statlabel.set_text("")
         self.hintlabel.set_text("Press [Start Export] to start...")
@@ -1294,7 +1322,7 @@ class WbAdminExportTab(WbAdminSchemaListTab):
                 version = s.groups()[0]
                 major, minor = [int(i) for i in version.split(".")[:2]]
                 if major < server_major or (major == server_major and minor < server_minor):
-                    msg = "%s is version %s, but the MySQL Server to be dumped has version %s.\nBecause the version of mysqldump is older than the server, some features may not be backed up properly.\nIt is recommended you upgrade your local MySQL client programs, including mysqldump"%(path, version, server_version)
+                    msg = "%s is version %s, but the MySQL Server to be dumped has version %s.\nBecause the version of mysqldump is older than the server, some features may not be backed up properly.\nIt is recommended you upgrade your local MySQL client programs, including mysqldump to a version equal to or newer than that of the target server.\nThe path to the dump tool must then be set in Preferences -> Administrator -> Path to mysqldump Tool:"%(path, version, server_version)
         if msg:
           if about_to_run:
               if error:
@@ -1361,9 +1389,17 @@ class WbAdminExportTab(WbAdminSchemaListTab):
             DumpThread.TaskData.__init__(self,title, 1, ["--skip-triggers" ,"--routines" ," --no-create-info" ," --no-data" ," --no-create-db"], [schema], make_pipe)
 
     def dump_to_folder(self, schemaname, tablename):
-        self.close_pipe()        
-        self.out_pipe = open(os.path.join(self.path, normalize_filename(schemaname) + "_" + normalize_filename(tablename) + '.sql'),"w")
-        self.out_pipe.write(self.table_list_model.get_schema_sql(schemaname))
+        self.close_pipe() 
+        path = os.path.join(self.path, normalize_filename(schemaname) + "_" + normalize_filename(tablename) + '.sql')
+        i = 0
+        # check if the path already exists (they could become duplicated because of normalization)
+        while os.path.exists(path):
+            path = os.path.join(self.path, normalize_filename(schemaname) + "_" + normalize_filename(tablename) + ('%i.sql'%i))
+        self.out_pipe = open(path,"w")
+        data = self.table_list_model.get_schema_sql(schemaname)
+        if type(data) is unicode:
+            data = data.encode("utf-8")
+        self.out_pipe.write(data)
         self.out_pipe.flush()
         return self.out_pipe
 
@@ -1479,7 +1515,6 @@ class WbAdminExportTab(WbAdminSchemaListTab):
                 del params["port"]
             if not params["host"]:
                 del params["host"]
-
         params.update(self.owner.get_export_options())
         cmd = self.get_path_to_mysqldump()
         if cmd == None:
@@ -1496,6 +1531,7 @@ class WbAdminExportTab(WbAdminSchemaListTab):
             return
         self.stop_button.set_enabled(True)
         self.hintlabel.set_text("Export is running ...")
+
         self.dump_thread = DumpThread(cmd, operations, password, self)
         self.dump_thread.is_import = False
         self.dump_thread.start()
@@ -1534,7 +1570,7 @@ class WbAdminExportTab(WbAdminSchemaListTab):
         self.update_paths()
         self.file_te.set_value(self.savefile_path)
         self.folder_te.set_value(self.savefolder_path)
-        logmsg = time.strftime('%X ') + "Export of %s has finished" % str(self.path)
+        logmsg = time.strftime('%X ') + u"Export of %s has finished" % self.path
         if self.dump_thread.error_count > 0:
             self.hintlabel.set_text("Export Completed With %i Errors" % self.dump_thread.error_count)
             logmsg += " with %i errors" % self.dump_thread.error_count
