@@ -12,7 +12,7 @@ import mforms
 
 from wb_admin_utils import not_running_warning_label
 from wb_admin_security_be import AdminSecurity, PrivilegeInfo, SecurityAdminRoles, WBSecurityValidationError
-
+from wb_common import PermissionDeniedError
 
 
 SCHEMA_OBJECT_RIGHTS = [
@@ -263,6 +263,7 @@ class SecurityAccount(mforms.Box):
         self.user_list.add_column(mforms.StringColumnType, "From Host", 120, False)
         self.user_list.end_columns()
         self.user_list.add_changed_callback(self.user_selected)
+        self.user_list.set_allow_sorting(True)
         account_list_box.add(self.user_list, True, True)
 
 
@@ -292,6 +293,7 @@ class SecurityAccount(mforms.Box):
         self.username = newTextEntry()
         self.username.set_size(150, -1)
         self.username.add_changed_callback(self.set_dirty)
+        self.username.set_max_length(16) # max username length for mysql 
         self.password = newTextEntry(mforms.PasswordEntry)
         self.password.set_size(150, -1)
         self.password.add_changed_callback(self.set_dirty)
@@ -353,6 +355,7 @@ class SecurityAccount(mforms.Box):
         self.role_priv_list.add_column(mforms.StringColumnType, "Global Privileges Assigned to User", 200, False)
         self.role_priv_list.end_columns()
         self.role_priv_list.set_size(220, -1)
+        self.role_priv_list.set_allow_sorting(True)
         lbox.add(self.role_priv_list, False, True)
 
         self.role_list.clear_rows()
@@ -539,6 +542,7 @@ class SecurityAccount(mforms.Box):
         
         su = self._selected_user
         suo = self._selected_user_original
+        self.user_list.freeze_refresh()
         self.user_list.clear_rows()
         self._selected_user = su
         self._selected_user_original = suo
@@ -553,9 +557,10 @@ class SecurityAccount(mforms.Box):
         if selected_row is not None:
             self.user_list.set_selected(selected_row)
             self.user_selected()
-
+        self.user_list.thaw_refresh()
 
     def refresh_priv_list(self):
+        self.role_priv_list.freeze_refresh()
         self.role_priv_list.clear_rows()
         if self._selected_user:
             privs = self._selected_user.raw_privilege_names
@@ -563,7 +568,7 @@ class SecurityAccount(mforms.Box):
             for priv in privs:
                 row = self.role_priv_list.add_row()
                 self.role_priv_list.set_string(row, 0, priv)
-
+        self.role_priv_list.thaw_refresh()
 
     def set_dirty(self):
         self.save_button.set_enabled(True)
@@ -625,6 +630,11 @@ class SecurityAccount(mforms.Box):
                 Utilities.show_error("Save Account Changes",
                       str(exc), "OK", "", "")
                 return
+            except PermissionDeniedError, exc:
+                Utilities.show_error("Save Account Changes",
+                      str(exc), "OK", "", "")
+                return
+            
             self._selected_user.load(self._selected_user.username, self._selected_user.host)
             self.owner.refresh()
 
@@ -821,6 +831,8 @@ class SecuritySchemaPrivileges(mforms.Box):
 
 
     def add_entry(self):
+        if not self._selected_user:
+            return
         addform = AddSchemaPrivilegeForm(self.owner.secman, self._selected_user.username)
 
         host, schema = addform.run()
@@ -985,8 +997,13 @@ class SecuritySchemaPrivileges(mforms.Box):
 
 
     def commit(self):
-        if self._selected_user:        
-            self._selected_user.save()
+        if self._selected_user:       
+            try:
+                self._selected_user.save()
+            except PermissionDeniedError, exc:
+                Utilities.show_error("Save Account Changes",
+                      str(exc), "OK", "", "")
+                return
             self.unset_dirty()
 
 
@@ -1045,7 +1062,10 @@ class WbAdminSecurity(mforms.Box):
       
     def refresh(self):
         if self.ctrl_be.is_sql_connected():
-            self.secman.async_refresh(self.do_refresh)
+            try:
+                self.secman.async_refresh(self.do_refresh)
+            except PermissionDeniedError, e:
+                Utilities.show_error("Access Denied", str(e), "OK", "", "")
 
 
 
