@@ -30,7 +30,7 @@ for sec in doc.documentElement.getElementsByTagName('section'):
 
             content = options_struct.struct[tab_name][grp_name]
             for opt_name in content.keys():
-              opt = opts[opt_name]
+              opt = opts.get(opt_name, None)
               grp_cont.append(opt)
               del olist[opt_name]
 
@@ -47,7 +47,7 @@ mo = olist.keys()
 mo.sort()
 for o in mo:
   v = olist[o]
-  print o, v['description']
+  print o#, v['description']
   print "--"
 
 for o in mo:
