@@ -114,13 +114,12 @@ struct = { 'Advanced': { 'General': { 'default_week_format': 'server',
                'Networking': { 'enable-named-pipe': 'server',
                                'port': 'server',
                                'skip-networking': 'security'}},
-  'InnoDB Parameters': { 'Activate InnoDB': { 'innodb': 'innodb',
-                                              'skip-innodb' : 'innodb'
+  'InnoDB Parameters': { 'Activate InnoDB': { 'innodb': 'innodb'
                                             },
                          'Datafiles': { 'innodb_data_file_path': 'innodb',
                                         'innodb_data_home_dir': 'innodb',
                                         'innodb_file_per_table': 'innodb'},
-                         'General': { 'ignore_builtin_innodb': 'server',
+                         'General': { 'ignore-builtin-innodb': 'server',
                                       'innodb_adaptive_flushing': 'innodb',
                                       'innodb_adaptive_hash_index': 'innodb',
                                       'innodb_autoextend_increment': 'innodb',
@@ -259,7 +258,7 @@ struct = { 'Advanced': { 'General': { 'default_week_format': 'server',
                                    'ndb_report_thresh_binlog_epoch_slip': 'ndb',
                                    'ndb_report_thresh_binlog_mem_usage': 'ndb',
                                    'ndb_use_transactions': 'ndb',
-                                   'ndb_wait_connected': 'ndb',
+                                   'ndb-wait-connected': 'ndb',
                                    'ndbcluster': 'ndb',
                                    'skip-ndbcluster': 'ndb'}},
   'Networking': { 'Advanced': { 'max_connect_errors': 'server',
@@ -312,7 +311,7 @@ struct = { 'Advanced': { 'General': { 'default_week_format': 'server',
                                       'disconnect-slave-event-count': 'replication',
                                       'log-slave-updates': 'replication',
                                       'skip-slave-start': 'replication',
-                                      'slave-allow-batching': 'ndb',
+                                      'slave_allow_batching': 'replication',
                                       'slave-load-tmpdir': 'replication',
                                       'slave-net-timeout': 'replication',
                                       'slave-skip-errors': 'replication',
